Package: OmicFlow
Title: Fast and Efficient (Automated) Analysis of Sparse Omics Data
Version: 1.6.0
Date: 2026-05-19
Authors@R: c(
    person(
        "Alem", "Gusinac",
        email = "alem.gusinac@gmail.com", 
        role = c("aut", "cre"),
        comment = c(ORCID = "0009-0006-1896-4176")),
    person(
        "Thomas", "Ederveen", 
        role = "aut",
        comment = c(ORCID = "0000-0003-0068-1275")),
    person(
        "Annemarie", "Boleij", 
        role = c("aut", "fnd"),
        comment = c(ORCID = "0000-0003-4495-5880"))
  )
Description: 
    A generalised data structure for fast and efficient loading and data munching of sparse omics data. The 'OmicFlow' requires an up-front validated metadata template from the user,
    which serves as a guide to connect all the pieces together by aligning them into a single object that is defined as an 'omics' class. 
    Once this unified structure is established, users can perform manual subsetting, visualisation, and statistical analysis, or leverage the automated 'autoFlow' method to generate a comprehensive report.
License: MIT + file LICENSE
URL: https://github.com/agusinac/OmicFlow
BugReports: https://github.com/agusinac/OmicFlow/issues
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.3.0), R6, data.table, Matrix
Imports: ape, ggpubr, ggrepel, ggplot2, jsonlite, jsonvalidate,
        magrittr, methods, patchwork, RColorBrewer, rhdf5, rstatix,
        Rcpp (>= 0.12.6), RcppParallel (>= 4.3.20), stats, tools,
        utils, vegan, yyjsonr, cli
Suggests: DT, downloadthis, rmarkdown, testthat (>= 3.0.0)
LinkingTo: Rcpp, RcppParallel, RcppArmadillo
NeedsCompilation: yes
Packaged: 2026-05-19 14:12:25 UTC; Z289224
Author: Alem Gusinac [aut, cre] (ORCID:
    <https://orcid.org/0009-0006-1896-4176>),
  Thomas Ederveen [aut] (ORCID: <https://orcid.org/0000-0003-0068-1275>),
  Annemarie Boleij [aut, fnd] (ORCID:
    <https://orcid.org/0000-0003-4495-5880>)
Maintainer: Alem Gusinac <alem.gusinac@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-19 14:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 16:51:28 UTC; unix
Archs: OmicFlow.so.dSYM

# Generated by roxygen2: do not edit by hand

export(bray)
export(canberra)
export(colormap)
export(column_exists)
export(composition_plot)
export(cosine)
export(diversity)
export(diversity_plot)
export(euclidean)
export(hill_taxa)
export(jaccard)
export(jsd)
export(manhattan)
export(matrix_to_dtable)
export(metagenomics)
export(omics)
export(ordination_plot)
export(pairwise_adonis)
export(pairwise_anosim)
export(plot_pairwise_stats)
export(proteomics)
export(unifrac)
export(volcano_plot)
import(Matrix)
import(R6)
import(rhdf5)
importFrom(Matrix,sparseMatrix)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rcpp,sourceCpp)
importFrom(RcppParallel,setThreadOptions)
importFrom(ape,keep.tip)
importFrom(ape,read.tree)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,stat_ellipse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(jsonlite,read_json)
importFrom(jsonlite,toJSON)
importFrom(jsonvalidate,json_validate)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(stats,as.dist)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,p.adjust.methods)
importFrom(stats,quantile)
importFrom(tools,file_ext)
importFrom(utils,globalVariables)
importFrom(yyjsonr,validate_json_file)
useDynLib(OmicFlow, .registration = TRUE)

#!/usr/bin/Rscript

# Load Library -----------------------------------------------------------------
library("OmicFlow")
library("ggplot2")

## Parse args$options from command line
option_list <- list (
  optparse::make_option("--omics",
                        action = "store",
                        type = "character",
                        default = "metagenomics",
                        help="The mode to be selected, default: metagenomics"),
  optparse::make_option(c("-m", "--metadata"),
                        action = "store",
                        type = "character",
                        help="
                        A tab separated file, it should contain a `SAMPLE_ID` column with matching ids in the count table.
                        Optionally, a `SAMPLEPAIR_ID` or `FEATURE_ID` can be provided. Note: spaces are not allowed!"),
  optparse::make_option(c("-b", "--biom"),
                        action = "store",
                        type = "character",
                        help="
                        A Biological Observation Matrix (BIOM) format v2.1.0 compatible to the python module: http://biom-format.org.
                        Supports both BIOM in HDF5 and JSON format."),
  optparse::make_option(c("-t", "--tree"),
                        action = "store",
                        type = "character",
                        help="Phylogenetic tree in newick format, should be supported by `ape::read.tree` (OPTIONAL, default: NULL)",
                        default = NULL),
  optparse::make_option(c("-o", "--outdir"),
                        action = "store",
                        type = "character",
                        help="The directory to write the report, by default the current path is used.",
                        default = getwd()),
  optparse::make_option(c("-f", "--filename"),
                        action = "store",
                        type = "character",
                        help="Name of the HTML report, by default it is named as 'report.html'",
                        default = "report.html"),
  optparse::make_option(c("--i-beta-div"),
                        action = "store",
                        type = "character",
                        help="A custom dissimilarity matrix in TSV/CSV format. Compressed files are also supported."),
  optparse::make_option(c("--i-alpha-div"),
                        action = "store",
                        type = "character",
                        help="A custom alpha diversity created from rarefaction from QIIME2, it should contain columns starting wtih `depth-`"),
  optparse::make_option("--threads",
                        action = "store",
                        type = "numeric",
                        help="Number of threads to be used, mainly for querying of data.tables, default is set at 4",
                        default = 4)
)

parser <- optparse::OptionParser(
  usage = "Usage: Rscript %autoFlow.R [options]",
  option_list = option_list
  )
args <- optparse::parse_args(parser, positional_arguments=TRUE)

## Set threads for data.table
data.table::setDTthreads(threads = args$options$threads)

# main -------------------------------------------------------------------------
# switch statement based on omic selected, create object
if (args$options$omics == "metagenomics") {
  tax <- metagenomics$new(
    metaData = args$options$metadata,
    biomData = args$options$biom,
    treeData = args$options$tree
  )
  tax$feature_subset(Kingdom == "Bacteria")
  tax$normalize()
  
  # Perform automated analysis
  tax$autoFlow(
    beta_div_table = args$options$`i-beta-div`,
    alpha_div_table = args$options$`i-alpha-div`,
    threads = args$options$threads,
    normalize = FALSE,
    filename = file.path(args$options$outdir, args$options$filename)
  )
  
} else {
  cli::cli_alert_warning("{args$options$omics} did not match `metagenomics`")
}

#' Sub-class proteomics
#'
#' @description This is a sub-class that is compatible to preprocessed data obtained from https://fragpipe.nesvilab.org/. 
#' It inherits all methods from the abstract class \link{omics} and only adapts the \code{initialize} function.
#' It supports pre-existing data structures or paths to text files.
#' When omics data is very large, data loading becomes very expensive. It is therefore recommended to use the [`reset()`](#method-reset) method to reset your changes.
#' Every omics class creates an internal memory efficient back-up of the data, the resetting of changes is an instant process.
#' @seealso \link{omics}
#' @import R6
#' @importFrom ape read.tree
#' @export

proteomics <- R6::R6Class(
  classname = "proteomics",
  cloneable = TRUE,
  inherit = omics,
  active = list(
    #' @field treeData A "phylo" class, see \link[ape]{as.phylo}.
    treeData = function(value) {
      # back-up
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      success <- FALSE
      on.exit({
        if (!success) {
          private$.countData <- .countData
          private$.featureData <- .featureData
          private$.metaData <- .metaData
          private$.treeData <- .treeData
        }
      }, add = TRUE)

      if (missing(value)) {
        success <- TRUE
        private$.treeData
      } else if (inherits(value, "phylo")) {
        private$.treeData <- value
        private$sync()
        self$print()
        success <- TRUE
        invisible(self)
      } else {
        cli::cli_abort("Data input must be {.cls phylo} like {.field treeData}.")
      }
    }
  ),
  public = list(
    #' @description
    #' Initializes the proteomics class object with \code{proteomics$new()}
    #' @param countData A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.
    #' @param featureData A path to an existing file, \link[data.table]{data.table} or data.frame.
    #' @param metaData A path to an existing file, \link[data.table]{data.table} or data.frame.
    #' @param treeData A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.
    #' @return A new `proteomics` object.
    initialize = function(countData = NULL, metaData = NULL, featureData = NULL, treeData = NULL) {
      super$initialize(countData = countData,
                       metaData = metaData,
                       featureData = featureData)

      #-------------------#
      ###   treeData    ###
      #-------------------#

      if (!is.null(treeData)) {
        if (is.character(treeData) && length(treeData) == 1 && file.exists(treeData)) {
          private$.treeData <- ape::read.tree(treeData)
          cli::cli_alert_success("{.field treeData} is loaded.")
        } else if (inherits(treeData, "phylo")) {
          private$.treeData <- treeData
          cli::cli_alert_success("{.field treeData} is loaded.")
        } else {
          cli::cli_alert_warning("The provided {.field treeData} could not be loaded. Make sure the tree is supported by {.fun ape::read.tree}")
        }

        # Aligning featureData and countData rows by tree tips
        private$.featureData <- private$.featureData[order(match(private$.featureData[[ private$.feature_id ]], private$.treeData$tip.label))]
        private$.countData <- private$.countData[private$.featureData[[ private$.feature_id ]], ]
      }
      private$sync()
      self$print()

      # saves data for reset function
      private$original_data = list(
        counts = private$.countData,
        features = private$.featureData,
        metadata = private$.metaData,
        tree = private$.treeData
      )
    }
  ),
  private = list(
    # Private data fields
    #-------------------------#
    .countData = NULL,
    .featureData = NULL,
    .metaData = NULL,
    .treeData = NULL,
    .feature_id = "FEATURE_ID",
    .sample_id = "SAMPLE_ID",
    .samplepair_id = "SAMPLEPAIR_ID",
    original_data = list()
  )
)

#' Compositional plot
#'
#' @description Creates a stacked barchart of features. It is possible to both show barcharts for each sample or group them by a categorical variable.
#' The function is compatible with the class \link{omics} method \code{composition()}.
#'
#' @param data A \link[base]{data.frame} or \link[data.table]{data.table}.
#' @param palette An object with names and hexcode or color names, see \link{colormap}.
#' @param feature_rank A character variable of the feature column.
#' @param title_name A character to set the \code{ggtitle} of the \link[ggplot2]{ggplot}, (Default: NULL).
#' @param group_by A character variable to aggregate the stacked bars by group (Default: NULL).
#' @return A \link[ggplot2]{ggplot2} object to be further modified
#'
#' @importFrom ggplot2 ggplot aes .data geom_bar coord_flip theme_bw theme element_text scale_x_discrete scale_fill_manual labs ggtitle
#' 
#' @examples
#' library("ggplot2")
#' 
#' # Create mock_data as data.frame (data.table is also supported)
#' mock_data <- data.frame(
#'   SAMPLE_ID = rep(paste0("Sample", 1:10), each = 5),
#'   Genus = rep(c("GenusA","GenusB","GenusC","GenusD","GenusE"), times = 10),
#'   value = c(
#'     0.1119, 0.1303, 0.0680, 0.5833, 0.1065,      # Sample1
#'     0.2080, 0.1179, 0.0211, 0.4578, 0.1951,      # Sample2
#'     0.4219, 0.1189, 0.2320, 0.1037, 0.1235,      # Sample3
#'     0.4026, 0.0898, 0.1703, 0.1063, 0.2309,      # Sample4
#'     0.1211, 0.0478, 0.5721, 0.1973, 0.0618,      # Sample5
#'     0.2355, 0.0293, 0.2304, 0.1520, 0.3528,      # Sample6
#'     0.2904, 0.0347, 0.3651, 0.0555, 0.2544,      # Sample7
#'     0.4138, 0.0299, 0.0223, 0.4996, 0.0345,      # Sample8
#'     0.4088, 0.0573, 0.0155, 0.2888, 0.2296,      # Sample9
#'     0.4941, 0.0722, 0.2331, 0.1023, 0.0983       # Sample10
#'   ),
#'   Group = rep(c("Group1","Group2","Group1",
#'                 "Group1","Group2","Group2",
#'                  "Group1","Group1","Group1","Group2"),
#'                each = 5)
#' )
#' 
#' # Create a colormap
#' mock_palette <- c(
#'   GenusA = "#1f77b4",  # blue
#'   GenusB = "#ff7f0e",  # orange
#'   GenusC = "#2ca02c",  # green
#'   GenusD = "#d62728",  # red
#'   GenusE = "#9467bd"   # purple
#' )
#' 
#' # Optionally: Use OmicFlow::colormap()
#' mock_palette <- colormap(
#'   data = mock_data,
#'   col_name = "Genus",
#'   Brewer.palID = "RdYlBu"
#' )
#' 
#' composition_plot(
#'   data = mock_data,
#'   palette = mock_palette,
#'   feature_rank = "Genus",
#'   title_name = "Mock Genus Composition"
#' )
#' 
#' composition_plot(
#'   data = mock_data,
#'   palette = mock_palette,
#'   feature_rank = "Genus",
#'   title_name = "Mock Genus Composition by Group",
#'   group_by = "Group"
#' )
#' @export

composition_plot <- function(data,
                             palette,
                             feature_rank,
                             title_name = NULL,
                             group_by = NULL) {
  ## Error handling
  #--------------------------------------------------------------------#

  if (!inherits(data, "data.frame") && !inherits(data, "data.table"))
    cli::cli_abort("Data must be a {.cls data.frame} or {.cls data.table}.")

  if (!is.character(palette))
    cli::cli_abort("{.val {palette}} needs to contain characters.")

  if (!is.character(feature_rank) && length(feature_rank) != 1) {
    cli::cli_abort("{.val {feature_rank}} needs to contain characters with length of 1.")
  } else if (!column_exists(feature_rank, data)) {
    cli::cli_abort("The {.val {feature_rank}} column does not exist in the provided {.arg data}.")
  }

  if (!is.null(title_name) && !is.character(title_name))
    cli::cli_abort("{.val {title_name}} needs to be of type character.")

  if (!is.null(group_by)) {
    if (!is.character(group_by) && length(group_by) != 1) {
      cli::cli_abort("{.val {group_by}} must be a character and of length 1")
    } else if (!column_exists(group_by, data)) {
      cli::cli_abort("The specified {.val {group_by}} does not exist in the provided {.arg data}.")
    }
  }

  if (!column_exists("SAMPLE_ID", data))
    cli::cli_abort("{.arg SAMPLE_ID} needs to exist within the provided {.arg data}.")

  ## MAIN
  #--------------------------------------------------------------------#

  # Generates a stacked barplot as base with custome palette
  if (!is.null(group_by)) {
    plt <- data %>%
      ggplot(mapping = aes(y = .data[["value"]],
                           x = as.factor(.data[[ group_by ]]),
                           fill = .data[[ feature_rank ]]))
  } else {
    plt <- data %>%
      ggplot(mapping = aes(y = .data[["value"]],
                           x = .data[["SAMPLE_ID"]],
                           fill = base::get(feature_rank, data)))
  }
  # Required for stacked barplot
  plt <- plt +
    geom_bar(
      position = "fill",
      stat = "identity"
    )

  if (is.null(group_by)) {
    plt <- plt +
      coord_flip()
  }
  plt <- plt +
    theme_bw() +
    theme(
      plot.title = element_text(size = 14, face = "bold"),
      plot.subtitle = element_text(size = 13),
      axis.text.x = element_text(angle = 90, size = 12,
                                 vjust = 0.5, hjust=1,
                                 colour = "black"),
      axis.title.y = element_text(size = 12),
      axis.title.x = element_text(size = 12, vjust=0.5),
      legend.title = element_text(size = 14, face = "bold"),
      legend.text = element_text(size = 12, colour = "black"),
      axis.text.y = element_text(colour = "black", size = 12)
    )

  if (is.null(group_by)) {
    plt <- plt +
      scale_x_discrete(limits = rev(levels(as.factor(data[["SAMPLE_ID"]]))))
  }
  plt <- plt +
    scale_fill_manual(values = palette, name = feature_rank) +
    labs(y = "Rel. Abun.",
         x = NULL) +
    ggtitle(title_name)

  return(plt)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom magrittr %>%
#' @importFrom data.table := .SD
#' @importFrom stats na.omit p.adjust
#' @useDynLib OmicFlow, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom utils globalVariables
## usethis namespace: end

utils::globalVariables(c(".", "group_col"))
NULL

#' Diversity plot
#' 
#' @description Creates an Alpha diversity plot. This function is built into the class \link{omics} with method \code{alpha_diversity()}.
#' It computes the pairwise wilcox test, paired or non-paired, given a data frame and adds useful labelling.
#' @param data A \link[base]{data.frame} or \link[data.table]{data.table} computed from \link{diversity}.
#' @param values A column name of a continuous variable.
#' @param col_name A column name of a categorical variable.
#' @param group_by A column name to perform grouped statistical test (default: NULL).
#' @param palette An object with names and hexcode or color names, see \link{colormap}.
#' @param method A character variable indicating what method is used to compute the diversity.
#' @param paired A boolean value to perform paired analysis in \link[stats]{wilcox.test}.
#' @param p.adjust.method A character variable to specify the p.adjust.method to be used (Default: fdr).
#' @return A \link[ggplot2]{ggplot2} object to be further modified
#'
#' @importFrom ggplot2 ggplot aes .data theme_bw theme element_text scale_colour_manual scale_x_continuous labs geom_boxplot geom_segment geom_point facet_wrap position_jitter
#' @importFrom stats p.adjust.methods quantile median
#' 
#' @examples
#' library("ggplot2")
#'  
#' n_row <- 1000
#' n_col <- 100
#' density <- 0.2
#' num_entries <- n_row * n_col
#' num_nonzero <- round(num_entries * density)
#' 
#' set.seed(123)
#' positions <- sample(num_entries, num_nonzero, replace=FALSE)
#' row_idx <- ((positions - 1) %% n_row) + 1
#' col_idx <- ((positions - 1) %/% n_row) + 1
#' 
#' values <- runif(num_nonzero, min = 0, max = 1)
#' sparse_mat <- Matrix::sparseMatrix(
#'    i = row_idx,
#'    j = col_idx,
#'    x = values,
#'    dims = c(n_row, n_col)
#'  )
#' 
#' div <- OmicFlow::diversity(
#'   x = sparse_mat,
#'   metric = "shannon"
#' )
#' 
#' dt <- data.table::data.table(
#'   "shannon" = div,
#'   "treatment" = c(rep("healthy", n_col / 2), rep("tumor", n_col / 2)),
#'   "sex" = c(rep("male", n_col / 4), rep("female", n_col / 4))
#' )
#' 
#' colors <- OmicFlow::colormap(dt, "treatment")
#' 
#' # Comparing two groups
#' plt <- OmicFlow::diversity_plot(
#'  data = dt,
#'  values = "shannon",
#'  col_name = "treatment",
#'  palette = colors,
#'  method = "shannon",
#'  paired = FALSE,
#'  p.adjust.method = "fdr"
#' )
#' 
#' # Performing a test while stratifying the plot in two groups
#' plt <- OmicFlow::diversity_plot(
#'  data = dt,
#'  values = "shannon",
#'  col_name = "treatment",
#'  group_by = "sex",
#'  palette = colors,
#'  method = "shannon",
#'  paired = FALSE,
#'  p.adjust.method = "fdr"
#' )
#' @export

diversity_plot <- function(data,
                           values,
                           col_name,
                           group_by = NULL,
                           palette,
                           method,
                           paired = FALSE,
                           p.adjust.method = "fdr") {
  
  ## Error handling
  #--------------------------------------------------------------------#
  
  if (!inherits(data, "data.frame") && !inherits(data, "data.table"))
    cli::cli_abort("Data must be a {.cls data.frame} or {.cls data.table}.")
  
  if (!is.character(palette))
    cli::cli_abort("{.val {palette}} needs to contain characters.")
  
  if (!is.character(method)) {
    cli::cli_abort("{.val {method}} needs to be a character {.cls vector}.")
  }
  
  if (!is.character(values) && length(values) != 1) {
    cli::cli_abort("{.val {values}} needs to contain characters with length of 1.")
  } else if (!column_exists(values, data)) {
    cli::cli_abort("The {.val values}} column does not exist in the provided {.arg data}.")
  }
  
  if (!is.character(col_name) && length(col_name) != 1) {
    cli::cli_abort("{.val {col_name}} needs to contain characters with length of 1.")
  } else if (!column_exists(col_name, data)) {
    cli::cli_abort("The {.val {col_name}} column does not exist in the provided {.arg data}.")
  }
  
  if (!is.null(group_by)) {
    if (!is.character(group_by) && length(group_by) != 1) {
      cli::cli_abort("{.val {group_by}} needs to contain characters with length of 1.")
    } else if (!column_exists(group_by, data)) {
      cli::cli_abort("The {.val {group_by}} column does not exist in the provided {.arg data}.")
    }
  }
  
  if (!c(p.adjust.method %in% p.adjust.methods))
    cli::cli_abort("{.val {p.adjust.method}} is not a valid option. \nValid options: {.val {p.adjust.methods}}")
  
  ## MAIN
  #--------------------------------------------------------------------#
  
  result <- list()
  
  if (!is.null(group_by)) {
    pvalues_adjusted <- data[, {
      tmp <- rstatix::pairwise_wilcox_test(
        data   = .SD,
        formula = stats::reformulate(col_name, response = values),
        p.adjust.method = "fdr",
        paired = TRUE
      )
      tmp <- rstatix::add_significance(tmp)
      tmp <- rstatix::add_xy_position(tmp, x = group_by)
      tmp
    }, by = group_by]
    
    # Creates box_stats for half geom_box
    data.table::setnames(data, old = group_by, new = "group_col")
    group_by <- "group_col"
    box_stats <- data[, .(
      ymin = base::min(base::get(values)),
      ymax = base::max(base::get(values)),
      lower = quantile(base::get(values), 0.25),
      middle = median(base::get(values)),
      upper = quantile(base::get(values), 0.75)
    ), by = .(group_numeric = as.numeric(as.factor(base::get(col_name))), group_col)]
  } else {
    pvalues_adjusted <- data %>%
      rstatix::pairwise_wilcox_test(formula = stats::reformulate(col_name, response = values),
                                    p.adjust.method = p.adjust.method,
                                    paired = paired) %>%
      rstatix::add_significance() %>%
      rstatix::add_xy_position(x = col_name)
    
    # Creates box_stats for half geom_box
    box_stats <- data[, .(
      ymin = base::min(base::get(values)),
      ymax = base::max(base::get(values)),
      lower = quantile(base::get(values), 0.25),
      middle = median(base::get(values)),
      upper = quantile(base::get(values), 0.75)
    ), by = .(group_numeric = as.numeric(as.factor(get(col_name))))]
  }
  pvalues_adjusted.filtered <- pvalues_adjusted[grepl("\\*", pvalues_adjusted$p.adj.signif) ,]
  
  plt <- data %>%
    ggplot(mapping = aes(x = as.numeric(as.factor(.data[[col_name]])), y = .data[[values]]))
  # Custom half-boxplot using pre-computed stats
  if (!is.null(group_by)) {
    suppressWarnings(
      plt <- plt + geom_boxplot(
        data = box_stats,
        mapping = aes(x = .data$group_numeric - 0.2,
            ymin = .data$lower, ymax = .data$upper,
            lower = .data$lower, middle = .data$middle, upper = .data$upper,
            width = 0.4,
            group = base::interaction(.data$group_numeric, .data$group_col)),
        stat = "identity",
        fill = "white", color = "black",
        alpha = 0.8,
        inherit.aes = FALSE 
      )
    )
  } else {
    suppressWarnings(
      plt <- plt + geom_boxplot(
        data = box_stats,
        mapping = aes(x = .data$group_numeric - 0.2,
            ymin = .data$lower, ymax = .data$upper,
            lower = .data$lower, middle = .data$middle, upper = .data$upper,
            width = 0.4,
            group = base::interaction(.data$group_numeric)),
        stat = "identity",
        fill = "white", color = "black",
        alpha = 0.8,
        inherit.aes = FALSE 
      )
    )
  }
  plt <- plt +
    # Points on right side
    geom_point(
      mapping = aes(x = as.numeric(as.factor(.data[[col_name]])) + 0.2, color = as.factor(.data[[col_name]])), 
      position = position_jitter(width = 0.1, height = 0, seed = 1970), 
      shape = 20, size = 2) +
    geom_segment(
      data = box_stats,
      mapping = aes(x = .data$group_numeric, y = .data$ymin,
          xend = .data$group_numeric, yend = .data$ymax),
      color = "black", linewidth = 0.3
    ) +
    # Top horizontal segment
    geom_segment(
      data = box_stats,
      mapping = aes(x = .data$group_numeric - 0.1, y = .data$ymax,
          xend = .data$group_numeric, yend = .data$ymax),
      color = "black", linewidth = 0.3
    ) +
    # Bottom horizontal segment  
    geom_segment(
      data = box_stats,
      mapping = aes(x = .data$group_numeric - 0.1, y = .data$ymin,
          xend = .data$group_numeric, yend = .data$ymin),
      color = "black", linewidth = 0.3
    )
  
  if (!is.null(group_by)) {
    plt <- plt +
      facet_wrap(~.data[[ group_by ]])
  }
  
  plt <- plt +
    theme_bw() +
    theme(
      legend.position = "none",
      text=element_text(size=14),
      legend.text = element_text(size=12),
      legend.title = element_text(size=14),
      axis.text = element_text(size=12),
      axis.text.y = element_text(size=12),
      axis.text.x = element_text(size=12)
    ) +
    # Restore proper x-axis labels
    scale_x_continuous(breaks = seq_along(unique(data[[col_name]])),
                       labels = levels(as.factor(data[[col_name]]))) +
    scale_colour_manual(name = "groups",
                        values = palette) +
    ggpubr::stat_pvalue_manual(pvalues_adjusted.filtered,
                               label = "p.adj",
                               step.increase = 0.05) +
    labs(title = NULL,
         subtitle = paste0(
           "Attribute: ", col_name,
           ", test: ", ifelse(paired, "Wilcox signed rank test", "Mann-Whitney U test"),
           ", p.adjusted by ", p.adjust.method),
         x = "sample groups",
         y = paste0("Alpha diversity metric: ", method))
  
  result <- list(
    plot = plt,
    stats = pvalues_adjusted
  )
  
  return(result)
}

test_that("Testing `diversity_plot` error-handling", {  
  # Testing Shannon
  n_row <- 1000
  n_col <- 100
  density <- 0.2
  num_entries <- n_row * n_col
  num_nonzero <- round(num_entries * density)

  set.seed(123)
  positions <- sample(num_entries, num_nonzero, replace=FALSE)
  row_idx <- ((positions - 1) %% n_row) + 1
  col_idx <- ((positions - 1) %/% n_row) + 1

  values <- runif(num_nonzero, min = 0, max = 1)
  sparse_mat <- Matrix::sparseMatrix(
    i = row_idx,
    j = col_idx,
    x = values,
    dims = c(n_row, n_col)
  )

  div <- OmicFlow::diversity(
  x = sparse_mat,
  metric = "shannon"
  )

  dt <- data.table::data.table(
  "shannon" = div,
  "treatment" = c(rep("healthy", n_col / 2), rep("tumor", n_col / 2)),
  "sex" = c(rep("male", n_col / 4), rep("female", n_col / 4))
  )

  colors <- OmicFlow::colormap(dt, "treatment")

  expect_error(OmicFlow::diversity_plot(
    data = as.matrix(dt),
    values = "shannon",
    col_name = "treatment",
    palette = colors,
    method = "shannon"
  ))
  expect_error(OmicFlow::diversity_plot(
    data = dt,
    values = "val",
    col_name = "treatment",
    palette = colors,
    method = "shannon"
  ))
  expect_error(OmicFlow::diversity_plot(
    data = dt,
    values = "shannon",
    col_name = "treat",
    palette = colors,
    method = "shannon"
  ))
  expect_error(OmicFlow::diversity_plot(
    data = dt,
    values = "shannon",
    col_name = "treatment",
    palette = colors,
    method = 5
  ))
  expect_no_error(OmicFlow::diversity_plot(
    data = dt,
    values = "shannon",
    col_name = "treatment",
    palette = colors,
    method = "shannon",
    group_by = "sex"
  ))
})

#' Color map of a variable
#'
#' @description Creates an object of hexcode colors with names given a vector of characters.
#' This function is built into the \code{ordination} method from the abstract class \link{omics} and inherited by other omics classes, such as;
#' \link{metagenomics} and \link{proteomics}.
#'
#' @param data A \link[base]{data.frame} or \link[data.table]{data.table}.
#' @param col_name A column name of a categorical variable.
#' @param Brewer.palID A character name that exists in \link[RColorBrewer]{brewer.pal} (Default: \code{"Set2"}).
#' @return A \link[stats]{setNames}.
#'
#' @examples 
#' library("data.table")
#' dt <- data.table(
#'   "SAMPLE_ID" = c("sample_1", "sample_2", "sample_3"),
#'   "treatment" = c("healthy", "tumor", NA)
#' )
#'
#' colors <- colormap(data = dt,
#'                    col_name = "treatment")
#' @importFrom RColorBrewer brewer.pal.info brewer.pal
#' @export

colormap <- function(data,
                     col_name,
                     Brewer.palID = "Set2") {

  ## Error handling
  #--------------------------------------------------------------------#
  OPTIONS <- rownames(RColorBrewer::brewer.pal.info)

  if (!inherits(data, "data.frame") && !inherits(data, "data.table"))
    cli::cli_abort("Data must be a {.cls data.frame} or {.cls data.table}.")

  if (!is.character(col_name) && length(col_name) != 1) {
    cli::cli_abort("{.val {col_name}} needs to contain characters with length of 1.")
  } else if (!column_exists(col_name, data)) {
    cli::cli_abort("The {.val {col_name}} column does not exist in the provided data.")
  }

  if (!is.character(Brewer.palID) && length(Brewer.palID) != 1) {
    cli::cli_abort("The {.val {Brewer.palID}} needs to contain characters with length of 1.")
  } else if (!c(Brewer.palID %in% OPTIONS)) {
    cli::cli_abort("{.val {Brewer.palID}} is not a valid Brewer pal ID. \nValid options: {.val {OPTIONS}}.")
  }

  ## MAIN
  #--------------------------------------------------------------------#

  unique_groups <- unique(data[[col_name]])
  len_unique_groups <- length(unique_groups)
  suppressWarnings(
    chosen_palette <- RColorBrewer::brewer.pal(len_unique_groups, Brewer.palID)
  )
  colors <- stats::setNames(chosen_palette[1:len_unique_groups], unique_groups)
  return(colors)
}

test_that("Testing colormap", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  expect_error(colormap(data = as.matrix(taxa$metaData), col_name = "CONTRAST_sex"))
  expect_error(colormap(data = taxa$metaData))
  expect_error(colormap(data = taxa$metaData, col_name = "sex"))
  expect_error(colormap(data = taxa$metaData, col_name = "CONTRAST_sex", Brewer.palID = 2))
  expect_error(colormap(data = taxa$metaData, col_name = "CONTRAST_sex", Brewer.palID = "colSet"))
})

#' Compute UniFrac Dissimilarity from a Dense or Sparse Matrix.
#'
#' @description Calculates the UniFrac dissimilarity between samples based on phylogenetic branch lengths and abundance or presence/absence data.
#' 
#' @details
#' The UniFrac distance between two samples \eqn{A} and \eqn{B}, with phylogenetic tree edges \eqn{i = 1 \ldots n} of lengths \eqn{L_i}, is computed differently depending on the \code{weighted} and \code{normalized} flags. 
#' When \code{weighted = FALSE}, input counts are first converted to presence/absence data.
#' \describe{
#'  \item{Weighted UniFrac (\code{normalized = FALSE} and \code{weighted = TRUE}):}{
#'      \eqn{d(A,B) = \frac{\sum_{i}^n L_i |A_i - B_i|}{\sum_{i}^n L_i (A_i + B_i)}}
#'  }
#'  \item{Normalized Weighted UniFrac (\code{normalized = TRUE} and \code{weighted = TRUE}):}{
#'      \eqn{d(A,B) = \sum_{i}^n L_i |A_i - B_i|}
#'  }
#'  \item{Unweighted UniFrac (\code{weighted = FALSE}, unweighted is always normalized):}{
#'      \eqn{d(A,B) = \frac{\sum_{i}^n L_i |A_i - B_i|}{\sum_{i}^n L_i \max(A_i, B_i)}}
#'  }
#'}
#' @param x A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix} of strictly positive counts or presence/absence data.
#' @param tree A `phylo` class tree.
#' @param weighted A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).
#' @param normalized A boolean value, whether to normalize weighted UniFrac distances to be between 0 and 1 (default: TRUE). Unweighted UniFrac is always normalized.
#' @param threads A wholenumber, the number of threads to use in \link[RcppParallel]{setThreadOptions} (default: 1).
#' @return A column x column \link[stats]{dist} object.
#' @references
#' Lozupone, C., & Knight, R. (2005). UniFrac: a new phylogenetic method for comparing microbial communities. Applied and Environmental Microbiology, 71(12), 8228–8235.
#' @examples 
#' library("OmicFlow")
#'
#' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
#' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
#' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
#' tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")
#'
#' taxa <- metagenomics$new(
#'     metaData = metadata_file,
#'     countData = counts_file,
#'     featureData = features_file,
#'     treeData = tree_file
#' )
#'
#' taxa$feature_subset(Kingdom == "Bacteria")
#' taxa$scale(method = "tss")
#'
#' # Weighted UniFrac
#' unifrac(x = taxa$countData, tree = taxa$treeData, weighted=TRUE, normalized=FALSE)
#' 
#' # Weighted Normalized UniFrac
#' unifrac(x = taxa$countData, tree = taxa$treeData, weighted=TRUE, normalized=TRUE)
#' 
#' # Unweighted UniFrac
#' unifrac(x = taxa$countData, tree = taxa$treeData, weighted=FALSE)
#' @importFrom RcppParallel setThreadOptions
#' @importFrom Matrix sparseMatrix
#' @importFrom stats as.dist
#' @export

unifrac <- function(x, tree, weighted = TRUE, normalized = TRUE, threads = 1) {

    ## Error handling
    #--------------------------------------------------------------------#
    if (!inherits(tree, "phylo"))
        cli::cli_abort("Tree must be of class `phylo`.")

    if (inherits(x, "denseMatrix") || inherits(x, "matrix") || inherits(x, "sparseMatrix")) {
        x <- as(x, "CsparseMatrix")
    } else cli::cli_abort("Input isn't a {.cls matrix}, {.cls denseMatrix} or {.cls sparseMatrix}.")

    if (!is.numeric(x@x))
        cli::cli_abort("Input data must be numeric.")

    if (any(x@x < 0, na.rm = TRUE))
        cli::cli_abort("Input data must be non-negative.")

    if (!is.wholenumber(threads))
        cli::cli_abort("{.val {threads}} must be a whole number.")

    ## MAIN
    #--------------------------------------------------------------------#
    RcppParallel::setThreadOptions(numThreads = threads)

    ## Check if rows are aligned by tip.labels

    out <- .Call(
        '_OmicFlow_unifrac', PACKAGE = 'OmicFlow', 
        x, tree$edge-1, tree$edge.length, weighted, normalized
        )

    col_names <- colnames(x)
    if (!is.null(col_names))
        dimnames(out) <- list(col_names, col_names)

    return(as.dist(out))
}

#' Abstract omics class
#'
#' @description This is the abstract class 'omics', contains a variety of methods that are inherited and applied in the omics classes:
#' \link{metagenomics} and \link{proteomics}. 
#'
#' @details
#' Every class is created with the \link[R6]{R6Class} method. Methods are either public or private, and only the public components are inherited by other omic classes.
#' The omics class by default uses a \link[Matrix]{sparseMatrix} and \link[data.table]{data.table} data structures for quick and efficient data manipulation and returns the object by reference, same as the R6 class.
#' The method by reference is very efficient when dealing with big data.
#' @import R6 Matrix
#' @importFrom jsonlite toJSON
#' @importFrom jsonvalidate json_validate
#' @importFrom ape keep.tip
#' @export

omics <- R6::R6Class(
  classname = "omics",
  cloneable = TRUE,
  active = list(
    #' @field metaData A \link[data.table]{data.table} with `SAMPLE_ID` column.
    metaData = function(value) {
      # back-up
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      success <- FALSE
      on.exit({
        if (!success) {
          private$.countData <- .countData
          private$.featureData <- .featureData
          private$.metaData <- .metaData
          private$.treeData <- .treeData
        }
      }, add = TRUE)

      if (missing(value)) {
        success <- TRUE
        private$.metaData
      } else if (inherits(value, "data.table")) {
        private$.metaData <- value
        private$sync()
        success <- TRUE
        self$print()
        invisible(self)
      } else {
        cli::cli_abort("Data input must be {.cls data.table} like {.field metaData}.")
      }
    },
    #' @field featureData A \link[data.table]{data.table} with `FEATURE_ID` column.
    featureData = function(value) {
      # back-up
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      success <- FALSE
      on.exit({
        if (!success) {
          private$.countData <- .countData
          private$.featureData <- .featureData
          private$.metaData <- .metaData
          private$.treeData <- .treeData
        }
      }, add = TRUE)

      if (missing(value)) {
        success <- TRUE
        private$.featureData
      } else if (inherits(value, "data.table")) {
        private$.featureData <- value
        private$sync()
        success <- TRUE
        self$print()
        invisible(self)
      } else {
        cli::cli_abort("Data input must be {.cls data.table} like {.field featureData}.")
      }
    },
    #' @field countData A dense or sparse \link[Matrix]{Matrix}.
    countData = function(value) {
      # back-up
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      success <- FALSE
      on.exit({
        if (!success) {
          private$.countData <- .countData
          private$.featureData <- .featureData
          private$.metaData <- .metaData
          private$.treeData <- .treeData
        }
      }, add = TRUE)

      if (missing(value)) {
        success <- TRUE
        private$.countData
      } else if (inherits(value, "Matrix")) {
        private$.countData <- value
        private$sync()
        success <- TRUE
        self$print()
        invisible(self)
      } else {
        cli::cli_abort("Data input must be {.cls Matrix} like {.field countData}.")
      }
    }
  ),
  public = list(
    #' @description
    #' Wrapper function that is inherited and adapted for each omics class.
    #' The omics classes requires a metadata samplesheet, that is validated by the metadata_schema.json.
    #' It requires a column `SAMPLE_ID` and optionally a `SAMPLEPAIR_ID` can be supplied. 
    #' The `SAMPLE_ID` will be used to link the metaData to the countData, and will act as the key during subsetting of other columns.
    #' To create a new object use [`new()`](#method-new) method. Do notice that the abstract class only checks if the metadata is valid!
    #' The `countData` and `featureData` will not be checked, these are handled by the sub-classes. 
    #' Using the omics class to load your data is not supported and still experimental.
    #' @param countData A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.
    #' @param featureData A path to an existing file, \link[data.table]{data.table} or data.frame.
    #' @param metaData A path to an existing file, \link[data.table]{data.table} or data.frame.
    #' @return A new `omics` object.
    #'
    initialize = function(countData = NULL, featureData = NULL, metaData = NULL) {
      #-------------------#
      ###   metaData    ###
      #-------------------#
      if (!is.null(metaData)) {
        duplicated_sample_ids <- FALSE
        private$.metaData <- private$check_table(metaData)
        self$validate()

        if (private$.valid_schema) {
          cli::cli_alert_success("{.field metaData} template passed the JSON validation.")

          #--------------------------------------------------------------------#
          ## Checking for duplicated sample identifiers
          #--------------------------------------------------------------------#

          cli::cli_alert_info("Checking for duplicated identifiers ..")
          duplicated_sample_idx <- duplicated(private$.metaData, by = private$.sample_id)
          duplicated_sample_ids <- any(duplicated_sample_idx)
          if (duplicated_sample_ids) {
            duplicated_sample_names <- unique(private$.metaData[[private$.sample_id]][duplicated_sample_idx])
            cli::cli_abort(
              "Found duplicated: {.val {duplicated_sample_names}}\
              \n Make sure {.arg SAMPLE_ID} column contains {.strong unique} identifiers!"
            )
          }
          #--------------------------------------------------------------------#
          ## Disable samplepair_id if not supplied
          #--------------------------------------------------------------------#
          if (!column_exists(private$.samplepair_id, private$.metaData))
            private$.samplepair_id <- NULL

        } else {
          errors <- attr(private$.valid_schema, "errors")
          cli::cli_abort(
            "JSON validation failed: \n{ paste(errors$message, collapse = '\n')}"
            )
        }

      } else {
        cli::cli_abort(
          "{.field metaData} cannot be empty, please provide a {.cls data.frame}, {.cls data.table} or {.val filepath}"
        )
      }

      #-------------------#
      ###  featureData  ###
      #-------------------#
      if (!is.null(featureData)) {
        duplicated_feature_ids <- FALSE
        private$.featureData <- private$check_table(featureData)

        if (column_exists(private$.feature_id, private$.featureData)) {
          duplicated_feature_idx <- duplicated(private$.featureData, by = private$.feature_id)
          duplicated_feature_ids <- any(duplicated_feature_idx)

          if (duplicated_feature_ids) {
            duplicated_feature_names <- unique(private$.featureData[[private$.feature_id]][duplicated_feature_idx])
            cli::cli_abort(
              "Found duplicated: {.val {duplicated_feature_names}} \
              \n Make sure {.arg FEATURE_ID} column contains {.strong unique} identifiers!"
            )
          }

        } else {
          FEATURE_ID <- paste0("feature_", 1:nrow(private$.featureData))
          private$.featureData[, private$.feature_id := FEATURE_ID]
          data.table::setcolorder(
            x = private$.featureData,
            neworder = c(private$.feature_id, base::setdiff(colnames(private$.featureData), private$.feature_id))
          )
        }
        cli::cli_alert_success("{.field featureData} is loaded.")
      }

      #-------------------#
      ###   countData   ###
      #-------------------#
      if (!is.null(countData)) {
        private$.countData <- private$check_matrix(countData)
        cli::cli_alert_success("{.field countData} is loaded.")

        if (is.null(private$.featureData)) {
          private$add_featureData()
          cli::cli_alert_warning("Created placeholder {.field featureData}.")
        } else {
          rownames(private$.countData) <- private$.featureData[[ private$.feature_id ]]
        }
      }

      #-------------------#
      ###     sync      ###
      #-------------------#
      private$sync()

      # saves data for reset function
      private$original_data = list(
        counts = private$.countData,
        features = private$.featureData,
        metadata = private$.metaData,
        tree = private$.treeData
      )
    },
    #' @description
    #' Create a copy of the object-class
    #' 
    #' This method is very similar to the existing [`clone()`](#method-clone) function, except it also resets the back-up of the OmicFlow data types that is invoked with [`reset()`](#method-reset)
    #' 
    #' @param deep A boolean value to create a shallow or deep copy.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #'
    #' obj <- omics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file
    #' )
    #'
    #' # Perform a modification and copy
    #' obj$scale()
    #'
    #' cloned <- obj$copy(deep=TRUE)
    #' cloned$scale(method = "clr")
    #' cloned$reset() # resets to data after clone creation.
    #' 
    #' @return A copy of `omics` object
    copy = function(deep = FALSE) {
      # Base clone
      cloned <- self$clone(deep)

      # Resetting back-up
      cloned$.__enclos_env__$private$original_data <- list(
        counts = private$.countData,
        features = private$.featureData,
        metadata = private$.metaData,
        tree = private$.treeData
      )

      cloned
    },
    #' @description
    #' Validates an input metadata against the JSON schema. The metadata should look as follows and should not contain any empty spaces.
    #' For example; \code{'sample 1'} is not allowed, whereas \code{'sample1'} is allowed!
    #' 
    #' Acceptable column headers:
    #' * SAMPLE_ID (required)
    #' * SAMPLEPAIR_ID (optional)
    #' * CONTRAST_ (optional), used for [`autoFlow()`](#method-autoFlow).
    #' * VARIABLE_ (optional), not supported yet.
    #' 
    #' This function is used during the creation of a new object via [`new()`](#method-new) to validate the supplied metadata 
    #' via a filepath or existing \link[data.table]{data.table} or \link[base]{data.frame}.
    #' 
    #' @return None
    validate = function() {
      # Creates temporary json file from metadata
      tmp_json <- base::tempfile(fileext = ".json")

      json_data <- jsonlite::toJSON(
        private$.metaData,
        dataframe = "rows",
        pretty = TRUE,
        auto_unbox = TRUE
        )

      writeLines(json_data, tmp_json)

      private$.valid_schema <- jsonvalidate::json_validate(
        tmp_json,
        system.file("metadata_schema.json", package = "OmicFlow"),
        engine = "ajv",
        verbose = TRUE,
        error = FALSE,
        strict = TRUE
      )

      unlink(tmp_json)
      invisible(self)
    },
    #' @description
    #' Displays parameters of the omics class via stdout.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #'
    #' obj <- omics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file
    #' )
    #'
    #' # method 1 to call print function
    #' obj
    #'
    #' # method 2 to call print function
    #' obj$print()
    #'
    #' @return object in place
    print = function() {
      cli::cli_h3("{.cls {class(self)[1]}} object")
      if (length(private$.metaData) > 0) 
        cli::cli_inform("{.field metaData}: {.val {ncol(private$.metaData)}} variables {cli::symbol$times} {.val {nrow(private$.metaData)}} samples")
      if (length(private$.countData) > 0) 
        cli::cli_inform("{.field countData}: {.val {ncol(private$.countData)}} samples {cli::symbol$times} {.val {nrow(private$.countData)}} features")
      if (length(private$.featureData) > 0)
        cli::cli_inform("{.field featureData}: {.val {ncol(private$.featureData)-1}} attributes {cli::symbol$times} {.val {nrow(private$.featureData)}} features")
      if (length(private$.treeData) > 0)
        cli::cli_inform("{.field treeData}: {.val {length(private$.treeData$tip.label)}} tips {cli::symbol$times} {.val {private$.treeData$Nnode}} nodes")
    },
    #' @description
    #' Upon creation of a new `omics` object a small backup of the original data is created.
    #' Since modification of the object is done by reference and duplicates are not made, it is possible to `reset` changes to the class.
    #' The methods from the abstract class \link{omics} also contains a private method to prevent any changes to the original object when using methods such as \code{ordination} \code{alpha_diversity} or \code{foldchange}.
    #' @examples
    #' library(ggplot2)
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' taxa <- omics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file
    #' )
    #'
    #' # Performs modifications
    #' taxa$scale(transform = log2)
    #'
    #' # resets
    #' taxa$reset()
    #'
    #' # An inbuilt reset function prevents unwanted modification to the taxa object.
    #' taxa$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))
    #'
    #' @return object in place
    reset = function() {
      if (!is.null(private$original_data)) {
        private$.countData = private$original_data$counts
        private$.featureData = private$original_data$features
        private$.metaData = private$original_data$metadata
        private$.treeData = private$original_data$tree
        invisible(self)
      } else cli::cli_alert_warning("There is no back-up of the data made. This typically happens when the class is not initialized via the {fun. new}.")
    },
    #' @description
    #' Remove NAs from `metaData` and updates the `countData`.
    #' @param column The column from where NAs should be removed, this can be either a wholenumbers or characters. Vectors are also supported.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' 
    #' obj$removeNAs(column = "treatment")
    #' 
    #' @return object in place
    removeNAs = function(column) {

      ## Error handling
      #--------------------------------------------------------------------#

      if (all(is.wholenumber(column)) && length(column) <= length(colnames(private$.metaData)))
        column <- colnames(private$.metaData[column])

      if (!is.character(column))
        cli::cli_abort("{.val {column}} needs to be a character or an integer.")

      if (!column_exists(column, private$.metaData))
        cli::cli_abort("{.val {column}} does not exist in the {.field metaData} or one of the specified columns is completely empty!")

      ## MAIN
      #--------------------------------------------------------------------#
      private$.metaData <- na.omit(private$.metaData, cols = column)
      private$sync()
      invisible(self)
    },
    #' @description
    #' Feature subset (based on `featureData`), automatically applies data synchronization.
    #' @param ... Expressions that return a logical value, and are defined in terms of the variables in `featureData`.
    #' Only rows for which all conditions evaluate to TRUE are kept.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' 
    #' obj$feature_subset(Genus == "Pseudomonas")
    #' 
    #' @return object in place
    feature_subset = function(...) {
      # Replace all NAs by empty string
      features <- data.table::copy(private$.featureData)
      features[, names(features) := lapply(.SD, function(x) {
        if (is.character(x)) ifelse(is.na(x), "", x) else x
      })]

      rows_to_keep <- features[, ...]
      private$.featureData <- private$.featureData[rows_to_keep, ]
      private$.countData <- private$.countData[rows_to_keep, ]
      private$sync()
      self$print()
      invisible(self)
    },
    #' @description
    #' Sample subset (based on `metaData`), automatically applies synchronization.
    #' @param ... Expressions that return a logical value, and are defined in terms of the variables in `metaData`.
    #' Only rows for which all conditions evaluate to TRUE are kept.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' 
    #' obj$sample_subset(treatment == "tumor")
    #'
    #' @return object in place
    sample_subset = function(...) {
      # set order of columns
      private$.countData <- private$.countData[, private$.metaData[[ private$.sample_id ]], drop = FALSE]
      # subset columns and rows
      rows_to_keep <- private$.metaData[, ...]
      private$.metaData <- private$.metaData[rows_to_keep, ]
      # NAs can occur in rows_to_keep, which then doesnt work on sparse Matrix.
      private$.countData <- private$.countData[, private$.metaData[[ private$.sample_id ]] ]
      private$sync()
      self$print()
      invisible(self)
    },
    #' @description
    #' Samplepair subset (based on `metaData`), automatically applies synchronization.
    #' @param num_unique_pairs An integer value to define the number of pairs to subset. The default is NULL, 
    #' meaning the maximum number of unique pairs will be used to subset the data. 
    #' Let's say you have three samples for each pair, then the `num_unique_pairs` will be set to 3.
    #' 
    #' @return object in place
    samplepair_subset = function(num_unique_pairs = NULL) {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.null(num_unique_pairs) && !is.wholenumber(num_unique_pairs))
        cli::cli_abort("{.val {num_unique_pairs}} must contain integers!")

      ## MAIN
      #--------------------------------------------------------------------#

      counts <- private$.metaData[, .(unique_count = data.table::uniqueN(SAMPLE_ID)), by = SAMPLEPAIR_ID]

      if (is.null(num_unique_pairs)) {
        num_unique_pairs <- counts[, max(unique_count)]
      }

      private$.metaData <- private$.metaData[SAMPLEPAIR_ID %in% counts[unique_count == num_unique_pairs, SAMPLEPAIR_ID]]
      private$sync()
      self$print()
      invisible(self)
    },
    #' @description
    #' Agglomerates features by column, automatically applies synchronization.
    #' @param feature_rank A character value or vector of columns to aggregate from the `featureData`.
    #' @param feature_filter A character value or vector of characters to remove features via regex pattern.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' 
    #' obj$feature_merge(feature_rank = c("Kingdom", "Phylum"))
    #' obj$feature_merge(feature_rank = "Genus", feature_filter = c("uncultured", "metagenome"))
    #'
    #' @return object in place
    feature_merge = function(feature_rank, feature_filter = NULL) {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.character(feature_rank))
        cli::cli_abort("{.val {feature_rank}} needs to be a character or vector containing characters")

      if (!is.null(feature_filter) && !is.character(feature_filter))
        cli::cli_abort("{.val {feature_filter}} needs to be a character or vector containing characters")

      if (!column_exists(feature_rank, private$.featureData))
        cli::cli_abort("{.val {feature_rank}} does not exist in {.field featureData}!")

      ## MAIN
      #--------------------------------------------------------------------#
      # creates a subset of unique feature rank, hashes combined for each unique rank
      counts <- data.table::data.table()
      counts[, (private$.feature_id) := rownames(private$.countData)]

      # Supports multiple features
      features <- data.table::copy(private$.featureData[private$.featureData[[ feature_rank[1] ]] != "", ])

      # set keys
      data.table::setkey(counts, FEATURE_ID)
      data.table::setkey(features, FEATURE_ID)

      # Create groups by ID
      grouped_ids <- features[, .(IDs = list(FEATURE_ID)), by = feature_rank]
      counts_glom <- Matrix::Matrix(0,
                                    nrow = nrow(grouped_ids),
                                    ncol = ncol(private$.countData),
                                    dimnames = list(NULL, colnames(private$.countData)),
                                    sparse = TRUE)

      # Populate sparse matrix by colsums of identical taxa
      for (i in 1:nrow(grouped_ids)) {
        ids <- grouped_ids$IDs[[i]]
        if (length(ids) == 1) {
          counts_glom[i, ] <- private$.countData[ids, ]
        } else {
          counts_glom[i, ] <- Matrix::colSums(private$.countData[ids, ])
        }
      }

      # Prepare final self-components
      private$.featureData <- base::unique(features, by = feature_rank)
      # Fetch first ID from each list
      grouped_ids$ID_first <- sapply(grouped_ids$IDs, `[[`, 1)
      # Reorder by matching IDs
      private$.featureData <- private$.featureData[ base::order(base::match(private$.featureData[[ private$.feature_id ]], grouped_ids$ID_first)) ]
      private$.countData <- counts_glom

      # Replaces strings matching feature_filter with NAs
      if (!is.null(feature_filter)) {
        regex_pattern <- paste(feature_filter, collapse = "|")
        for (col in feature_rank) {
          private$.featureData[
            grepl(regex_pattern, get(col), ignore.case = TRUE),
            (col) := NA_character_
          ]
        }
      }

      # Clean up featureData
      empty_strings <- !is.na(private$.featureData[[ feature_rank[1] ]])
      private$.featureData <- private$.featureData[empty_strings, ]
      private$.countData <- private$.countData[empty_strings, ]
      rownames(private$.countData) <- private$.featureData[[ private$.feature_id ]]

      private$sync()
      self$print()
      invisible(self)
    },
    #' @description
    #' Feature scaling on the `countData`. The `scale` function is able to apply transformations element-wise on the positive values, (optional: add pseudocounts) and perform normalisation or standardisation methods.
    #' @param method A character to choose a standardisation/normalisation method, options: `tss`, `clr`, `binary`, `hellinger`, `none` (default: \code{"tss"}).
    #' @param transform A function to apply on the positive values of `countData`, skip standardisation/normalisation with \code{method = "none"} (default: \code{NULL}).
    #' @param base Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other (default: \code{exp(1)}) in CLR.
    #' @param pseudocount A numeric value to replace zero's (default: \code{NULL}).
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' # standard relative abundance computation
    #' obj$scale()
    #' 
    #' # CLR
    #' obj$reset()
    #' obj$scale(method = "clr")
    #' 
    #' # transform
    #' obj$reset()
    #' obj$scale(method = "none", transform = log2)
    #' 
    #' @return object in place
    scale = function(method = "tss", transform = NULL, base = exp(1), pseudocount = NULL) {

      ## Nested Functions
      #--------------------------------------------------------------------#
      tss <- function(x) {
        x@x <- x@x / rep(Matrix::colSums(x), base::diff(x@p))
        x
      }

      ## Error handling
      #--------------------------------------------------------------------#
      OPTIONS <- c("tss", "clr", "binary", "hellinger", "none")
      if (!is.null(method) && !is.character(method) && length(method) != 1) {
        cli::cli_abort("{.val {method}} needs to contain characters with length of 1.")
      } else if (!method %in% OPTIONS) {
        cli::cli_abort("{.val {method}} is not a valid method. Valid options: <{.val {OPTIONS}}>")
      }

      if (!is.null(pseudocount) && !is.numeric(pseudocount))
        cli::cli_abort("{.val {pseudocount}} needs to be a {.cls numeric} type.")

      if (!is.numeric(base))
        cli::cli_abort("{.val {base}} needs to be a {.cls numeric} type.")

      if (!is.null(transform) && !is.function(transform))
        cli::cli_abort("{.val {transform}} must be a function!")

      ## MAIN
      #--------------------------------------------------------------------#
      if (!is.null(pseudocount)) 
        private$.countData <- private$.countData + pseudocount
      
      if (is.function(transform))
        private$.countData@x <- transform(private$.countData@x)

      private$.countData <- switch(
        method,
        "tss" = tss(private$.countData),
        "clr" = {
          ref <- private$.countData
          ref@x <- log(ref@x, base=base)
          ref - Matrix::rowMeans(ref)
        },
        "binary" = {
          ref <- private$.countData
          ref@x[] <- 1
          ref
          },
        "hellinger" = {
          ref <- tss(private$.countData)
          ref@x <- sqrt(ref@x)
          ref
        },
        "none" = private$.countData
      )

      invisible(self)
    },
    #' @description
    #' Rank statistics based on `featureData`
    #' @details
    #' Counts the number of features identified for each column, for example in case of 16S metagenomics it would be the number of OTUs or ASVs on different taxonomy levels.
    #' @param feature_ranks A vector of characters or integers that match the `featureData`.
    #' @param unique A boolean value to display only unique entries in `feature_ranks`.
    #' @examples
    #' library("ggplot2")
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' 
    #' plt <- obj$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))
    #' plt
    #' @return A \link[ggplot2]{ggplot} object.
    #'
    rankstat = function(feature_ranks, unique = FALSE) {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.character(feature_ranks))
        cli::cli_abort("{.val {feature_ranks}} needs to be of character or integer type.")

      if (all(is.wholenumber(feature_ranks)) && length(feature_ranks) > length(colnames(private$.featureData)))
        feature_ranks <- colnames(private$.featureData[feature_ranks])

      if (!column_exists(feature_ranks, private$.featureData))
        cli::cli_abort("Specified {.val {feature_ranks}} do not exist in the {.field featureData}.")

      ## MAIN
      #--------------------------------------------------------------------#

      # Counts number of ASVs without empty values
      if (unique) {
        values <- private$.featureData[, 
          lapply(.SD, data.table::uniqueN)
          ][, .SD, .SDcols = feature_ranks] 
      } else {
        values <- private$.featureData[, 
          lapply(.SD, function(x) sum(!is.na(x) & x != ""))
          ][, .SD, .SDcols = feature_ranks]  
      }
      
      # Pivot into long table
      long_values <- data.table::melt(data = values,
                                      measure.vars = names(values),
                                      variable.name = "variable",
                                      value.name = "counts")

      # Sets order level of taxonomic ranks
      long_values[, variable := factor(variable, levels = base::rev(feature_ranks))]

      # Returns rankstat plot
      return(long_values %>%
               ggplot(mapping = aes(x = variable,
                                    y = counts)) +
               geom_col(fill = "grey",
                        colour = "grey15",
                        linewidth = 0.25) +
               coord_flip() +
               geom_text(mapping = aes(label = counts),
                         hjust = -0.1,
                         fontface = "bold") +
               ylim(0, max(long_values$counts)*1.10) +
               theme_bw() +
               labs(x = "Rank",
                    y = "Number of features classified"))
    },
    #' @description
    #' Alpha diversity based on \link{diversity}
    #' @param col_name A character variable from the `metaData`.
    #' @param metric An alpha diversity metric as input to \link{diversity}.
    #' @param group_by A column name to perform grouped statistical test in \link{diversity_plot} (default: NULL).
    #' @param Brewer.palID A character name for the palette set to be applied, see \link[RColorBrewer]{brewer.pal} or \link{colormap}.
    #' @param evenness A boolean wether to divide diversity by number of species, see \link[vegan]{specnumber}.
    #' @param paired A boolean value to perform paired analysis in \link[stats]{wilcox.test} and samplepair subsetting via [`samplepair_subset()`](#method-samplepair_subset)
    #' @param p.adjust.method A character variable to specify the p.adjust.method to be used, default is 'fdr'.
    #' @examples
    #' library("ggplot2")
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #' 
    #' plt <- obj$alpha_diversity(col_name = "treatment",
    #'                            metric = "shannon")
    #'
    #' @returns A list of components:
    #'  * `div` A \link[base]{data.frame} from \link{diversity}.
    #'  * `stats` A pairwise statistics from \link[rstatix]{pairwise_wilcox_test}.
    #'  * `plot` A \link[ggplot2]{ggplot} object.
    #' 
    #' @seealso \link{diversity_plot}
    alpha_diversity = function(col_name,
                               metric = c("shannon", "invsimpson", "simpson"),
                               Brewer.palID = "Set2",
                               group_by = NULL,
                               evenness = FALSE,
                               paired = FALSE,
                               p.adjust.method = "fdr") {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.character(col_name) && length(col_name) != 1) {
        cli::cli_abort("{.val {col_name}} must be a character and of length 1")
      } else if (!column_exists(col_name, private$.metaData)) {
        cli::cli_abort("The specified {.val {col_name}} does not exist in the {.field metaData}.")
      }

      if (!c(p.adjust.method %in% p.adjust.methods))
        cli::cli_abort("Specified {.val {p.adjust.method}} is not valid. \nValid options: {.val {p.adjust.methods}}")

      ## MAIN
      #--------------------------------------------------------------------#

      # OUTPUT: Plot list
      plot_list <- list()

      # Save omics class components
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      on.exit({
        private$.countData <- .countData
        private$.featureData <- .featureData
        private$.metaData <- .metaData
        private$.treeData <- .treeData
      }, add = TRUE)

      # Remove NAs when col_name is specified
      if (!is.null(col_name))
        self$removeNAs(col_name)

      if (!is.null(group_by)) {
        combined_cols <- c(col_name, group_by)
      } else {
        combined_cols <- col_name
      }

      # Subset by samplepair completion
      if ( paired && !is.null(private$.samplepair_id) )
        self$samplepair_subset()

      # Alpha diversity based on 'metric'
      div <- data.table::data.table(diversity(x = private$.countData, metric=metric))
      div[, (combined_cols) := private$.metaData[, .SD, .SDcols = c(combined_cols)]]
      # Adjusts for evenness
      if (evenness) div$V1 <- div$V1 / log(vegan::specnumber(div$V1))

      # get colors
      colors <- colormap(private$.metaData, col_name, Brewer.palID)

      # Create and saves plots
      plot_list$data <- div
      diversity_plt <- diversity_plot(
        data = na.omit(div),
        values = "V1",
        col_name = col_name,
        group_by = group_by,
        palette = colors,
        method = metric,
        paired = paired,
        p.adjust.method = p.adjust.method
        )

      plot_list$stats <- as.data.frame(diversity_plt$stats)
      plot_list$plot <- diversity_plt$plot

      return(plot_list)
    },
    #' @description
    #' Creates a table most abundant compositional features. Also assigns a color blind friendly palette for visualizations.
    #' @param feature_rank A character variable in `featureData` to aggregate via [`feature_merge()`](#method-feature_merge).
    #' @param feature_filter A character or vector of characters to removes features by regex pattern.
    #' @param col_name Optional, a character or vector of characters to add to the final compositional data output.
    #' @param feature_top A wholenumber of the top features to visualize, the max is 15, due to a limit of palettes.
    #' @param Brewer.palID A character name for the palette set to be applied, see \link[RColorBrewer]{brewer.pal} or \link{colormap}.
    #' @examples
    #' library("ggplot2")
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #'
    #' result <- obj$composition(feature_rank = "Genus",
    #'                           feature_filter = c("uncultured"),
    #'                           feature_top = 10)
    #'
    #' plt <- composition_plot(data = result$data,
    #'                         palette = result$palette,
    #'                         feature_rank = "Genus")
    #'
    #' @returns A list of components:
    #'  * `data` A \link[data.table]{data.table} of feature compositions.
    #'  * `palette` A \link[stats]{setNames} palette from \link{colormap}.
    #' 
    #' @seealso \link{composition_plot}
    composition = function(feature_rank,
                           feature_filter = NULL,
                           col_name = NULL,
                           feature_top = c(10, 15),
                           Brewer.palID = "RdYlBu") {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.null(col_name)) {
        if (!is.character(col_name) && length(col_name) != 1) {
          cli::cli_abort("{.val {col_name}} must be a character and of length 1")
        } else if (!column_exists(col_name, private$.metaData)) {
          cli::cli_abort("The specified {.val {col_name}} does not exist in the {.field metaData}.")
        }
      }

      if (!is.wholenumber(feature_top)) {
        cli::cli_abort("{.val {feature_top}} must be an integer!")
      } else if (feature_top > 15) {
        cli::cli_alert_warning("The {.val {feature_top}} is set to an integer higher than 15.\n This may lead that colors are difficult to be distinguished.\n For color-blind people it is recommended to use a feature_top of maximum 15.")
      }

      if (!is.character(Brewer.palID) && length(Brewer.palID) != 1)
        cli::cli_abort("{.val {Brewer.palID}} must be a character and of length 1")

      ## MAIN
      #--------------------------------------------------------------------#
      # Copies object to prevent modification of omics class components
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      on.exit({
        private$.countData <- .countData
        private$.featureData <- .featureData
        private$.metaData <- .metaData
        private$.treeData <- .treeData
      }, add = TRUE)

      # Agglomerate by feature_rank
      self$feature_merge(feature_rank = feature_rank, feature_filter = feature_filter)

      # Remove NAs when col_name is specified
      if (!is.null(col_name))
        self$removeNAs(col_name)

      # Converts matrix to data.table
      counts <- matrix_to_dtable(private$.countData)

      # Fetch unfiltered and filtered features
      dt <- counts[, (feature_rank) := private$.featureData[[feature_rank]]]

      # Create row_sums
      dt[, row_sum := rowSums(.SD), .SDcols = !c(feature_rank)]

      # Orders by row_sum in descending order
      data.table::setorder(dt, -row_sum)

      # Subset taxa for visualization
      final_dt <- rbind(dt[1:feature_top][, .SD, .SDcols = !c("row_sum")],
                        dt[(feature_top+1):nrow(dt)][, lapply(.SD, function(x) sum(x)),
                                                                 .SDcols = !c(feature_rank, "row_sum")],
                        fill = TRUE)
      
      # Creates palette
      df_taxa_len <- length(final_dt[[feature_rank]])
      if (df_taxa_len-1 <= 15 && df_taxa_len-1 > 10) {
        chosen_palette <- c("#000000","#004949","#009292","#ff6db6","#ffb6db",
                            "#490092","#006ddb","#b66dff","#6db6ff","#b6dbff",
                            "#920000","#924900","#db6d00","#24ff24","#ffff6d")[1:df_taxa_len-1]
      } else {
        chosen_palette <- RColorBrewer::brewer.pal(df_taxa_len-1, Brewer.palID)
      }
      

      # Add 'Others'
      if (df_taxa_len == feature_top+1) {
        final_dt[nrow(final_dt), (feature_rank)] <- "Other"
        taxa_colors_ordered <- stats::setNames(c(chosen_palette, "lightgrey"), final_dt[[feature_rank]])
      } else {
        taxa_colors_ordered <- stats::setNames(chosen_palette, final_dt[[feature_rank]])
      }

      # Pivoting in long table and factoring feature ranke
      final_long <- data.table::melt(final_dt,
                                     id.vars = c(feature_rank),
                                     variable.factor = FALSE,
                                     value.factor = TRUE)
      # Rename colnames for merge step
      colnames(final_long) <- c(feature_rank, private$.sample_id, "value")

      # Adds metadata columns by user input
      if (!is.null(col_name)) {
        composition_final <- base::merge(final_long,
                                         private$.metaData[, .SD, .SDcols = c(private$.sample_id, col_name)],
                                         by = private$.sample_id,
                                         all = TRUE,
                                         allow.cartesian = TRUE) %>%
          unique()
      } else {
        composition_final <- final_long
      }

      # Factors the melted data.table by the original order of Taxa
      # Important for scale_fill_manual taxa order
      composition_final[[feature_rank]] <- factor(composition_final[[feature_rank]], levels = final_dt[[feature_rank]])

      # returns results as list
      return(
        list(
          data = composition_final,
          palette = taxa_colors_ordered
        )
      )
    },
    #' @description
    #' Compute a distance metric from `countData`
    #' @param metric A dissimilarity metric to be applied on the `countData`, 
    #' thus far supports 'bray', 'jaccard', 'cosine', 'manhattan', 'aitchison', 'euclidean', 'jsd' (jensen-shannon divergence), 'canberra' and 'unifrac' when a tree is provided via `treeData`, see [`distance()`](#method-distance).
    #' @param weighted A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).
    #' @param normalized A boolean value, whether to normalize weighted UniFrac distances to be between 0 and 1. Unweighted UniFrac is always normalized (default: TRUE).
    #' @param pseudocount A numeric value to replace zero's, used in [`scale()`](#method-scale) (default: \code{1e-15}).
    #' @param base Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other (default: \code{exp(1)}).
    #' @param threads A wholenumber, indicating the number of threads to use (Default: 1).
    #' @return A column x column \link[stats]{dist} object.
    #' @references
    #' Aitchison, J. (1986) The Statistical Analysis of Compositional Data. Chapman and Hall, London, 416 p.
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'     metaData = metadata_file,
    #'     countData = counts_file,
    #'     featureData = features_file
    #' )
    #'
    #' obj$feature_subset(Kingdom == "Bacteria")
    #' dist <- obj$distance(metric = "bray")
    #' @seealso \link{bray}, \link{canberra}, \link{cosine}, \link{jaccard}, \link{jsd}, \link{manhattan}, \link{unifrac}
    distance = function(metric, weighted = TRUE, threads = 1, normalized = TRUE, base = exp(1)) {

      ## Error handling
      #--------------------------------------------------------------------#
      OPTIONS <- c(
        "bray", "jaccard", "cosine", "manhattan",
        "jsd", "canberra", "unifrac", "euclidean", 
        "aitchison"
        )

      if (!is.character(metric) && length(metric) != 1) {
        cli::cli_abort("{.val {metric}} needs to be a character with a length of 1")
      } else if (!metric %in% OPTIONS) {
        cli::cli_abort("{.val {metric}} is not a valid metric. \nValid options: {.val {OPTIONS}}")
      }

      if (!is.wholenumber(threads))
        cli::cli_abort("{.val {threads}} need to be an integer!")

      if (is.null(private$.treeData) && metric == "unifrac")
        cli::cli_abort("The specified {.val {metric}} is invalid since no {.field treeData} is supplied.")

      ## MAIN
      #--------------------------------------------------------------------#

      # Copies object to prevent modification of omics class components
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      on.exit({
        private$.countData <- .countData
        private$.featureData <- .featureData
        private$.metaData <- .metaData
        private$.treeData <- .treeData
      }, add = TRUE)

      distmat <- switch(
        metric,
        "unifrac" = OmicFlow::unifrac(x = private$.countData, tree = private$.treeData, weighted=weighted, normalized=normalized, threads=threads),
        "manhattan" = OmicFlow::manhattan(x = private$.countData, weighted=weighted, threads=threads),
        "canberra" = OmicFlow::canberra(x = private$.countData, weighted=weighted, threads=threads),
        "jaccard" = OmicFlow::jaccard(x = private$.countData, weighted=weighted, threads=threads),
        "bray" = OmicFlow::bray(x = private$.countData, weighted=weighted, threads=threads),
        "jsd" = OmicFlow::jsd(x = private$.countData, weighted=weighted, threads=threads),
        "cosine" = OmicFlow::cosine(x = private$.countData, weighted=weighted, threads=threads),
        "euclidean" = OmicFlow::euclidean(x = private$.countData, weighted=weighted, threads=threads),
        "aitchison" = {
            self$scale(method = "clr", base=base, pseudocount=NULL)
            OmicFlow::euclidean(x = private$.countData, weighted=weighted, threads=threads)
        }
      )
      return(distmat)
    },
    #' @description
    #' Ordination of `countData` with statistical testing.
    #' @param metric A dissimilarity or similarity metric to be applied on the `countData`, 
    #' thus far supports 'bray', 'jaccard', 'cosine', 'manhattan', 'jsd' (jensen-shannon divergence), 'canberra' and 'unifrac' when a tree is provided via `treeData`, see [`distance()`](#method-distance).
    #' @param method Ordination method, supports "pcoa" and "nmds", see \link[vegan]{wcmdscale}.
    #' @param distmat A custom distance matrix in either \link[stats]{dist} or \link[Matrix]{Matrix} format.
    #' @param group_by A character variable in `metaData` to be used for the \link{pairwise_adonis} or \link{pairwise_anosim} statistical test.
    #' @param weighted A boolean value, whether to compute weighted or unweighted dissimilarities (default: \code{TRUE}).
    #' @param threads A wholenumber, indicating the number of threads to use (Default: 1).
    #' @param perm_design A function that takes `metaData` and constructs a permutation design with \link[permute]{how} (default: \code{NULL}).
    #' @param perm A wholenumber, number of permutations to compare against the null hypothesis of \link[vegan]{adonis2} and \link[vegan]{anosim} (default: \code{perm=999}).
    #' @examples
    #' library("ggplot2")
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #' )
    #'
    #' pcoa_plots <- obj$ordination(metric = "bray",
    #'                              method = "pcoa",
    #'                              group_by = "treatment",
    #'                              weighted = TRUE)
    #' pcoa_plots
    #'
    #' @returns A list of components:
    #'  * `distmat` A distance dissimilarity in \link[base]{matrix} format.
    #'  * `stats` A statistical test as a \link[base]{data.frame}.
    #'  * `pcs` principal components as a \link[base]{data.frame}.
    #'  * `scree_plot` A \link[ggplot2]{ggplot} object.
    #'  * `anova_plot` A \link[ggplot2]{ggplot} object.
    #'  * `scores_plot` A \link[ggplot2]{ggplot} object.
    #' 
    #' @seealso \link{ordination_plot}, \link{plot_pairwise_stats}, \link{pairwise_anosim}, \link{pairwise_adonis}
    ordination = function(metric = "bray",
                          method = c("pcoa", "nmds"),
                          group_by,
                          distmat = NULL,
                          weighted = TRUE,
                          threads = 1,
                          perm_design = NULL,
                          perm = 999) {

      ## Error handling
      #--------------------------------------------------------------------#
      if (!is.character(method) && length(method) != 1)
        cli::cli_abort("{.val {method}} needs to be a character with a length of 1")

      if (!is.character(group_by) && length(group_by) != 1) {
        cli::cli_abort("{.val {group_by}} needs to be a character with a length of 1")
      } else if (!column_exists(group_by, private$.metaData)) {
        cli::cli_abort("{.val {group_by}} does not exist in the metaData or is empty.")
      }

      if (!is.null(perm_design) && !is.function(perm_design))
        cli::cli_abort("perm_design must be a function.")

      if (!is.wholenumber(perm))
        cli::cli_abort("Permutations {.val {perm}} need to be an integer")

      if (!is.null(distmat) && (!inherits(distmat, "Matrix") && !inherits(distmat, "dist")))
        cli::cli_abort("{.arg distmat} need to be {.cls Matrix} or {.cls dist}")

      if (is.null(private$.treeData) && metric == "unifrac") {
        cli::cli_alert_warning("The specified {.val {metric}} is invalid since no tree is supplied.\n Switching to bray-curtis metric.")
        metric <- "bray"
      }

      ## MAIN
      #--------------------------------------------------------------------#
      # Copies object to prevent modification of omics class components
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      on.exit({
        private$.countData <- .countData
        private$.featureData <- .featureData
        private$.metaData <- .metaData
        private$.treeData <- .treeData
      }, add = TRUE)

      # Subset by missing values
      self$removeNAs(group_by)
      if (inherits(distmat, "Matrix")) {
        distmat <- distmat[private$.metaData[[ private$.sample_id ]], private$.metaData[[ private$.sample_id ]]]
        distmat <- as.dist(distmat)
      }

      # Creates a list of plots
      plot_list <- list()

      if (is.null(distmat)) {
        distmat <- self$distance(
          metric = metric,
          weighted = weighted,
          threads = threads
          )
      }

      plot_list$dist <- as.matrix(distmat)

      # Switch case to compute loading scores
      pcs <- switch(
        method,
        "pcoa" = vegan::wcmdscale(d = distmat,
                                  k = 15,
                                  eig = TRUE),
        "nmds" = vegan::metaMDS(distmat,
                                trace = FALSE,
                                autotransform = FALSE)
      )
      if (!is.null(perm_design)) metadata <- private$.metaData else metadata <- NULL
      # Switch case to compute relevant statistics
      stats_results <- switch(
        method,
        "pcoa" = pairwise_adonis(distmat, groups = private$.metaData[[ group_by ]], perm = perm, perm_design = perm_design, metadata = metadata),
        "nmds" = pairwise_anosim(distmat, groups = private$.metaData[[ group_by ]], perm = perm, perm_design = perm_design, metadata = metadata)
      )
      plot_list$anova_data <- stats_results

      # Data table of loading scores
      df_pcs_points <- data.table::data.table(pcs$points)

      if (method == "pcoa") {
        # Normalisation of eigenvalues
        pcs$eig_norm <- unlist(lapply(pcs$eig, function(x) x / sum(pcs$eig) * 100))
        colnames(df_pcs_points) <- paste0("PC", 1:ncol(df_pcs_points))

      } else if (method == "nmds") {
        df_pcs_points[['stress']] <- pcs$stress
      }
      plot_list$pcs <- df_pcs_points

      # Adds relevant data
      df_pcs_points[, groups := private$.metaData[[ group_by ]] ]
      df_pcs_points[, samples := row.names(df_pcs_points) ]

      if (method == "pcoa") {
        # Scree plot of first 10 dimensions
        plot_list$scree_plot <- data.table::data.table(
          dims = seq(length(pcs$eig_norm[1:10])),
          dims.explained = pcs$eig_norm[1:10]
        ) %>%
          ggplot(mapping = aes(x = dims,
                               y = dims.explained)) +
          geom_col() +
          theme_bw() +
          scale_x_continuous(breaks=seq(1, 10, 1)) +
          scale_y_continuous(breaks=seq(0, 100, 10)) +
          labs(title = paste0("Screeplot of ", length(pcs$eig_norm[1:10])," PCs"),
               x = "Principal Components (PCs)",
               y = "dissimilarity explained [%]")

        # PERMANOVA
        plot_list$anova_plot <- plot_pairwise_stats(
          data = stats_results,
          group_col = "pairs",
          stats_col = "F.Model",
          label_col = "p.adj",
          y_axis_title = "Pseudo F test statistic",
          plot_title = "PERMANOVA"
        )
        # Loading score plot
        plot_list$scores_plot <- ordination_plot(
          data = df_pcs_points,
          col_name = "groups",
          pair=c("PC1", "PC2"),
          dist_explained = pcs$eig_norm,
          dist_metric = metric
        )

      } else if (method == "nmds") {
        plot_list$anova_plot <- plot_pairwise_stats(
          data = stats_results,
          group_col = "pairs",
          stats_col = "anosimR",
          label_col = "p.adj",
          y_axis_title = "ANOSIM R statistic",
          plot_title = "ANOSIM"
        )

        plot_list$scores_plot <- ordination_plot(
          data = df_pcs_points,
          col_name = "groups",
          pair=c("MDS1", "MDS2"),
          dist_metric = metric
        )
      }

      return(plot_list)
    },
    #' @description
    #' Differential feature expression (DFE) on log-transformed values for both paired and non-paired test.
    #' 
    #' The function performs feature agglomeration, subsetting to remove NAs in `condition.group` and finding samplepairs. It expects that the data is already log-transformed, this can be accomplished via [`scale()`](#method-scale)
    #' 
    #' @param feature_rank A character or vector of characters in the `featureData` to aggregate via [`feature_merge()`](#method-feature_merge) (default: \code{"FEATURE_ID"}).
    #' @param feature_filter A character or vector of characters to remove features via regex pattern (default: \code{NULL}).
    #' @param paired A boolean value, the paired is only applicable when a `SAMPLEPAIR_ID` column exists within the `metaData`. See \link[stats]{wilcox.test} and [`samplepair_subset()`](#method-samplepair_subset).
    #' @param condition.group A character variable of an existing column name in `metaData`, wherein the conditions A and B are located.
    #' @param group_by A character variable of an existing column in `metaData` to split the table in chunks prior to fold-change computation (default: \code{NULL}). When disabled then column names will end with `_in_all`.
    #' @param condition_A A character value or vector of characters.
    #' @param condition_B A character value or vector of characters.
    #' @param pvalue.threshold A numeric value used as a p-value threshold to label and color significant features (default: 0.05).
    #' @param logfold.threshold A numeric value used as a fold-change threshold to label and color significantly expressed features (default: 0.06).
    #' @param abundance.threshold A numeric value used as an abundance threshold to size the scatter dots based on their mean abundance (default: 0.01).
    #' @examples
    #' library("ggplot2")
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- omics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file
    #' )
    #' obj$scale(method = "clr")
    #' 
    #' dfe <- obj$foldchange(feature_rank = "Genus",
    #'                       paired = FALSE,
    #'                       condition.group = "treatment",
    #'                       condition_A = c("healthy"),
    #'                       condition_B = c("tumor"))
    #' 
    #' @returns
    #'  * `data` A long \link[data.table]{data.table} table.
    #'  * `volcano_plot` A \link[ggplot2]{ggplot} object.
    #'  * `A` A \link[data.table]{data.table} table for (each) condition A
    #'  * `B` A \link[data.table]{data.table} table for (each) condition B
    #'
    #' @seealso \link{volcano_plot}
    foldchange = function(
      condition.group,
      condition_A,
      condition_B,
      group_by = NULL,
      feature_rank = "FEATURE_ID",
      feature_filter = NULL,
      paired = FALSE,
      pvalue.threshold = 0.05,
      logfold.threshold = 0.06,
      abundance.threshold = 0
      ) {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.character(feature_rank) && length(feature_rank) != 1) {
        cli::cli_abort("{.val {feature_rank}} needs to be a character with a length of 1")
      } else if (!column_exists(feature_rank, private$.featureData)) {
        cli::cli_abort("The {.val {feature_rank}} column does not exist in the {.field featureData}.")
      }

      if (!is.character(condition.group) && length(condition.group) != 1) {
        cli::cli_abort("{.val {condition.group}} needs to be a character with a length of 1")
      } else if (!column_exists(condition.group, private$.metaData)) {
        cli::cli_abort("{.val {condition.group}} does not exist in the {.field metaData} or is empty.")
      }
      if (!is.character(condition_A))
        cli::cli_abort("{.val {condition_A}} needs to be a character.")

      if (!is.character(condition_B))
        cli::cli_abort("{.val {condition_B}} needs to be a character.")

      if (!is.numeric(pvalue.threshold))
        cli::cli_abort("{.val {pvalue.threshold}} need to be numeric.")

      if (!is.numeric(logfold.threshold))
        cli::cli_abort("{.val {logfold.threshold}} need to be numeric.")

      if (paired && is.null(private$.samplepair_id)) {
        cli::cli_alert_warning("Paired is set to {.val {paired}} but {.arg SAMPLEPAIR_ID} does not exist in the {.field metaData}.\n Differential feature analysis will continue now with paired set to {.val FALSE}!")
        paired <- FALSE
      }

      if (!is.null(group_by)) {
        if (!is.character(group_by) && length(group_by) != 1) {
          cli::cli_abort("{.val {group_by}} needs to contain characters with length of 1.")
        } else if (!column_exists(group_by, private$.metaData)) {
          cli::cli_abort("The {.val {group_by}} column does not exist in the {.field metaData}.")
        }
      }

      ## MAIN
      #--------------------------------------------------------------------#

      # Final output
      output <- list()

      # Copies object to prevent modification of omics class components
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      on.exit({
        private$.countData <- .countData
        private$.featureData <- .featureData
        private$.metaData <- .metaData
        private$.treeData <- .treeData
      }, add = TRUE)

      # Agglomerate taxa by feature rank and filter unwanted taxa
      self$feature_merge(feature_rank = feature_rank,
                         feature_filter = feature_filter)

      # Subset by missing values
      self$removeNAs(condition.group)

      # Subset by samplepair completion
      if (paired && !is.null(private$.samplepair_id))
        self$samplepair_subset()
      
      # Extract mean abundance
      abun <- as.matrix(Matrix::rowMeans(private$.countData))
      rownames(abun) <- private$.featureData[[ feature_rank ]]

      # Get data.table format abundances
      dt <- matrix_to_dtable(private$.countData)[, (feature_rank) := private$.featureData[[feature_rank]]]

      # Compute 2-fold expression based on (un)paired samples
      # Supports multiple inputs for A and B.
      tmp_dt <- data.table::copy(dt)

      # Apply `group_by`
      if (!is.null(group_by)) {
        chunks <- base::split(private$.metaData, by = group_by)
      } else {
        chunks <- list(all = private$.metaData)
      }
      group_names <- names(chunks)

      # subset feature labels before removing them
      feature_labels <- tmp_dt[[ feature_rank ]]
      tmp_dt <- tmp_dt[, .SD, .SDcols = !c(feature_rank)]

      # Create data.tables for results
      foldchange_dt <- data.table::data.table(feature_rank = feature_labels)
      colnames(foldchange_dt) <- feature_rank

      for (group_name in group_names) {
        condition_labels <- chunks[[group_name]][[condition.group]]

        for (i in seq_along(condition_A)) {
          # Subset by condition_A value
          dt_A <- tmp_dt[, .SD, .SDcols = colnames(tmp_dt)[condition_labels %in% condition_A[i]]]
          dt_B <- tmp_dt[, .SD, .SDcols = colnames(tmp_dt)[condition_labels %in% condition_B[i]]]

          # save intermediate condition tables
          output[[paste0(group_name, "_", condition_A[i])]] <- dt_A
          output[[paste0(group_name, "_", condition_B[i])]] <- dt_B

          # Convert to dense matrix
          mat_A <- as.matrix(dt_A)
          mat_B <- as.matrix(dt_B)

          # Feature means per condition
          row_means_A <- Matrix::rowMeans(mat_A)
          row_means_B <- Matrix::rowMeans(mat_B)

          # Log A - log B
          result <- row_means_A - row_means_B

          # Combines to final foldchange data table
          foldchange_dt <- cbind(foldchange_dt, result)
          result_col_title <- paste0(condition_A[i], "_vs_", condition_B[i], "_in_", group_name)
          colnames(foldchange_dt)[grepl("result", colnames(foldchange_dt))] <- paste0("Log2FC_", result_col_title)

          for (k in seq_along(feature_labels)) {
            # save p-values in data.table
            suppressWarnings(
              foldchange_dt[
                k, (paste0("pvalue_", result_col_title)) := stats::wilcox.test(
                  mat_A[k, ], mat_B[k, ],
                  correct = TRUE,
                  paired = paired
                  )$p.value
                ]
            )
          }
        }
      }

      # Add abundance, and save data as output list
      dfe <- foldchange_dt[, "abun" := abun]
      output$data <- dfe

      #----------------------#
      # Visualization        #
      #----------------------#

      # Create & save volcano plot
      colnames_dfe <- colnames(dfe)
      diff_columns <- colnames_dfe[grepl("Log2FC", colnames_dfe)]
      pvalue_columns <- colnames_dfe[grepl("pvalue", colnames_dfe)]
      n_diff_columns <- length(diff_columns)

      output$volcano_plot <- lapply(1:n_diff_columns, function(i) {
        volcano_plot(
          data = dfe,
          logfold_col = diff_columns[i],
          pvalue_col = pvalue_columns[i],
          feature_rank = feature_rank,
          abundance_col = "abun",
          pvalue.threshold = pvalue.threshold,
          logfold.threshold = logfold.threshold,
          abundance.threshold = abundance.threshold,
          label_A = condition_A,
          label_B = condition_B
        ) + labs(
            subtitle = paste0(
              "Attribute: ", condition.group,
              ", test: ", ifelse(paired, "Wilcox signed rank test", "Mann-Whitney U test")
              )
          )
      })

      return(output)
    },
    #' @description
    #' Automated Omics Analysis based on the `metaData`, see [`validate()`](#method-validate).
    #' For now only works with headers that start with prefix `CONTRAST_`. If the data is from the class `omics` or `proteomics` than FDR adjusted p-values are computed for the volcano plots. Log-transformed values will lead to the skipping of [`composition()`](#method-composition) and [`alpha_diversity()`](#method-alpha_diversity) methods.
    #' @param feature_contrast A character vector of feature columns in the `featureData` to aggregate via [`feature_merge()`](#method-feature_merge) (default: \code{"FEATURE_ID"}).
    #' @param feature_filter A character vector to filter unwanted features, (default: \code{NULL}).
    #' @param feature_ranks A character vector as input to [`rankstat()`](#method-rankstat) (default: \code{NULL}).
    #' @param distance_metrics A character vector specifying what (dis)similarity metrics to use (default: \code{c("bray")}) When you are working with log-transformed data it is advised to use the `euclidean`.
    #' @param distmat A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format (default: \code{NULL}).
    #' @param weighted A boolean value, whether to compute weighted or unweighted dissimilarities (default: \code{TRUE}).
    #' @param pvalue.threshold A numeric value, the p-value is used to include/exclude composition and foldchanges plots coming from alpha- and beta diversity analysis (default: 0.05).
    #' @param logfold.threshold A numeric value used as a fold-change threshold to label and color significantly expressed features, see [`foldchange()`](#method-foldchange) (Default: 1).
    #' @param abundance.threshold A numeric value used as an abundance threshold to size the scatter dots based on their mean abundance, see [`foldchange()`](#method-foldchange) (default: 0.01).
    #' @param perm A wholenumber, number of permutations to compare against the null hypothesis of \link[vegan]{adonis2} or \link[vegan]{anosim} (default: 999).
    #' @param threads Number of threads to use, only used in [`distance()`](#method-distance) when distmat is not supplied (default: 1).
    #' @param report A boolean value to create a HTML markdown report (default: \code{FALSE}). If \code{FALSE} a nested list of the plots and data is returned.
    #' @param filename A character to name the HTML report to be saved in the current working directory (default: \code{paste0(getwd(), "/report.html")}). The \code{getwd()} is required for rmarkdown to save it in the right path.
    #' @importFrom patchwork plot_layout wrap_plots
    #' @return List of plots/data or rendered HTML report
    autoFlow = function(feature_contrast = "FEATURE_ID",
                        feature_filter = NULL,
                        feature_ranks = NULL,
                        distance_metrics = c("bray"),
                        distmat = NULL,
                        weighted = TRUE,
                        pvalue.threshold = 0.05,
                        logfold.threshold = 1,
                        abundance.threshold = 0.01,
                        perm = 999,
                        threads = 1,
                        report = TRUE,
                        filename = paste0(getwd(), "/report.html")
                      ) {
    ## Error handling
    #--------------------------------------------------------------------#

    if (!is.character(filename) && length(filename) != 1)
      cli::cli_abort("{.val {filename}} needs to be a character with a length of 1")
      
    if (!is.character(feature_contrast) && length(feature_contrast) != 1) {
      cli::cli_abort("{.val {feature_contrast}} needs to be a character with a length of 1")
    } else if (!column_exists(feature_contrast, private$.featureData)) {
      cli::cli_abort("{.val {feature_contrast}} does not exist in {.field featureData}!")
    }

    if (!is.null(distmat) && !is.character(distmat) && length(distmat) != 1) {
      cli::cli_abort("{.arg distmat} needs to be a character with a length of 1")
    
      if (!file.exists(distmat))
        cli::cli_abort("{.arg distmat} does not exists!")
    }

    ## MAIN
    #--------------------------------------------------------------------#
    is_empty = function(obj) {
      if (length(obj) == 0) {
        return(NULL)
      } else if (length(obj) > 0) {
        keep_cells <- sapply(obj, function(x) is.null(x))
        obj <- obj[!keep_cells]
        if (length(obj) == 0) {
          return(NULL)
        } else return(obj)
      }
    }

    # Creates empty plots and data list
    plots <- list()
    data <- list()
    
    # Save omics class components
    .countData <- private$.countData
    .featureData <- private$.featureData
    .metaData <- private$.metaData
    .treeData <- private$.treeData

    # restore on error
    on.exit({
      private$.countData <- .countData
      private$.featureData <- .featureData
      private$.metaData <- .metaData
      private$.treeData <- .treeData
    }, add = TRUE)

    # Collect columns: CONTRAST_ and VARIABLE_
    metacols <- colnames(private$.metaData)

    CONTRAST_data <- private$.metaData[, .SD, .SDcols = grepl("CONTRAST_", metacols)]
    CONTRAST_names <- colnames(CONTRAST_data)

    VARIABLE_data <- private$.metaData[, .SD, .SDcols = grepl("VARIABLE_", metacols)]
    VARIABLE_names <- colnames(VARIABLE_data)

    if (ncol(CONTRAST_data) == 0)
      cli::cli_abort("No columns with prefix {.val CONTRAST} found.. Did you forgot to add a prefix?")

    #---------------------------------------------#
    # Perform standard visualizations             #
    #---------------------------------------------#
    #
    # CONTRAST
    #
    feature_nrow <- length(feature_contrast)
    CONTRAST_ncol <- length(CONTRAST_data)
    VARIABLE_ncol <- length(VARIABLE_data)

    # Standard rank stats
    if (!is.null(feature_ranks)) {
      plots$rankstat_plot <- self$rankstat(feature_ranks)
    }

    # Main loop
    if (CONTRAST_ncol > 0) {

      # Load custom distance matrix if supplied
      if (!is.null(distmat)) {
        distmat <- private$check_matrix(filepath = distmat)
        distmat <- distmat[private$.metaData[[private$.sample_id]], private$.metaData[[private$.sample_id]]]
      }

      # Initialize plot containers
      composition_plots <- matrix(list(), CONTRAST_ncol, feature_nrow)
      Log2FC_plots <- matrix(list(), CONTRAST_ncol, feature_nrow)
      alpha_div_plots <- list()
      metrics_nrow <- length(distance_metrics)
      pcoa_plots <- matrix(list(), CONTRAST_ncol, metrics_nrow)

      # Initialize data containers
      composition_data <- matrix(list(), CONTRAST_ncol, feature_nrow)
      Log2FC_data <- matrix(list(), CONTRAST_ncol, feature_nrow)
      alpha_div_data <- list()
      pcoa_data <- matrix(list(), CONTRAST_ncol, metrics_nrow)

      for (i in 1:CONTRAST_ncol) {
        col_name <- CONTRAST_names[i]
        conditions <- NULL
        cli::cli_alert_info(paste0("Processing ... column: ", col_name, " \n"))

        #--------------------------------------------------------------------#
        ## Alpha diversity
        #--------------------------------------------------------------------#
        res <- tryCatch(
          {
            # Default attempt
            self$alpha_diversity(
              col_name = col_name,
              metric = "shannon",
              paired = ifelse(!is.null(private$.samplepair_id), TRUE, FALSE)
            )
          },
          error = function(e) {
            cli::cli_alert_warning("{.arg alpha_diversity} with {.val paired=TRUE} failed. Retrying with {.val paired=FALSE}.")

          # Retry with paired = FALSE
          res2 <- tryCatch(
            self$alpha_diversity(
              col_name = col_name,
              metric = "shannon",
              paired = FALSE
            ),
            error = function(e2) {
              cli::cli_alert_info("Skipping {.arg alpha_diversity}, which failed due to an error: {.val {e2}}.")
              NULL
              }
            )
            res2
          }
        )

        if (!is.null(res)) {
          ## Save plots & data
          alpha_div_plots[[i]] <- res$plot
          alpha_div_data[[i]] <- list(data = res$data, stats = res$stats)
          
          ### Identify significant groups for composition plots & volcano plots
          signif_pairs <- res$stats[res$stats$p.adj < pvalue.threshold, ][c("group1", "group2")]
          if (nrow(signif_pairs) > 0)
            conditions <- signif_pairs
        }

        #--------------------------------------------------------------------#
        ## Beta diversity
        #--------------------------------------------------------------------#

        for (j in 1:metrics_nrow) {
          if (inherits(distmat, "Matrix")) {
            res <- self$ordination(
              distmat = distmat,
              method = "pcoa",
              perm = perm,
              group_by = col_name
              )
          } else {
            res <- self$ordination(
              metric = distance_metrics[j],
              method = "pcoa",
              group_by = col_name,
              weighted = weighted,
              perm = perm,
              threads = threads
              )
          }
          
          ## Save plots and identify significant groups for composition plots & volcano plots
          signif_pairs <- res$anova_data[res$anova_data$p.adj < pvalue.threshold, ]
          if (nrow(signif_pairs) > 0) {
            pairs_split <- strsplit(as.character(signif_pairs$pairs), " vs ")
            
            # Create group1 and group2 columns from split
            signif_pairs$group1 <- sapply(pairs_split, `[`, 1)
            signif_pairs$group2 <- sapply(pairs_split, `[`, 2)
            
            signif_pairs <- signif_pairs[c("group1", "group2")]
            
            conditions <- combine_conditions(conditions, signif_pairs)
          }
          
          ### Store plot and data
          pcoa_plots[[i, j]] <- patchwork::wrap_plots(res[c("scree_plot", "anova_plot", "scores_plot")],
                                                      nrow = 1) +
            patchwork::plot_layout(widths = c(rep(5, 3)),
                                   guides = "collect")
          pcoa_data[[i, j]] <- list(
            stats = res$anova_data,
            dist_mat = res$dist,
            pcs = res$pcs
          )

          # Creates temporary plot results for NMDS
          if (inherits(distmat, "Matrix")) {
            res <- self$ordination(
              distmat = distmat,
              method = "nmds",
              group_by = col_name,
              perm = perm
              )
          } else {
            res <- self$ordination(
              metric = distance_metrics[j],
              method = "nmds",
              group_by = col_name,
              weighted = weighted,
              perm = perm,
              threads = threads
              )
          }

          ## Save plots and identify significant groups for composition plots & volcano plots
          signif_pairs <- res$anova_data[res$anova_data$p.adj < pvalue.threshold, ]
          if (nrow(signif_pairs) > 0) {
            pairs_split <- strsplit(as.character(signif_pairs$pairs), " vs ")
            
            # Create group1 and group2 columns from split
            signif_pairs$group1 <- sapply(pairs_split, `[`, 1)
            signif_pairs$group2 <- sapply(pairs_split, `[`, 2)
            
            signif_pairs <- signif_pairs[c("group1", "group2")]
            
            conditions <- combine_conditions(conditions, signif_pairs)
          }      
        }
      
        #--------------------------------------------------------------------#
        ## Feature composition & FOLDCHANGE
        #--------------------------------------------------------------------#

        for (j in 1:feature_nrow) {
          if (!any(private$.countData@x < 0, na.rm = TRUE)) {
            # Creates composition long table
            res <- self$composition(
              feature_rank = feature_contrast[j],
              feature_filter = feature_filter,
              feature_top = 15,
              col_name = col_name
              )
            # Creates composition ggplot and stores plot with data
            composition_plots[[i, j]] <- composition_plot(
              data = res$data,
              palette = res$palette,
              feature_rank = feature_contrast[j],
              group_by = col_name
              )
            composition_data[[i, j]] <- list(data = res$data)
          } else {
            cli::cli_alert_info("Skipping {.arg composition} method due to the detection of negative values.")
          }
          
          if (!is.null(conditions) && nrow(conditions) > 0) {

            dfe <- tryCatch(
              {
              # Default attempt
              self$foldchange(
                feature_rank = feature_contrast[j],
                feature_filter = feature_filter,
                paired = ifelse(!is.null(private$.samplepair_id), TRUE, FALSE),
                condition.group = col_name,
                condition_A = c(conditions$group1),
                condition_B = c(conditions$group2),
                pvalue.threshold = pvalue.threshold,
                abundance.threshold = abundance.threshold,
                logfold.threshold = logfold.threshold
                )
              },
              error = function(e) {
                cli::cli_alert_warning("DFE with paired=TRUE failed. Retrying with paired=FALSE.")
                self$foldchange(
                  feature_rank = feature_contrast[j],
                  feature_filter = feature_filter,
                  paired = FALSE,
                  condition.group = col_name,
                  condition_A = c(conditions$group1),
                  condition_B = c(conditions$group2),
                  pvalue.threshold = pvalue.threshold,
                  abundance.threshold = abundance.threshold,
                  logfold.threshold = logfold.threshold
                  )
              }
            )
            # if (class(self)[1] %in% c("omics", "proteomics")) {
            #   dfe$data$p.adj <- p.adjust(p = dfe$data$pvalue_1, method = "fdr")
            #   dfe$volcano_plot <- volcano_plot(
            #     data = dfe$data,
            #     logfold_col = "Log2FC_1",
            #     pvalue_col = "p.adj",
            #     feature_rank = feature_contrast[j],
            #     abundance_col = "abun",
            #     label_A = conditions$group1,
            #     label_B = conditions$group2,
            #     pvalue.threshold = pvalue.threshold,
            #     abundance.threshold = abundance.threshold,
            #     logfold.threshold = logfold.threshold
            #   )
            # }
            Log2FC_plots[[i, j]] <- patchwork::wrap_plots(dfe$volcano_plot, nrow=1)
            Log2FC_data[[i, j]] <- list(data = dfe$data)
          }
        }
      }
      
      # Checks if plots aren't empty
      plots$alpha_div_plots <- is_empty(alpha_div_plots)
      plots$composition_plots <- is_empty(composition_plots)
      plots$Log2FC_plots <- is_empty(Log2FC_plots)
      plots$pcoa_plots <- is_empty(pcoa_plots)

      # Checks if data aren't empty
      data$composition_data <- is_empty(composition_data)
      data$Log2FC_data <- is_empty(Log2FC_data)
      data$alpha_div_data <- is_empty(alpha_div_data)
      data$pcoa_data <- is_empty(pcoa_data)
    }
    
    #--------------------------------------------------------------------#
    ## CREATING REPORT
    #--------------------------------------------------------------------#
    if (report) {
      # Locate the template Rmd and CSS within the installed package
      rmd_path <- system.file("report.Rmd", package = "OmicFlow")
      css_path <- system.file("styles.css", package = "OmicFlow")

      ## To bypass R CMD error and define for docker
      knit_dir <- dirname(filename)
      
      rmarkdown::render(
        input = rmd_path,
        output_file = filename,
        intermediates_dir = knit_dir,
        knit_root_dir = knit_dir,
        output_options = list(css = css_path)
      )
    } else {
      return(list(
        plots = plots,
        data = data
      ))
    }
  }
  ),
  private = list(

    # Private data fields
    #-------------------------#
    .countData = NULL,
    .featureData = NULL,
    .metaData = NULL,
    .treeData = NULL,
    .valid_schema = NULL,
    .feature_id = "FEATURE_ID",
    .sample_id = "SAMPLE_ID",
    .samplepair_id = "SAMPLEPAIR_ID",
    original_data = list(),

    # Function for synchronization of private data fields
    #---------------------------------------------------------#
    sync = function() {
      if (!is.null(private$.metaData)) {
        if (!column_exists(private$.sample_id, private$.metaData))
          return("{private$.sample_id} doesn't exist in {.field metaData}.")

        private$.metaData <- private$.metaData[, lapply(.SD, function(x) ifelse(x == "", NA, x)),
                                        .SDcols = colnames(private$.metaData)]

        colnames(private$.metaData) <- gsub("\\s+", "_", colnames(private$.metaData))  

        # Keep only common samples based on metaData
        if (!is.null(private$.countData)) {
          private$.countData <- private$check_matrix(private$.countData)
          common_samples <- base::intersect(private$.metaData[[ private$.sample_id ]], colnames(private$.countData))

          if (length(common_samples) == 0)
            cli::cli_abort("None SAMPLE_IDs are matching, check if {.val SAMPLE_ID} are matching the colnames in {.field countData}!")

          private$.countData <- private$.countData[, common_samples, drop = FALSE]
          private$.metaData <- private$.metaData[private$.metaData[[ private$.sample_id ]] %in% common_samples, ]
        }
      }

      if (!is.null(private$.featureData)) {
        if (!column_exists(private$.feature_id, private$.featureData))
          cli::cli_abort("{private$.feature_id} doesn't exist in {.field featureData}.")

        private$.featureData <- private$check_table(private$.featureData)
        colnames(private$.featureData) <- gsub("\\s+", "_", colnames(private$.featureData))

        # Keep only common tips based on treeData
        if (!is.null(private$.treeData)) {
          common_tips <- base::intersect(private$.treeData$tip.label, private$.featureData[[ private$.feature_id ]])

          if (length(common_tips) == 0)
            cli::cli_abort("None FEATURE_IDs are matching, check if {.val FEATURE_ID} matches the tip labels in {.field treeData}!")

          private$.treeData <- ape::keep.tip(private$.treeData, common_tips)
          private$.featureData <- private$.featureData[private$.featureData[[ private$.feature_id ]] %in% common_tips, ]
        }

        # Keep only common features based on countData
        if (!is.null(private$.countData)) {
          common_features <- base::intersect(private$.featureData[[ private$.feature_id ]], rownames(private$.countData))
          
          if (length(common_features) == 0)
            cli::cli_abort("None FEATURE_IDs are matching, check if {.val FEATURE_ID} matches the rownames in {.field countData}!")

          private$.featureData <- private$.featureData[private$.featureData[[ private$.feature_id ]] %in% common_features, ]
          private$.countData <- private$.countData[common_features, ]
          private$removeZeros()
        }
      } else if (!is.null(private$.countData)) {
        private$add_featureData()
        cli::cli_alert_warning("Placeholder {.field featureData} created.")
      }
    },
    removeZeros = function() {
      keep_cols <- base::diff(private$.countData@p) > 0
      keep_rows <- base::diff(t(private$.countData)@p) > 0

      private$.countData <- private$.countData[keep_rows, keep_cols]
      private$.metaData <- private$.metaData[keep_cols, ]
      private$.featureData <- private$.featureData[keep_rows]

      if (!is.null(private$.treeData))
        private$.treeData <- ape::keep.tip(private$.treeData, private$.featureData[[ private$.feature_id ]])
    },
    add_featureData = function() {
      private$.featureData <- data.table::data.table()
      countData_with_rownames <- rownames(private$.countData)

      if (is.null(countData_with_rownames)) {
        FEATURE_ID <- paste0("feature_", 1:nrow(private$.countData))
        private$.featureData <- private$.featureData[, (private$.feature_id) := FEATURE_ID]
        rownames(private$.countData) <- FEATURE_ID
      } else {
        private$.featureData <- private$.featureData[, (private$.feature_id) := countData_with_rownames]
      }          
    },
    # Checks & loads input table/filepath
    #--------------------------------------#
    check_table = function(data) {
    if (is.character(data) && length(data) == 1 && file.exists(data))
      return(data.table::fread(data, header = TRUE))

    if (inherits(data, "data.table"))
      return(data)

    if (is.data.frame(data))
      return(data.table::as.data.table(data))

    cli::cli_abort("Input must be an existing {.val filepath}, {.cls data.frame} or {.cls data.table}.")
  },

  # Checks & loads input matrix/filepath
  #--------------------------------------#
  check_matrix = function(data) {
    if (is.character(data) && length(data) == 1 && file.exists(data)) {
      dt <- data.table::fread(data, header = TRUE)
      # Change character values to numeric
      for (col in names(dt)) {
        dt[is.na(get(col)), (col) := 0]
        dt[get(col) == "", (col) := 0]
      }

      # Removes rownames if present
      if (!is.null(dt$V1)) {
        dt_rownames <- dt$V1
        dt[, V1 := NULL]
      } else {
        dt_rownames <- NULL
      }
      # Convert to matrix format
      mat <- Matrix::Matrix(
        data = as.matrix(dt),
        dimnames = list(dt_rownames, colnames(dt))
      )
      
      # Return CsparseMatrix
      return(as(mat, "CsparseMatrix"))
    }

    if (inherits(data, "sparseMatrix"))
      return(data)

    if (is.matrix(data) || inherits(data, "denseMatrix"))
      return(as(data, "CsparseMatrix"))
      
    cli::cli_abort("Input must be an existing {.val filepath}, {.cls matrix} or {.cls Matrix}.")
    }
  )
)

test_that("Testing Log2 Foldchanges", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  suppressWarnings(
    dfe_ungrouped <- taxa$foldchange(
      feature_rank = "Genus",
      feature_filter = c("uncultured"),
      paired = FALSE,
      normalize = TRUE,
      condition.group = "CONTRAST_sex",
      condition_A = c("male"),
      condition_B = c("female")
    ))

  suppressWarnings(
    dfe_grouped <- taxa$foldchange(
      feature_rank = "Genus",
      feature_filter = c("uncultured"),
      paired = FALSE,
      normalize = TRUE,
      group_by = "CONTRAST_sex",
      condition.group = "treatment",
      condition_A = c("tumor"),
      condition_B = c("healthy")
    ))

  skip_if(grepl("devel", R.version$status)) # Due to numerical differences in p-value
  expect_snapshot(dfe_ungrouped$data[, .SD, .SDcols = colnames(dfe_ungrouped$data)[!grepl("pvalue", colnames(dfe_ungrouped$data))]])
  expect_snapshot(dfe_grouped$data[, .SD, .SDcols = colnames(dfe_grouped$data)[!grepl("pvalue", colnames(dfe_grouped$data))]])
})

test_that("Testing Rankstat", {
  # Loading hdf5 format
  taxa_hdf5 <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  expect_no_error(taxa_hdf5$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species")))
  expect_no_error(taxa_hdf5$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"), unique=TRUE))
})

test_that("Testing transformations of data", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  # Perform log transformation
  taxa$scale(transform = log2)
  expect_snapshot(as.vector(taxa$countData[, 1]))
  
  # Perform sqrt transformation
  taxa$reset()
  taxa$scale(transform = sqrt)
  expect_snapshot(as.vector(taxa$countData[, 1]))
  
  # Perform clr standardisation
  taxa$reset()
  taxa$scale(method = "clr")
  expect_snapshot(as.vector(taxa$countData[, 1]))
  
  # Perform tss normalisation
  taxa$reset()
  taxa$scale(method = "tss")
  expect_snapshot(as.vector(taxa$countData[, 1]))
})

test_that("Writes biom in expected format", {
    output_file <- paste0(tempdir(),"/test.biom")

    taxa <- metagenomics$new(
        biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
        metaData = "input/metagenomics/metadata.tsv",
        treeData = "input/metagenomics/rooted_tree.newick"
    )
    taxa$write_biom(filename = output_file)

    expect_true(file.exists(output_file))
    
    taxa1 <- metagenomics$new(
        biomData = output_file,
        metaData = "input/metagenomics/metadata.tsv",
        treeData = "input/metagenomics/rooted_tree.newick"
    )
    
    expect_equal(taxa$print(), taxa1$print())
    file.remove(output_file)
})

test_that("Testing autoFlow on metagenomics data", {
    taxa <- metagenomics$new(
        biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
        metaData = "input/metagenomics/metadata.tsv",
        treeData = "input/metagenomics/rooted_tree.newick"
    )
  
    suppressWarnings(
        taxa_autoflow <- taxa$autoFlow(
            feature_contrast = c("Phylum", "Family", "Genus"),
            pvalue.threshold = 1,
            report = FALSE
        )
    )
    expect_true(length(taxa_autoflow$plots) == length(taxa_autoflow$data))
})

test_that("Testing autoFlow on proteomics data", {
    prot <- proteomics$new(
        metaData = "input/proteomics/metadata.csv",
        countData = "input/proteomics/counts.csv",
        treeData = "input/proteomics/tree.newick"
    )
    prot$scale(method = "clr")

    suppressWarnings(
        prot_autoflow <- prot$autoFlow(
            pvalue.threshold = 1,
            distance_metrics = "euclidean",
            report = FALSE,
        )
    )
    expect_true(length(prot_autoflow$plots) == length(prot_autoflow$data))
})

test_that("Testing Alpha diversity", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  ## Testing Shannon
  res_shannon <- suppressWarnings(
    taxa$alpha_diversity(
      metric = "shannon",
      col_name = "CONTRAST_sex",
    ))

  expect_snapshot(res_shannon$data)
  expect_snapshot(res_shannon$stats)

  # with stratification
  res_grouped_shannon <- suppressWarnings(
    taxa$alpha_diversity(
      metric = "shannon",
      col_name = "CONTRAST_sex",
      group_by = "treatment"
    ))

  expect_snapshot(res_grouped_shannon$data)
  expect_snapshot(res_grouped_shannon$stats)
  
  ## Testing inverse simpson
  res_invsimpson <- suppressWarnings(
    taxa$alpha_diversity(
      metric = "invsimpson",
      col_name = "CONTRAST_sex",
    ))
  
  expect_snapshot(res_invsimpson$data)
  expect_snapshot(res_invsimpson$stats)
  
  ## Testing simpson
  res_simpson <- suppressWarnings(
    taxa$alpha_diversity(
      metric = "simpson",
      col_name = "CONTRAST_sex",
    ))
  
  expect_snapshot(res_simpson$data)
  expect_snapshot(res_simpson$stats)
  
})

test_that("Testing subsetting of data", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  # Perform feature subset
  taxa$feature_subset(Kingdom == "Bacteria")
  expect_snapshot(taxa)
  
  # In between reset
  taxa$reset()
  
  # Perform metadata subset
  taxa$sample_subset(treatment == "tumor")
  expect_snapshot(taxa)

  # active binding subset
  taxa$reset()
  taxa$featureData <- taxa$featureData[1:200, ]
  expect_snapshot(taxa)

  taxa$reset()
  taxa$countData <- taxa$countData[1:50, ]
  expect_snapshot(taxa)
})

test_that("Testing Compositional data", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  taxa$scale(method = "tss")
  
  res <- taxa$composition(
    feature_rank = "Genus",
    feature_filter = c("uncultured"),
    col_name = "CONTRAST_sex",
    feature_top = 10
  )

  expect_snapshot(res$data)
  expect_snapshot(res$palette)

  # Testing composition_plot
  expect_no_error(composition_plot(
    data = res$data, 
    palette = res$palette, 
    feature_rank = "Genus", 
    group_by = "CONTRAST_sex", 
    title_name = "Awesome title"
  ))
  expect_error(composition_plot(data = res$data, palette = res$palette, feature_rank = "Species"))
  expect_error(composition_plot(data = res$data, palette = res$palette, feature_rank = "Genus", group_by = "treatment"))
  expect_error(composition_plot(data = res$data, palette = res$palette, feature_rank = "Genus", group_by = 1))
  expect_error(composition_plot(data = res$data, palette = c(1,2,3,4), feature_rank = "Genus"))
  expect_error(composition_plot(data = as.matrix(res$data), palette = res$palette, feature_rank = "Genus"))
})

test_that("Testing UniFrac ordination", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  taxa$scale(method = "tss")
  
  res <- taxa$ordination(
    metric = "unifrac",
    method = "pcoa",
    group_by = "CONTRAST_sex",
    weighted = TRUE,
    threads = 1
  )
  
  expect_snapshot(res$anova_data)
  expect_snapshot(res$dist)
  expect_snapshot(res$pcs)
})

#' Converting a Matrix to data.table
#'
#' @description Wrapper function that converts a sparseMatrix to data.table
#'
#' @param x A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.
#' @return A \link[data.table]{data.table} class.
#' @export
matrix_to_dtable <- function(x) {
  if (inherits(x, "denseMatrix") || inherits(x, "matrix") || inherits(x, "sparseMatrix")) {
      return(data.table::data.table(as.matrix(x)))
  } else cli::cli_abort("Input isn't a {.cls matrix}, {.cls denseMatrix} or {.cls sparseMatrix}.")
}
#' Checks if column exists in table
#'
#' @description Mainly used within \link{omics} and other functions to check if given column name does exist in the table and is not completely empty (containing NAs).
#'
#' @param column A character of length 1.
#' @param table A \link[data.table]{data.table} or \link[base]{data.frame}.
#' @return A boolean value.
#' @export
column_exists <- function(column, table) {

  ## Error handling
  #--------------------------------------------------------------------#

  if (!is.character(column) && length(column) != 1)
    cli::cli_abort("{.val {column}} needs to contain characters with length of 1.")

  if (!inherits(table, "data.frame") && !inherits(table, "data.table"))
    cli::cli_abort("{.arg table} must be a {.cls data.frame} or {.cls data.table}.")

  ## MAIN
  #--------------------------------------------------------------------#

  valid_columns <- column[column %in% colnames(table)]

  if (length(valid_columns) == 0) {
    return(FALSE)
  }

  # For each existing column, check if it's *not entirely NA*
  columns_empty <- all(sapply(valid_columns, function(col) {
    any(!is.na(table[[col]]))
  }))

  return (length(valid_columns) == length(column) && columns_empty)
}

is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
  if (is.character(x)) {
    return(FALSE)
  } else {
    abs(x - round(x)) < tol
  }
}

combine_conditions <- function(condition1, condition2) {
  if (!is.null(condition1) && !is.null(condition2)) {
    if (!inherits(condition1, "data.frame") && !inherits(condition1, "data.table"))
      cli::cli_abort("{.arg condition1} must be a {.cls data.frame} or {.cls data.table}.")

    if (!inherits(condition2, "data.frame") && !inherits(condition2, "data.table"))
      cli::cli_abort("{.arg condition2} must be a {.cls data.frame} or {.cls data.table}.")
  }

  # Combine to strings for easy comparison
  cond1_str <- paste(
    pmin(condition1$group1, condition1$group2),
    pmax(condition1$group1, condition1$group2), 
    sep = "_")

  cond2_str <- paste(
    pmin(condition2$group1, condition2$group2),
    pmax(condition2$group1, condition2$group2), 
    sep = "_")

  # Extend new unique pairs
  new_pairs_idx <- !cond2_str %in% cond1_str

  if (any(new_pairs_idx)) {
    new_rows <- condition2[new_pairs_idx, ]
    updated_conditions <- rbind(condition1, new_rows)
  } else {
    updated_conditions <- condition1
  }

  return(updated_conditions)
}

test_that("Testing exported utils functions", {
    # Load test data
    taxa <- metagenomics$new(
        biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
        metaData = "input/metagenomics/metadata.tsv",
        treeData = "input/metagenomics/rooted_tree.newick"
    )

    # Testing `column_exists`
    expect_true(column_exists("CONTRAST_sex", taxa$metaData))
    expect_false(column_exists("features", taxa$metaData))

    # Testing `matrix_to_dtable`
    expect_no_error(matrix_to_dtable(taxa$countData))
    expect_error(matrix_to_dtable(dt_sparse))
})

test_that("Testing non-exported utils functions", {
    condition1 <- data.frame(
        group1 = c("A", "A", "B"),
        group2 = c("B", "C", "C")
    )

    condition2 <- data.frame(
        group1 = c("B", "D", "C", "E"),
        group2 = c("A", "B", "D", "A")
    )

    expect_no_error(combine_conditions(condition1, condition2))
    expect_error(combine_conditions(as.matrix(condition1), condition2))
})

test_that("Testing `diversity` on sparse data", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  expect_no_error(diversity(taxa$countData, metric = "shannon"))
  expect_error(diversity(x = matrix_to_dtable(taxa$countData), metric = "shannon"))
  expect_error(diversity(taxa$countData, metric = 5))
  expect_error(diversity(taxa$countData, metric = "shan"))
})

test_that("Testing `diversity` on dense data", {
  prot <- proteomics$new(
      metaData = "input/proteomics/metadata.csv",
      countData = "input/proteomics/counts.csv"
  )
  
  expect_no_error(diversity(prot$countData, metric = "shannon"))
})

test_that("Testing Hill numbers on sparse data", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  res_0 <- hill_taxa(x = taxa$countData, q = 0)
  res_1 <- hill_taxa(x = taxa$countData, q = 1)
  res_2 <- hill_taxa(x = taxa$countData, q = 2)
  
  expect_snapshot(cat(res_0))
  expect_snapshot(cat(res_1))
  expect_snapshot(cat(res_2))
  expect_error(hill_taxa(x = matrix_to_dtable(taxa$countData), q = 0))
  expect_error(hill_taxa(x = taxa$countData, q = 5))
})

test_that("Testing Hill numbers on dense data", {
  prot <- proteomics$new(
      metaData = "input/proteomics/metadata.csv",
      countData = "input/proteomics/counts.csv"
  )
  
  res_0 <- hill_taxa(x = prot$countData, q = 0)
  res_1 <- hill_taxa(x = prot$countData, q = 1)
  res_2 <- hill_taxa(x = prot$countData, q = 2)
  
  expect_snapshot(cat(res_0))
  expect_snapshot(cat(res_1))
  expect_snapshot(cat(res_2))
})

#' Create pairwise stats plot
#'
#' @description Creates a pairwise stats plot from \link{pairwise_adonis} or \link{pairwise_anosim} results. 
#' This function is built into the class \link{omics} with method \code{ordination()} and inherited by other omics classes, such as;
#' \link{metagenomics} and \link{proteomics}.
#'
#' @param data A \link[base]{data.frame} or \link[data.table]{data.table}.
#' @param stats_col A column name of a continuous variable.
#' @param group_col A column name of a categorical variable.
#' @param label_col A column name of a categorical variable to label the bars.
#' @param y_axis_title A character variable to name the Y - axis title (default: NULL).
#' @param plot_title A character variable to name the plot title (default: NULL).
#' @return A \link[ggplot2]{ggplot2} object to be further modified
#' @importFrom ggplot2 ggplot aes geom_bar geom_label theme_bw theme element_text labs
#' @examples 
#' library("ggplot2")
#' 
#' # Create random data
#' set.seed(42)
#' mock_data <- matrix(rnorm(15 * 10), nrow = 15, ncol = 10)
#' 
#' # Create euclidean dissimilarity matrix
#' mock_dist <- dist(mock_data, method = "euclidean")
#' 
#' # Define group labels, should be equal to number of columns and rows to dist
#' mock_groups <- rep(c("A", "B", "C"), each = 5)
#' 
#' # Compute pairwise adonis
#' adonis_res <- pairwise_adonis(x = mock_dist, 
#'                               groups = mock_groups, 
#'                               p.adjust.method = "bonferroni", 
#'                               perm = 99)
#' # Compute pairwise anosim
#' anosim_res <- pairwise_anosim(x = mock_dist, 
#'                               groups = mock_groups, 
#'                               p.adjust.method = "bonferroni", 
#'                               perm = 99)
#' 
#' # Visualize PERMANOVA pairwise stats
#' plot_pairwise_stats(data = adonis_res,
#'                     group_col = "pairs",
#'                     stats_col = "F.Model",
#'                     label_col = "p.adj",
#'                     y_axis_title = "Pseudo F test statistic",
#'                     plot_title = "PERMANOVA")
#' 
#' # Visualize ANOSIM pairwise stats
#' plot_pairwise_stats(data = anosim_res,
#'                     group_col = "pairs",
#'                     stats_col = "anosimR",
#'                     label_col = "p.adj",
#'                     y_axis_title = "ANOSIM R statistic",
#'                     plot_title = "ANOSIM")
#' @export

plot_pairwise_stats <- function(data,
                                stats_col,
                                group_col,
                                label_col,
                                y_axis_title=NULL,
                                plot_title=NULL) {

  ## Error handling
  #--------------------------------------------------------------------#

  if (!inherits(data, "data.frame") && !inherits(data, "data.table"))
    cli::cli_abort("Data must be a {.cls data.frame} or {.cls data.table}.")

  if (!is.character(stats_col) && length(stats_col) != 1) {
    cli::cli_abort("{.val {stats_col}} needs to contain characters with length of 1.")
  } else if (!column_exists(stats_col, data)) {
    cli::cli_abort("The {.val {stats_col}} column does not exist in the provided {.arg data}.")
  }

  if (!is.character(group_col) && length(group_col) != 1) {
    cli::cli_abort("{.val {group_col}} needs to contain characters with length of 1.")
  } else if (!column_exists(group_col, data)) {
    cli::cli_abort("The {.val {group_col}} column does not exist in the provided {.arg data}.")
  }

  if (!is.character(label_col) && length(group_col) != 1) {
    cli::cli_abort("{.val {label_col}} needs to contain characters with length of 1.")
  } else if (!column_exists(label_col, data)) {
    cli::cli_abort("The {.val {label_col}} column does not exist in the provided {.arg data}.")
  }

  if (!is.null(y_axis_title) && !is.character(y_axis_title))
    cli::cli_abort("{.val {y_axis_title}} needs to contain characters.")

  if (!is.null(plot_title) && !is.character(plot_title))
    cli::cli_abort("{.val {plot_title}} needs to contain characters.")

  ## MAIN
  #--------------------------------------------------------------------#

  return(
    data %>%
      ggplot(mapping=aes(x = base::get(group_col, data),
                         y = base::get(stats_col, data),
                         label = base::get(label_col, data))) +
      geom_bar(stat = "identity",
               fill = "blue") +
      geom_label(nudge_y = 0) +
      labs(title = plot_title,
           subtitle = paste0("Above each bar: ", label_col, " values"),
           x = "groups",
           y = y_axis_title) +
      theme_bw() +
      theme(axis.text = element_text(angle = 45,
                                     vjust = 1,
                                     hjust = 1))
  )
}

test_that("Testing `plot_pairwise_stats` error-handling", {
  set.seed(42)
  mock_data <- matrix(rnorm(15 * 10), nrow = 15, ncol = 10)
  mock_dist <- dist(mock_data, method = "euclidean")
  mock_groups <- rep(c("A", "B", "C"), each = 5)

  # Compute pairwise adonis
  adonis_res <- pairwise_adonis(
    x = mock_dist,
    groups = mock_groups, 
    p.adjust.method = "bonferroni", 
    perm = 99)


  expect_error(plot_pairwise_stats(
    data = as.matrix(adonis_res),
    group_col = "pairs",
    stats_col = "F.Model",
    label_col = "p.adj",
    y_axis_title = "Pseudo F test statistic",
    plot_title = "PERMANOVA"
  ))

  expect_error(plot_pairwise_stats(
    data = adonis_res,
    group_col = "pair",
    stats_col = "F.Model",
    label_col = "p.adj",
    y_axis_title = "Pseudo F test statistic",
    plot_title = "PERMANOVA"
  ))

  expect_error(plot_pairwise_stats(
    data = adonis_res,
    group_col = "pairs",
    stats_col = "mod",
    label_col = "p.adj",
    y_axis_title = "Pseudo F test statistic",
    plot_title = "PERMANOVA"
  ))

  expect_error(plot_pairwise_stats(
    data = adonis_res,
    group_col = "pairs",
    stats_col = "F.Model",
    label_col = "p_adj",
    y_axis_title = "Pseudo F test statistic",
    plot_title = "PERMANOVA"
  ))

  expect_error(plot_pairwise_stats(
    data = adonis_res,
    group_col = "pairs",
    stats_col = "F.Model",
    label_col = "p.adj",
    y_axis_title = 5,
    plot_title = "PERMANOVA"
  ))

  expect_error(plot_pairwise_stats(
    data = adonis_res,
    group_col = "pairs",
    stats_col = "F.Model",
    label_col = "p.adj",
    y_axis_title = "Pseudo F test statistic",
    plot_title = 10
  ))
})

#' Compute Bray-Curtis Dissimilarity from a Dense or Sparse Matrix.
#'
#' @description 
#' Calculates the Bray-Curtis dissimilarity of a Matrix pairwise for each column.
#' 
#' @details
#' The Bray-Curtis dissimilarity between two samples \eqn{A} and \eqn{B}, each of length \eqn{n}, is defined as:
#'
#' \eqn{d(A,B) = \frac{\sum_{i}^n |A_i - B_i|}{\sum_{i}^n (A_i + B_i)}}
#'
#' where \eqn{A_i} and \eqn{B_i} are the abundances of the \eqn{i}-th feature in sample \eqn{A} and \eqn{B}, respectively. 
#' When weighted is set to FALSE, counts are replaced by presence/absence data.
#'
#' @param x A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.
#' @param weighted A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).
#' @param threads A wholenumber, the number of threads to use in \link[RcppParallel]{setThreadOptions} (default: 1).
#' @return A column x column \link[stats]{dist} object.
#' @references
#' Bray, J.R. & Curtis, J.T. (1957) An Ordination of the Upland Forest Communities of Southern Wisconsin. Ecological Monographs, 27(4), 325–349.
#' @examples 
#' library("OmicFlow")
#'
#' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
#' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
#' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
#' tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")
#'
#' taxa <- metagenomics$new(
#'     metaData = metadata_file,
#'     countData = counts_file,
#'     featureData = features_file,
#'     treeData = tree_file
#' )
#'
#' taxa$feature_subset(Kingdom == "Bacteria")
#' taxa$scale(method = "tss")
#'
#' bray(taxa$countData)
#' @importFrom RcppParallel setThreadOptions
#' @importFrom Matrix sparseMatrix
#' @importFrom stats as.dist
#' @export

bray <- function(x, weighted = TRUE, threads = 1) {

    ## Error handling
    #--------------------------------------------------------------------#
    if (inherits(x, "denseMatrix") || inherits(x, "matrix") || inherits(x, "sparseMatrix")) {
        x <- as(x, "CsparseMatrix")
    } else cli::cli_abort("Input isn't a {.cls matrix}, {.cls denseMatrix} or {.cls sparseMatrix}.")

    if (!is.numeric(x@x))
        cli::cli_abort("Input data must be numeric.")

    if (!is.wholenumber(threads))
        cli::cli_abort("{.val {threads}} must be a whole number.")

    ## MAIN
    #--------------------------------------------------------------------#

    if (!weighted) x@x[] <- 1

    RcppParallel::setThreadOptions(numThreads = threads)
    out <- .Call('_OmicFlow_bray', PACKAGE = 'OmicFlow', x)

    col_names <- colnames(x)
    if (!is.null(col_names))
        dimnames(out) <- list(col_names, col_names)

    return(as.dist(out))
}

#' Pairwise anosim (ANOSIM) computation
#'
#' @description Computes pairwise \link[vegan]{anosim}, given a distance matrix and a vector of labels.
#' This function is built into the class \link{omics} with method \code{ordination()} and inherited by other omics classes, such as;
#' \link{metagenomics} and \link{proteomics}.
#'
#' @param x  A distance matrix in the form of \link[stats]{dist}.
#' Obtained from a dissimilarity metric, in the case of similarity metric please use \code{1-dist}
#' @param groups  A vector (column from a table) of labels.
#' @param metadata A data.table or data.frame of extra metadata for \code{perm_design} (default: NULL).
#' @param perm_design A function that takes a data.frame and constructs a permutation design with \link[permute]{how} (default: NULL).
#' @param p.adjust.method P adjust method see \link[stats]{p.adjust}
#' @param perm Number of permutations to compare against the null hypothesis of anosim (default: \code{perm=999}).
#' @seealso \link[vegan]{anosim}
#' @return A \link[base]{data.frame} of
#'  * pairs that are used
#'  * R2 of H_0
#'  * p value of F^p > F
#'  * p adjusted
#' @importFrom stats p.adjust.methods
#' @examples 
#' # Create random data
#' set.seed(42)
#' mock_data <- matrix(rnorm(15 * 10), nrow = 15, ncol = 10)
#' 
#' # Create euclidean dissimilarity matrix
#' mock_dist <- dist(mock_data, method = "euclidean")
#' 
#' # Define group labels, should be equal to number of columns and rows to dist
#' mock_groups <- rep(c("A", "B", "C"), each = 5)
#' 
#' # Compute pairwise anosim
#' result <- pairwise_anosim(x = mock_dist, 
#'                           groups = mock_groups, 
#'                           p.adjust.method = "bonferroni", 
#'                           perm = 99)
#' @export

pairwise_anosim <- function(x,
                            groups,
                            metadata = NULL,
                            perm_design = NULL,
                            p.adjust.method = "bonferroni",
                            perm = 999){

  ## Error handling
  #--------------------------------------------------------------------#

  if (!inherits(x, "dist"))
    cli::cli_abort("{.val x} must be a {.cls dist}")

  if (is.list(groups))
    cli::cli_abort("{.val groups} must be a {.cls vector} and not a {.cls list}.")
  
  if (!is.null(metadata) && !inherits(metadata, "data.frame") && !inherits(metadata, "data.table"))
    cli::cli_abort("{.val metadata} must be a {.cls data.frame} or {.cls data.table}.")
  
  if (!is.null(perm_design) && !is.function(perm_design))
    cli::cli_abort("{.val perm_design} must be a function.")

  if (!c(p.adjust.method %in% p.adjust.methods))
    cli::cli_abort("{.val {p.adjust.method}} is not a valid method. \nValid options: {.val {p.adjust.methods}}.")

  if (!is.wholenumber(perm))
    cli::cli_abort("{perm} needs to be an integer.")

  ## MAIN
  #--------------------------------------------------------------------#

  co <- utils::combn(unique(as.character(groups)), 2)
  n <- ncol(co)
  pairs <- vector(mode = "numeric", length = n)
  anosimR <- vector(mode = "numeric", length = n)
  p.value <- vector(mode = "numeric", length = n)

  for(i in 1:n){
    if(inherits(x, "dist")){
      rows_to_keep <- groups %in% co[, i]
      m <- as.matrix(x)[rows_to_keep, rows_to_keep]
    }

    # Apply permutation design
    if (!is.null(perm_design) && !is.null(metadata)) {
      sub_meta <- metadata[rows_to_keep, ]
      h1 <- perm_design(sub_meta)
      ano <- vegan::anosim(
        x = m,
        grouping = groups,
        permutations = h1
      )
    } else {
      ano <- vegan::anosim(
        x = m, 
        grouping = groups[rows_to_keep],
        permutations = perm
      )
    }
    
    pairs[i] <- paste(co[1, i],'vs',co[2, i])
    anosimR[i] <- ano$statistic
    p.value[i] <- ano$signif
  }
  p.adj <- p.adjust(p.value, method = p.adjust.method)
  pairw.res <- data.frame(pairs, anosimR, p.value, p.adj)
  return(pairw.res)
}

test_that("Testing pairwise_adonis & pairwise_anosim", {
  taxa <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )

  # Example test data
  set.seed(42)
  mock_data <- matrix(rnorm(15 * 10), nrow = 15, ncol = 10)
  mock_dist <- dist(mock_data, method = "euclidean")
  mock_groups <- rep(c("A", "B", "C"), each = 5)

  expect_error(pairwise_adonis(x = mock_dist, groups = mock_groups, p.adjust.method = "test", perm = 99))
  expect_error(pairwise_adonis(x = as.matrix(mock_dist), groups = mock_groups))
  expect_error(pairwise_adonis(x = mock_dist, groups = list(mock_groups)))
  expect_error(pairwise_adonis(x = mock_dist, groups = mock_groups, perm = 5.5))

  expect_error(pairwise_anosim(x = mock_dist, groups = mock_groups, p.adjust.method = "test", perm = 99))
  expect_error(pairwise_anosim(x = as.matrix(mock_dist), groups = mock_groups))
  expect_error(pairwise_anosim(x = mock_dist, groups = list(mock_groups)))
  expect_error(pairwise_anosim(x = mock_dist, groups = mock_groups, perm = 5.5))
})

#' Ordination plot
#'
#' @description Creates an ordination plot pre-computed principal components from \link[vegan]{wcmdscale}. 
#' This function is built into the class \link{omics} with method \code{ordination()} and inherited by other omics classes, such as;
#' \link{metagenomics} and \link{proteomics}.
#' 
#' @param data A \link[base]{data.frame} or \link[data.table]{data.table} of Principal Components as columns and rows as loading scores.
#' @param col_name A categorical variable to color the contrasts (e.g. "groups").
#' @param pair A vector of character variables indicating what dimension names (e.g. PC1, NMDS2).
#' @param dist_explained A vector of numeric values of the percentage dissimilarity explained for the dimension pairs, default is NULL.
#' @param dist_metric A character variable indicating what metric is used (e.g. unifrac, bray-curtis), default is NULL.
#' @return A \link[ggplot2]{ggplot2} object to be further modified
#' 
#' @importFrom ggplot2 ggplot aes .data theme_bw theme element_text stat_ellipse labs geom_point
#' 
#' @examples 
#' library(ggplot2)
#' 
#' # Mock principal component scores
#' set.seed(123)
#' mock_data <- data.frame(
#'   SampleID = paste0("Sample", 1:10),
#'   PC1 = rnorm(10, mean = 0, sd = 1),
#'   PC2 = rnorm(10, mean = 0, sd = 1),
#'   groups = rep(c("Group1", "Group2"), each = 5)
#' )
#' 
#' # Basic usage
#' ordination_plot(
#'   data = mock_data,
#'   col_name = "groups",
#'   pair = c("PC1", "PC2")
#' )
#' 
#' # Adding variance/dissimilarity explained.
#' ordination_plot(
#'   data = mock_data,
#'   col_name = "groups",
#'   pair = c("PC1", "PC2"),
#'   dist_explained = c(45, 22),
#'   dist_metric = "bray-curtis"
#' )
#' @export

ordination_plot <- function(data, 
                            col_name,
                            pair, 
                            dist_explained = NULL, 
                            dist_metric = NULL) {
  ## Error handling
  #--------------------------------------------------------------------#

  if (!inherits(data, "data.frame") && !inherits(data, "data.table"))
    cli::cli_abort("Data must be a {.cls data.frame} or {.cls data.table}.")

  if (!is.character(pair)) {
    cli::cli_abort("{.val {pair}} needs to be a characters {.cls vector}.")
  } else if (length(pair) != 2) {
    cli::cli_abort("{.val {pair}} needs to be a {.cls vector} of length 2.")
  }

  if (!is.character(col_name) && length(col_name) != 1) {
    cli::cli_abort("{.val {col_name}} needs to contain characters with length of 1.")
  } else if (!column_exists(col_name, data)) {
    cli::cli_abort("The {.val {col_name}} column does not exist in the provided {.arg data}.")
  }

  if (!is.null(dist_explained) && !is.numeric(dist_explained)) {
    cli::cli_abort("{.val {dist_explained}} needs to be a numeric {.cls vector}.")
  } else if (!is.null(dist_explained) && length(dist_explained) < 2) {
    cli::cli_abort("{.val {dist_explained}} needs to be a {.cls vector} of at least length is 2.")
  }

  if (!is.null(dist_metric) && !is.character(dist_metric) && length(dist_metric) != 1)
    cli::cli_abort("{.val {dist_metric}} needs to contain characters with length of 1.")

  ## MAIN
  #--------------------------------------------------------------------#

  if (!is.null(dist_metric))
    dist_metric <- paste0("Distance metric used: ", dist_metric)
  
  if (!is.null(dist_explained)) {
    x_label <- paste0(pair[1], " (", round(as.numeric(dist_explained[1]), 2), "%)")
    y_label <- paste0(pair[2], " (", round(as.numeric(dist_explained[2]), 2), "%)")
  } else {
    x_label <- paste0(pair[1])
    y_label <- paste0(pair[2])
  }

  data[[ col_name ]] <- as.factor(data[[ col_name ]])

  return(
    data %>%
      ggplot(mapping = aes(x = .data[[ pair[1] ]],
                           y = .data[[ pair[2] ]],
                           color = .data[[ col_name ]],
                           linetype = .data[[ col_name ]])) +
      geom_point(alpha = 5) +
      stat_ellipse(type = "t") +
      theme_bw() +
      theme(text=element_text(size=14),
            legend.text = element_text(size=12),
            legend.title = element_text(size=14),
            axis.text = element_text(size=12),
            axis.text.y = element_text(size=12),
            axis.text.x = element_text(size=12)
      ) +
      labs(title = dist_metric,
           subtitle = NULL,
           x = x_label,
           y = y_label
      )
  )
}

test_that("Testing `ordination_plot` error-handling", {
  set.seed(123)
  mock_data <- data.frame(
    SampleID = paste0("Sample", 1:10),
    PC1 = rnorm(10, mean = 0, sd = 1),
    PC2 = rnorm(10, mean = 0, sd = 1),
    groups = rep(c("Group1", "Group2"), each = 5)
  )

  expect_error(ordination_plot(
    data = as.matrix(mock_data),
    col_name = "groups",
    pair = c("PC1", "PC2"),
    dist_explained = c(45, 22),
    dist_metric = "bray-curtis"
  ))
  expect_error(ordination_plot(
    data = mock_data,
    col_name = "group",
    pair = c("PC1", "PC2"),
    dist_explained = c(45, 22),
    dist_metric = "bray-curtis"
  ))
  expect_error(ordination_plot(
    data = mock_data,
    col_name = "groups",
    pair = c("PC1"),
    dist_explained = c(45, 22),
    dist_metric = "bray-curtis"
  ))
  expect_error(ordination_plot(
    data = mock_data,
    col_name = "groups",
    pair = c("PC1", "PC2"),
    dist_explained = c(45),
    dist_metric = "bray-curtis"
  ))
})

#' Sub-class metagenomics
#'
#' @description This is a sub-class that is compatible to data obtained from either 16S rRNA marker-gene sequencing or shot-gun metagenomics sequencing.
#' It inherits all methods from the abstract class \link{omics} and only adapts the \code{initialize} function.
#' It supports BIOM format data (v2.1.0 from \url{http://biom-format.org/}) in both HDF5 and JSON format, also pre-existing data structures can be used or text files.
#' When omics data is very large, data loading becomes very expensive. It is therefore recommended to use the [`reset()`](#method-reset) method to reset your changes.
#' Every omics class creates an internal memory efficient back-up of the data, the resetting of changes is an instant process.
#' @seealso \link{omics}
#' @import R6 rhdf5 Matrix
#' @importFrom ape read.tree
#' @importFrom tools file_ext
#' @importFrom yyjsonr validate_json_file
#' @importFrom jsonlite read_json
#' @export

metagenomics <- R6::R6Class(
  classname = "metagenomics",
  cloneable = TRUE,
  inherit = omics,
  active = list(
    #' @field treeData A "phylo" class, see \link[ape]{as.phylo}.
    treeData = function(value) {
      # back-up
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      success <- FALSE
      on.exit({
        if (!success) {
          private$.countData <- .countData
          private$.featureData <- .featureData
          private$.metaData <- .metaData
          private$.treeData <- .treeData
        }
      }, add = TRUE)

      if (missing(value)) {
        success <- TRUE
        private$.treeData
      } else if (inherits(value, "phylo")) {
        private$.treeData <- value
        private$sync()
        self$print()
        success <- TRUE
        invisible(self)
      } else {
        cli::cli_abort("Data input must be {.cls phylo} like {.field treeData}.")
      }
    }
  ),
  public = list(
    #' @description
    #' Initializes the metagenomics class object with \code{metagenomics$new()}
    #' @param countData A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.
    #' @param featureData A path to an existing file, \link[data.table]{data.table} or data.frame.
    #' @param metaData A path to an existing file, \link[data.table]{data.table} or data.frame.
    #' @param treeData A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.
    #' @param biomData A path to an existing biom file, version 2.1.0 (http://biom-format.org/), see \link[rhdf5]{h5read}.
    #' @param feature_names A character vector to name the feature names that fit the supplied `featureData`.
    #' @return A new `metagenomics` object.
    initialize = function(countData = NULL,
                          metaData = NULL,
                          featureData = NULL,
                          treeData = NULL,
                          biomData = NULL,
                          feature_names = c("Kingdom", "Phylum", "Class", "Order", "Family", "Genus", "Species")) {

      super$initialize(countData = countData,
                       featureData = featureData,
                       metaData = metaData)

      if (!is.null(biomData)) {

        if (tools::file_ext(biomData) == "biom") {
          
          #---------------------#
          ###  biomData HDF5  ###
          #---------------------#

          if (rhdf5::H5Fis_hdf5(biomData)) {

            hdf5_contents <- data.table::data.table(rhdf5::h5ls(biomData))
            hdf5_contents[, content := paste(group, name, sep = "/")]

            expected_content <- c(
              "/observation/matrix/data",
              "/observation/matrix/indptr",
              "/observation/matrix/indices",
              "/observation/ids",
              "/sample/ids")

            missing <- base::setdiff(expected_content, hdf5_contents$content)

            if (length(missing) > 0 ) {
              cli::cli_abort(
                "Expected content is missing",
                "i" = "\n{ paste(missing, collapse = ',')}"
              )
            }

            # Checks if data contains any dimensions
            list_of_dimensions <- hdf5_contents$dim[grepl(paste(expected_content, collapse="|"), hdf5_contents$content)]
            if (!all(as.numeric(list_of_dimensions) > 0)) {
              cli::cli_abort(
                "Expected content does not contain any dimensions",
                "i" = "\n{ paste(expected_content, collapse = ',')}"
              )
            }

            # Loads data in memory
            private$.biomData <- rhdf5::h5read(biomData, "/", read.attributes = TRUE)
            private$construct_hdf5_featureData()
            private$construct_hdf5_countData()

            #---------------------#
            ###  biomData JSON  ###
            #---------------------#

          } else if (yyjsonr::validate_json_file(biomData)) {
            
            private$.biomData <- jsonlite::read_json(biomData)
            private$construct_json_featureData(feature_names)
            private$construct_json_countData()

          } else {
            cli::cli_abort("{.field biomData} could not be loaded. Not a valid JSON or HDF5 format!")
          }

          # Create placeholder featureData
          if (is.null(private$.featureData)) {
            FEATURE_ID <- paste0("feature_", 1:nrow(private$.countData))
            private$.featureData <- data.table::data.table()
            private$.featureData <- private$.featureData[, (private$.feature_id) := FEATURE_ID]
            rownames(private$.countData) <- private$.featureData[[ private$.feature_id ]]
            cli::cli_alert_warning("Created placeholder {.field featureData}.")
          }
          rownames(private$.countData) <- private$.featureData[[ private$.feature_id ]]
          data.table::setcolorder(
            x = private$.featureData,
            neworder = c(private$.feature_id, base::setdiff(colnames(private$.featureData), private$.feature_id))
          )
          private$.featureData <- private$.featureData[, 
            lapply(.SD, function(x) ifelse(x == "", NA, x)),
            .SDcols = colnames(private$.featureData)]
        }
      }

      #-------------------#
      ###   treeData    ###
      #-------------------#

      if (!is.null(treeData)) {
        if (is.character(treeData) && length(treeData) == 1 && file.exists(treeData)) {
          private$.treeData <- ape::read.tree(treeData)
          cli::cli_alert_success("{.field treeData} is loaded.")
        } else if (inherits(treeData, "phylo")) {
          private$.treeData <- treeData
          cli::cli_alert_success("{.field treeData} is loaded.")
        } else {
          cli::cli_alert_warning("The provided {.field treeData} could not be loaded. Make sure the tree is supported by {.fun ape::read.tree}")
        }

        # Aligning featureData and countData rows by tree tips
        private$.featureData <- private$.featureData[order(match(private$.featureData[[ private$.feature_id ]], private$.treeData$tip.label))]
        private$.countData <- private$.countData[private$.featureData[[ private$.feature_id ]], ]
      }

      #-------------------#
      ###     CLEANUP   ###
      #-------------------#

      cli::cli_alert_info("Final steps .. cleaning & creating back-up")

      # Removing prefix of taxonomic features
      private$.featureData <- private$.featureData[, lapply(.SD, function(x) gsub("^[dpcofgs]_{2}", "", x)),
                                           .SDcols = colnames(private$.featureData)]
      # Rename last column names by feature_names
      n_feature_names <- length(feature_names)
      n_cols_featureData <- ncol(private$.featureData)
      colnames(private$.featureData)[n_cols_featureData:(n_cols_featureData - n_feature_names + 1)] <- base::rev(feature_names)

      # Subsetting countData by metadata
      private$sync()
      self$print()

      # saves data for reset function
      private$original_data = list(
        counts = private$.countData,
        features = private$.featureData,
        metadata = private$.metaData,
        tree = private$.treeData
      )
    },
    #' @description
    #' Creates a BIOM file in HDF5 format of the loaded items via ['new()'](#method-new), which is compatible to the python biom-format version 2.1, see http://biom-format.org.
    #' @param filename A character variable of either the full path of filename of the biom file (e.g. `output.biom`)
    #' @examples
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #' tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")
    #'
    #' taxa <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file,
    #'  treeData = tree_file
    #' )
    #' 
    #' taxa$write_biom(filename = "output.biom")
    #' file.remove("output.biom")
    #'
    write_biom = function (filename) {

      res <- try(
        rhdf5::h5createFile(filename),
        silent = TRUE
      )
      if (!res) {
        cli::cli_abort("Can't create file {.filename {filename}}: {res}")
      }

      groups <- c(
        'observation',
        'observation/matrix',
        'observation/metadata',
        'observation/group-metadata',
        'sample',
        'sample/matrix',
        'sample/metadata',
        'sample/group-metadata'
      )

      for (group in groups)
        invisible(rhdf5::h5createGroup(filename, group))

      h5 <- try(
        rhdf5::H5Fopen(name = filename,
                       flags = 'H5F_ACC_RDWR',
                       native = TRUE),
        silent = TRUE
      )
      if (!inherits(h5, "H5IdComponent"))
        cli::cli_abort("Can't open HDF5 file {.filename {filename}}: {h5}")

      # convert countData to triplet matrix
      triplets <- Matrix::summary(private$.countData)

      #----------------------------#
      #       Add Attributes       #
      #----------------------------#
      rhdf5::h5writeAttribute(attr ="No Table ID",
                              h5obj = h5,
                              name = 'id')
      rhdf5::h5writeAttribute(attr = "OTU table",
                              h5obj = h5,
                              name = 'type')
      rhdf5::h5writeAttribute(attr = "Auto-generated biom file",
                              h5obj = h5,
                              name = 'comment')
      rhdf5::h5writeAttribute(attr = "http://biom-format.org",
                              h5obj = h5,
                              name = 'format-url')
      rhdf5::h5writeAttribute(attr = as.integer(c(2,1,0)),
                              h5obj = h5,
                              name = 'format-version')
      rhdf5::h5writeAttribute(attr = paste(Sys.Date()),
                              h5obj = h5,
                              name = 'creation-date')
      rhdf5::h5writeAttribute(attr = dim(private$.countData),
                              h5obj = h5,
                              name = 'shape')
      rhdf5::h5writeAttribute(attr = length(triplets),
                              h5obj = h5,
                              name = 'nnz')
      rhdf5::h5writeAttribute(attr = paste("OmicFlow", utils::packageVersion("OmicFlow")),
                              h5obj = h5,
                              name = 'generated-by')

      #----------------------------#
      #       Counts by row        #
      #----------------------------#
      x <- matrix(c(triplets$i - 1, triplets$j - 1, triplets$x), ncol = 3)
      x <- x[order(x[,1]),,drop=FALSE]

      counts_per_row <- base::tabulate(x[,1] + 1L, nbins = nrow(private$.countData))
      indptr <- c(0L, base::cumsum(counts_per_row))

      rhdf5::h5writeDataset(obj = base::rownames(private$.countData),
                            h5loc = h5,
                            name = 'observation/ids')
      rhdf5::h5writeDataset(obj = as.numeric(x[,3]),
                            h5loc = h5,
                            name = 'observation/matrix/data')
      rhdf5::h5writeDataset(obj = as.integer(x[,2]),
                            h5loc = h5,
                            name = 'observation/matrix/indices')
      rhdf5::h5writeDataset(obj = as.integer(indptr),
                            h5loc = h5,
                            name = 'observation/matrix/indptr')

      #----------------------------#
      #       Counts by column     #
      #----------------------------#
      x <- x[order(x[,2]),,drop=FALSE]
      counts_per_col <- base::tabulate(x[,2] + 1L, nbins = ncol(private$.countData))
      indptr <- c(0L, cumsum(counts_per_col))

      rhdf5::h5writeDataset(obj = base::colnames(private$.countData),
                            h5loc = h5,
                            name = 'sample/ids')
      rhdf5::h5writeDataset(obj = as.numeric(x[,3]),
                            h5loc = h5,
                            name = 'sample/matrix/data')
      rhdf5::h5writeDataset(obj = as.integer(x[,1]),
                            h5loc = h5,
                            name = 'sample/matrix/indices')
      rhdf5::h5writeDataset(obj = as.integer(indptr),
                            h5loc = h5,
                            name = 'sample/matrix/indptr')

      #----------------------------#
      #       Add Taxonomy         #
      #----------------------------#
      if (all(dim(private$.featureData)) > 0) {
        h5path <- 'observation/metadata/taxonomy'
        features <- as.matrix(private$.featureData[, .SD, .SDcols = !c(private$.feature_id)])
        dimnames(features) <- list(NULL, NULL)
        rhdf5::h5writeDataset(obj = features,
                              h5loc = h5,
                              name = h5path)
      }

      # Close hdf5 file connection
      rhdf5::H5Fclose(h5)
    },
    #' @description
    #' Differential feature expression (DFE) Total Sample Sum (TSS) transformed values for both paired and non-paired test.
    #' 
    #' The function performs feature agglomeration, subsetting to remove NAs in `condition.group`, finding samplepairs and finally feature scaling prior to fold-change computation. 
    #' Based on the `transform` method, fold-changes will be computed either via subtraction or division.
    #' 
    #' @param feature_rank A character or vector of characters in the `featureData` to aggregate via [`feature_merge()`](#method-feature_merge) (default: \code{"FEATURE_ID"}).
    #' @param feature_filter A character or vector of characters to remove features via regex pattern (default: \code{NULL}).
    #' @param paired A boolean value, the paired is only applicable when a `SAMPLEPAIR_ID` column exists within the `metaData`. See \link[stats]{wilcox.test} and [`samplepair_subset()`](#method-samplepair_subset).
    #' @param condition.group A character variable of an existing column name in `metaData`, wherein the conditions A and B are located.
    #' @param group_by A character variable of an existing column in `metaData` to split the table in chunks prior to fold-change computation (default: NULL).
    #' @param condition_A A character value or vector of characters.
    #' @param condition_B A character value or vector of characters.
    #' @param normalize A boolean value wether to normalize via Total Sample Sums (TSS) or not (default: \code{FALSE}).
    #' @param pvalue.threshold A numeric value used as a p-value threshold to label and color significant features (default: 0.05).
    #' @param logfold.threshold A numeric value used as a fold-change threshold to label and color significantly expressed features (default: 0.06).
    #' @param abundance.threshold A numeric value used as an abundance threshold to size the scatter dots based on their mean abundance (default: 0.01).
    #' @examples
    #' library("ggplot2")
    #' library("OmicFlow")
    #'
    #' metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
    #' counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
    #' features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
    #'
    #' obj <- metagenomics$new(
    #'  metaData = metadata_file,
    #'  countData = counts_file,
    #'  featureData = features_file
    #' )
    #'
    #' dfe <- obj$foldchange(feature_rank = "Genus",
    #'                       paired = FALSE,
    #'                       condition.group = "treatment",
    #'                       condition_A = c("healthy"),
    #'                       condition_B = c("tumor"))
    #'
    #' @returns
    #'  * `data` A long \link[data.table]{data.table} table.
    #'  * `volcano_plot` A \link[ggplot2]{ggplot} object.
    #'  * `A` A \link[data.table]{data.table} table for (each) condition A.
    #'  * `B` A \link[data.table]{data.table} table for (each) condition B.
    #'
    #' @seealso \link{volcano_plot}
    foldchange = function(
      condition.group,
      condition_A,
      condition_B,
      group_by = NULL,
      feature_rank = "FEATURE_ID",
      feature_filter = NULL,
      paired = FALSE,
      normalize = FALSE,
      pvalue.threshold = 0.05,
      logfold.threshold = 0.06,
      abundance.threshold = 0
      ) {

      ## Error handling
      #--------------------------------------------------------------------#

      if (!is.character(feature_rank) && length(feature_rank) != 1) {
        cli::cli_abort("{.val {feature_rank}} needs to be a character with a length of 1")
      } else if (!column_exists(feature_rank, private$.featureData)) {
        cli::cli_abort("The {.val {feature_rank}} column does not exist in the {.field featureData}.")
      }

      if (!is.character(condition.group) && length(condition.group) != 1) {
        cli::cli_abort("{.val {condition.group}} needs to be a character with a length of 1")
      } else if (!column_exists(condition.group, private$.metaData)) {
        cli::cli_abort("{.val {condition.group}} does not exist in the {.field metaData} or is empty.")
      }
      if (!is.character(condition_A))
        cli::cli_abort("{.val {condition_A}} needs to be a character.")

      if (!is.character(condition_B))
        cli::cli_abort("{.val {condition_B}} needs to be a character.")

      if (!is.numeric(pvalue.threshold))
        cli::cli_abort("{.val {pvalue.threshold}} need to be numeric.")

      if (!is.numeric(logfold.threshold))
        cli::cli_abort("{.val {logfold.threshold}} need to be numeric.")

      if (paired && is.null(private$.samplepair_id)) {
        cli::cli_alert_warning("Paired is set to {.val {paired}} but {.arg SAMPLEPAIR_ID} does not exist in the {.field metaData}.\n Differential feature analysis will continue now with paired set to {.val FALSE}!")
        paired <- FALSE
      }

      if (!is.null(group_by)) {
        if (!is.character(group_by) && length(group_by) != 1) {
          cli::cli_abort("{.val {group_by}} needs to contain characters with length of 1.")
        } else if (!column_exists(group_by, private$.metaData)) {
          cli::cli_abort("The {.val {group_by}} column does not exist in the {.field metaData}.")
        }
      }

      ## MAIN
      #--------------------------------------------------------------------#

      # Final output
      output <- list()

      # Copies object to prevent modification of omics class components
      .countData <- private$.countData
      .featureData <- private$.featureData
      .metaData <- private$.metaData
      .treeData <- private$.treeData

      # restore on error
      on.exit({
        private$.countData <- .countData
        private$.featureData <- .featureData
        private$.metaData <- .metaData
        private$.treeData <- .treeData
      }, add = TRUE)

      # normalization if applicable
      if (normalize)
        self$scale(method = "tss")

      # Subset by missing values
      self$removeNAs(condition.group)

      # Subset by samplepair completion
      if (paired && !is.null(private$.samplepair_id))
        self$samplepair_subset()
      
      # Agglomerate taxa by feature rank and filter unwanted taxa
      self$feature_merge(feature_rank = feature_rank,
                         feature_filter = feature_filter)
      
      # Extract mean abundance
      abun <- as.matrix(Matrix::rowMeans(private$.countData))
      rownames(abun) <- private$.featureData[[ feature_rank ]]

      # Get data.table format abundances
      dt <- matrix_to_dtable(private$.countData)[, (feature_rank) := private$.featureData[[feature_rank]]]

      # Compute 2-fold expression based on (un)paired samples
      # Supports multiple inputs for A and B.
      tmp_dt <- data.table::copy(dt)

      # Apply `group_by`
      if (!is.null(group_by)) {
        chunks <- base::split(private$.metaData, by = group_by)
      } else {
        chunks <- list(all = private$.metaData)
      }
      group_names <- names(chunks)

      # subset feature labels before removing them
      feature_labels <- tmp_dt[[ feature_rank ]]
      tmp_dt <- tmp_dt[, .SD, .SDcols = !c(feature_rank)]

      # Create data.tables for results
      foldchange_dt <- data.table::data.table(feature_rank = feature_labels)
      colnames(foldchange_dt) <- feature_rank

      for (group_name in group_names) {
        condition_labels <- chunks[[group_name]][[condition.group]]

        for (i in seq_along(condition_A)) {
          # Subset by condition_A value
          dt_A <- tmp_dt[, .SD, .SDcols = colnames(tmp_dt)[condition_labels %in% condition_A[i]]]
          dt_B <- tmp_dt[, .SD, .SDcols = colnames(tmp_dt)[condition_labels %in% condition_B[i]]]

          # save intermediate condition tables
          output[[paste0(group_name, "_", condition_A[i])]] <- dt_A
          output[[paste0(group_name, "_", condition_B[i])]] <- dt_B

          # Convert to dense matrix
          mat_A <- as.matrix(dt_A)
          mat_B <- as.matrix(dt_B)

          # Feature means per condition
          row_means_A <- Matrix::rowMeans(mat_A)
          row_means_B <- Matrix::rowMeans(mat_B)

          # Empty vector
          result <- numeric(length(row_means_A))
          max_val <- base::max(mat_A)

          # Find zero's to prevent Inf
          both_zero <- row_means_A == 0 & row_means_B == 0
          row_means_A_zero <- row_means_A == 0 & row_means_B != 0
          row_means_B_zero <- row_means_A != 0 & row_means_B == 0
          both_non_zero <- row_means_A != 0 & row_means_B != 0

          # Compute log2 fold change
          result[both_zero] <- 0
          result[row_means_A_zero] <- row_means_A[row_means_A_zero] - log2(row_means_B[row_means_A_zero])
          result[row_means_B_zero] <- log2(row_means_A[row_means_B_zero]) - row_means_B[row_means_B_zero]
          result[both_non_zero] <- log2(row_means_A[both_non_zero]) - log2(row_means_B[both_non_zero])

          # Reverse flipped values with zero's based on max_val
          if (max_val < 1.0) {
            result[row_means_A_zero] <- result[row_means_A_zero] * -1
            result[row_means_B_zero] <- result[row_means_B_zero] * -1
          }

          # Combines to final foldchange data table
          foldchange_dt <- cbind(foldchange_dt, result)
          result_col_title <- paste0(condition_A[i], "_vs_", condition_B[i], "_in_", group_name)
          colnames(foldchange_dt)[grepl("result", colnames(foldchange_dt))] <- paste0("Log2FC_", result_col_title)

          for (k in seq_along(feature_labels)) {
            # save p-values in data.table
            suppressWarnings(
              foldchange_dt[
                k, (paste0("pvalue_", result_col_title)) := stats::wilcox.test(
                  mat_A[k, ], mat_B[k, ],
                  correct = TRUE,
                  paired = paired
                  )$p.value
                ]
            )
          }
        }
      }

      # Add abundance, and save data as output list
      dfe <- foldchange_dt[, "abun" := abun]
      output$data <- dfe

      #----------------------#
      # Visualization        #
      #----------------------#

      # Create & save volcano plot
      colnames_dfe <- colnames(dfe)
      diff_columns <- colnames_dfe[grepl("Log2FC", colnames_dfe)]
      pvalue_columns <- colnames_dfe[grepl("pvalue", colnames_dfe)]
      n_diff_columns <- length(diff_columns)

      output$volcano_plot <- lapply(1:n_diff_columns, function(i) {
        volcano_plot(
          data = dfe,
          logfold_col = diff_columns[i],
          pvalue_col = pvalue_columns[i],
          feature_rank = feature_rank,
          abundance_col = "abun",
          pvalue.threshold = pvalue.threshold,
          logfold.threshold = logfold.threshold,
          abundance.threshold = abundance.threshold,
          label_A = condition_A,
          label_B = condition_B
        ) + labs(
            subtitle = paste0(
              "Attribute: ", condition.group,
              ", test: ", ifelse(paired, "Wilcox signed rank test", "Mann-Whitney U test")
              )
          )
      })

      return(output)
    }
  ),
  private = list(
    # Private data fields
    #-------------------------#
    .countData = NULL,
    .featureData = NULL,
    .metaData = NULL,
    .treeData = NULL,
    .biomData = NULL,
    .feature_id = "FEATURE_ID",
    .sample_id = "SAMPLE_ID",
    .samplepair_id = "SAMPLEPAIR_ID",
    original_data = list(),
    construct_hdf5_featureData = function() {
      if (!is.null(private$.biomData$observation$metadata$taxonomy)) {
        private$.featureData <- data.table::data.table(t(private$.biomData$observation$metadata$taxonomy))
      }
      
      if (any(grepl(private$.feature_id, colnames(private$.metaData))) && !all(is.na(private$.metaData[[ private$.feature_id ]]))) {
        FEATURE_ID <- private$.metaData[[private$.feature_id]]
      } else if (!is.null(private$.biomData$observation$ids)) {
        FEATURE_ID <- private$.biomData$observation$ids
      } else {
        FEATURE_ID <- NULL
      }

      # Adds feature id as first column
      if (!is.null(FEATURE_ID) && !is.null(private$.featureData)) {
        private$.featureData[[ private$.feature_id ]] <- FEATURE_ID
      }

      cli::cli_alert_success("{.field featureData} is loaded.")
    },
    construct_hdf5_countData = function() {
      indptr <- as.numeric(private$.biomData$observation$matrix$indptr)

      private$.countData <- Matrix::sparseMatrix(
        i        = unlist(sapply(1:(length(indptr)-1), function (i) rep(i, diff(indptr[c(i,i+1)])))),
        j        = as.numeric(private$.biomData$observation$matrix$indices) + 1,
        x        = as.numeric(private$.biomData$observation$matrix$data),
        dims     = c(length(private$.biomData$observation$ids), length(private$.biomData$sample$ids)),
        dimnames = list(
          as.character(private$.biomData$observation$ids),
          as.character(private$.biomData$sample$ids)
        ))

      cli::cli_alert_success("{.field countData} is loaded.")
    },
    construct_json_featureData = function(feature_names) {
      # Create empty featureData
      private$.featureData <- data.table::data.table(matrix(NA_character_,
                                                     nrow = length(private$.biomData$rows),
                                                     ncol = length(c(private$.feature_id, feature_names))
                                                     ))
      setNames(private$.featureData, c(private$.feature_id, feature_names))

      # Fill first column with $id values
      private$.featureData[[ private$.feature_id ]] <- vapply(private$.biomData$rows, function(x) as.character(x$id), character(1))

      for (i in seq_along(private$.biomData$rows)) {
        taxonomy <- private$.biomData$rows[[i]]$metadata$taxonomy

        # Skip if taxonomy is missing or NULL
        if (is.null(taxonomy)) next

        # Get taxonomy index and values
        tax_indices <- seq_along(taxonomy)
        tax_values <- as.character(taxonomy)

        # Fill featureData with tax values by index
        col_positions <- tax_indices + 1
        private$.featureData[i, (col_positions) := as.list(tax_values)]
      }
      cli::cli_alert_success("{.field featureData} is loaded.")
    },
    construct_json_countData = function() {
      feature_ids <- sapply(private$.biomData$rows, function(x) unlist(x$id))
      sample_ids <- sapply(private$.biomData$columns, function(x) unlist(x$id))

      private$.countData <- Matrix::sparseMatrix(
        i        = sapply(private$.biomData$data, function(x) x[[1]]) + 1,
        j        = sapply(private$.biomData$data, function(x) x[[2]]) + 1,
        x        = sapply(private$.biomData$data, function(x) x[[3]]),
        dims     = c( length(feature_ids), length(sample_ids) ),
        dimnames = list(
          as.character(feature_ids),
          as.character(sample_ids)
        )
      )
      cli::cli_alert_success("{.field countData} is loaded.")
    }
  )
)

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bray <- function(mat) {
    .Call(`_OmicFlow_bray`, mat)
}

euclidean <- function(mat) {
    .Call(`_OmicFlow_euclidean`, mat)
}

cosine <- function(mat) {
    .Call(`_OmicFlow_cosine`, mat)
}

jaccard <- function(mat) {
    .Call(`_OmicFlow_jaccard`, mat)
}

manhattan <- function(mat) {
    .Call(`_OmicFlow_manhattan`, mat)
}

canberra <- function(mat) {
    .Call(`_OmicFlow_canberra`, mat)
}

jsd <- function(mat) {
    .Call(`_OmicFlow_jsd`, mat)
}

unifrac <- function(mat, edge, edge_lengths, weighted, normalized) {
    .Call(`_OmicFlow_unifrac`, mat, edge, edge_lengths, weighted, normalized)
}

test_that("Testing dissimilarity metrics on sparse data", {
        taxa <- metagenomics$new(
        biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
        metaData = "input/metagenomics/metadata.tsv",
        treeData = "input/metagenomics/rooted_tree.newick"
    )
    taxa$scale(method = "tss")

    ## Testing Weighted Normalized UniFrac
    wunifrac_n <- unifrac(
        x = taxa$countData,
        tree = taxa$treeData,
        weighted = TRUE,
        normalized = TRUE
    )
    expect_snapshot(cat(wunifrac_n))
    expect_error(unifrac(
        x = matrix_to_dtable(taxa$countData),
        tree = taxa$treeData
    ))
    expect_error(unifrac(
        x = taxa$countData,
        tree = taxa$treeData,
        threads = 1.2
    ))
    expect_error(unifrac(
        x = taxa$countData
    ))

    ## Testing Weighted UniFrac
    wunifrac <- unifrac(
        x = taxa$countData,
        tree = taxa$treeData,
        weighted = TRUE,
        normalized = FALSE
    )
    expect_snapshot(cat(wunifrac))

    ## Testing Unweighted UniFrac
    uunifrac <- unifrac(
        x = taxa$countData,
        tree = taxa$treeData,
        weighted = FALSE,
        normalized = FALSE
    )
    expect_snapshot(cat(uunifrac))

    ## Testing Cosine
    cos <- cosine(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(cos))
    expect_no_error(cosine(x = taxa$countData, weighted = FALSE))
    expect_error(cosine(x = taxa$countData, threads = 1.2))
    expect_error(cosine(x = matrix_to_dtable(taxa$countData)))

    ## Testing Jaccard
    jac <- jaccard(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(jac))
    expect_no_error(jaccard(x = taxa$countData, weighted = FALSE))
    expect_error(jaccard(x = taxa$countData, threads = 1.2))
    expect_error(jaccard(x = matrix_to_dtable(taxa$countData)))

    ## Testing Bray
    br <- bray(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(br))
    expect_no_error(bray(x = taxa$countData, weighted = FALSE))
    expect_error(bray(x = taxa$countData, threads = 1.2))
    expect_error(bray(x = matrix_to_dtable(taxa$countData)))

    ## Testing Euclidean
    eu <- euclidean(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(eu))
    expect_no_error(euclidean(x = taxa$countData, weighted = FALSE))
    expect_error(euclidean(x = taxa$countData, threads = 1.2))
    expect_error(euclidean(x = matrix_to_dtable(taxa$countData)))

    ## Testing Canberra
    can <- canberra(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(can))
    expect_no_error(canberra(x = taxa$countData, weighted = FALSE))
    expect_error(canberra(x = taxa$countData, threads = 1.2))
    expect_error(canberra(x = matrix_to_dtable(taxa$countData)))

    ## Testing Jensen-Shannon Divergence
    jsd_res <- jsd(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(jsd_res))
    expect_no_error(jsd(x = taxa$countData, weighted = FALSE))
    expect_error(jsd(x = taxa$countData, threads = 1.2))
    expect_error(jsd(x = matrix_to_dtable(taxa$countData)))

    ## Testing Manhattan
    man <- manhattan(
        x = taxa$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(man))
    expect_no_error(manhattan(x = taxa$countData, weighted = FALSE))
    expect_error(manhattan(x = taxa$countData, threads = 1.2))
    expect_error(manhattan(x = matrix_to_dtable(taxa$countData)))
    }
)

test_that("Testing dissimilarity metrics on dense data", {
    prot <- proteomics$new(
        metaData = "input/proteomics/metadata.csv",
        countData = "input/proteomics/counts.csv",
        treeData = "input/proteomics/tree.newick"
    )
    prot$scale(method = "tss")

    # ## Testing Weighted Normalized UniFrac
    # wunifrac_n <- unifrac(
    #     x = prot$countData,
    #     tree = prot$treeData,
    #     weighted = TRUE,
    #     normalized = TRUE
    # )
    # expect_snapshot(cat(wunifrac_n))

    # ## Testing Weighted UniFrac
    # wunifrac <- unifrac(
    #     x = prot$countData,
    #     tree = prot$treeData,
    #     weighted = TRUE,
    #     normalized = FALSE
    # )
    # expect_snapshot(cat(wunifrac))

    # ## Testing Unweighted UniFrac
    # uunifrac <- unifrac(
    #     x = prot$countData,
    #     tree = prot$treeData,
    #     weighted = FALSE,
    #     normalized = FALSE
    # )
    # expect_snapshot(cat(uunifrac))

    ## Testing Cosine
    cos <- cosine(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(cos))

    ## Testing Jaccard
    jac <- jaccard(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(jac))

    ## Testing Bray
    br <- bray(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(br))

    ## Testing Euclidean
    eu <- euclidean(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(eu))

    ## Testing Canberra
    can <- canberra(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(can))

    ## Testing Jensen-Shannon Divergence
    jsd_res <- jsd(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(jsd_res))

    ## Testing Manhattan
    man <- manhattan(
        x = prot$countData,
        weighted = TRUE
    )
    expect_snapshot(cat(man))
    }
)

#' Sparse implementation of Hill numbers
#'
#' @description Computes the hill numbers for q is 0, 1 or 2.
#' Code is adapted from \link[hillR]{hill_taxa} and uses \link[Matrix]{sparseMatrix} in triplet format over the dense matrix.
#' The code is much faster and memory efficient, while still being mathematical correct.
#'
#' @param x A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.
#' @param q A wholenumber for 0, 1 or 2, default is 0.
#' @param normalize A boolean variable for sample normalization by column sums.
#' @param base Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other.
#' @return A numeric vector with type double.
#' @seealso \link[hillR]{hill_taxa}
#' 
#' @importFrom methods as
#' 
#' @examples 
#' library("Matrix")
#' 
#' n_row <- 1000
#' n_col <- 100
#' density <- 0.2
#' num_entries <- n_row * n_col
#' num_nonzero <- round(num_entries * density)
#'
#' set.seed(123)
#' positions <- sample(num_entries, num_nonzero, replace=FALSE)
#' row_idx <- ((positions - 1) %% n_row) + 1
#' col_idx <- ((positions - 1) %/% n_row) + 1
#'
#' values <- runif(num_nonzero, min = 0, max = 1)
#' sparse_mat <- sparseMatrix(
#'   i = row_idx,
#'   j = col_idx,
#'   x = values,
#'   dims = c(n_row, n_col)
#' )
#' 
#' result <- OmicFlow::hill_taxa(
#'  x = sparse_mat,
#'  q = 2
#' )
#' @export

hill_taxa <- function (x,
                       q = 0,
                       normalize = TRUE, 
                       base = exp(1)) {

  ## Error handling
  #--------------------------------------------------------------------#
  if (inherits(x, "denseMatrix") || inherits(x, "matrix") || inherits(x, "sparseMatrix")) {
    x <- as(x, "CsparseMatrix")
  } else cli::cli_abort("Input isn't a {.cls matrix}, {.cls denseMatrix} or {.cls sparseMatrix}.")

  if (!is.numeric(x@x))
    cli::cli_abort("input data must be numeric")

  if (!is.wholenumber(q)) {
    cli::cli_abort("{.val {q}} needs to be an integer.")
  } else if (!c(q %in% c(0, 1, 2))) {
    cli::cli_abort("{.val {q}} needs to be a whole number of either {.val 0}, {.val 1} or {.val 2}.")
  }

  if (any(x@x < 0, na.rm = TRUE))
    cli::cli_abort("input data must be non-negative")

  ## MAIN
  #--------------------------------------------------------------------#
  if (normalize) {
    total <- rep(Matrix::colSums(x), base::diff(x@p))
    x@x <- x@x / total
  }
  if (q == 0) {
    hill <- base::diff(x@p)
  } else {
    if (q == 1) {
      x@x <- -x@x * log(x@x, base)
      hill <- exp(Matrix::colSums(x, na.rm = TRUE))
    } else {
      x@x <- x@x^q
      total <- Matrix::colSums(x, na.rm = TRUE)
      hill <- total^(1/(1 - q))
    }
  }
  hill
}

#' Volcano plot
#'
#' @description Creates a Volcano plot from the output of `foldchange` method from class \link{omics}, it plots the foldchanges on the x-axis, 
#' log10 trasnformed p-values on the y-axis and adjusts the scatter size based on the percentage abundance of the features. 
#' This function is built into the class \link{omics} with method \code{DFE()} and inherited by other omics classes, such as;
#' \link{metagenomics} and \link{proteomics}.
#'
#' @param data A \link[data.table]{data.table}.
#' @param logfold_col A column name of a continuous variable.
#' @param pvalue_col A column name of a continuous variable.
#' @param abundance_col A column name of a continuous variable.
#' @param feature_rank A character variable of the feature column.
#' @param logfold.threshold A Log2(A/B) Fold Change threshold (default: 0.6).
#' @param pvalue.threshold A P-value threshold (default: 0.05).
#' @param abundance.threshold An abundance threshold (default: 0.01).
#' @param label_A A character to describe condition A.
#' @param label_B A character to describe condition B.
#' 
#' @return A \link[ggplot2]{ggplot2} object to be further modified.
#' @importFrom ggplot2 ggplot aes .data theme_bw theme element_text labs geom_point geom_vline geom_hline element_rect scale_color_gradient2 scale_size_continuous
#' @examples 
#' library(data.table)
#' library(ggplot2)
#' 
#' # Create mock data frame
#' mock_volcano_data <- data.table(
#' 
#'   # Feature names (feature_rank)
#'   Feature = paste0("Gene", 1:20),   
#' 
#'   # Log2 fold changes (X)        
#'   log2FC = c(1.2, -1.5, 0.3, -0.7, 2.3,    
#'              -2.0, 0.1, 0.5, -1.0, 1.8,
#'              -0.4, 0.7, -1.4, 1.5, 0.9,
#'              -2.1, 0.2, 1.0, -0.3, -1.8),
#'   
#'   # P-values (Y)
#'   pvalue = c(0.001, 0.02, 0.3, 0.04, 0.0005, 
#'              0.01, 0.7, 0.5, 0.02, 0.0008,
#'              0.15, 0.06, 0.01, 0.005, 0.3,
#'              0.02, 0.8, 0.04, 0.12, 0.03),
#'   
#'   # Mean (relative) abundance for point sizing
#'   rel_abun = runif(20, 0.01, 0.1)            
#' )
#' 
#' volcano_plot(
#'   data = mock_volcano_data,
#'   logfold_col = "log2FC",
#'   pvalue_col = "pvalue",
#'   abundance_col = "rel_abun",
#'   feature_rank = "Feature",
#' )
#' @export

volcano_plot <- function(data,
                         logfold_col,
                         pvalue_col,
                         feature_rank,
                         abundance_col,
                         pvalue.threshold = 0.05,
                         logfold.threshold = 0.6,
                         abundance.threshold = 0.01,
                         label_A = "A",
                         label_B = "B") {

  ## Error handling
  #--------------------------------------------------------------------#

  if (!inherits(data, "data.table"))
    cli::cli_abort("Data must be a {.cls data.table}.")

  if (!is.character(logfold_col) && length(logfold_col) != 1) {
    cli::cli_abort("{.val {logfold_col}} needs to contain characters with length of 1.")
  } else if (!column_exists(logfold_col, data)) {
    cli::cli_abort("The {.val {logfold_col}} column does not exist in the provided {.arg data}.")
  }

  if (!is.character(pvalue_col) && length(pvalue_col) != 1) {
    cli::cli_abort("{.val {pvalue_col}} needs to contain characters with length of 1.")
  } else if (!column_exists(pvalue_col, data)) {
    cli::cli_abort("The {.val {pvalue_col}} column does not exist in the provided {.arg data}.")
  }

  if (!is.character(abundance_col) && length(abundance_col) != 1) {
    cli::cli_abort("{.val {abundance_col}} needs to contain characters with length of 1.")
  } else if (!column_exists(abundance_col, data)) {
    cli::cli_abort("The {.val {abundance_col}} column does not exist in the provided {.arg data}.")
  }

  if (!is.character(feature_rank) && length(feature_rank) != 1) {
    cli::cli_abort("{.val {feature_rank}} needs to contain characters with length of 1.")
  } else if (!column_exists(feature_rank, data)) {
    cli::cli_abort("The {.val {feature_rank}} column does not exist in the provided {.arg data}.")
  }

  if (!is.numeric(pvalue.threshold))
    cli::cli_abort("{.val {pvalue.threshold}} need to be numeric.")

  if (!is.numeric(logfold.threshold))
    cli::cli_abort("{.val {logfold.threshold}} need to be numeric.")

  if (!is.numeric(abundance.threshold))
    cli::cli_abort("{.val {abundance.threshold}} need to be numeric.")

  if (!is.character(label_A))
    cli::cli_abort("{.val {label_A}} needs to contain characters.")

  if (!is.character(label_B))
    cli::cli_abort("{.val {label_B}} needs to contain characters.")

  ## MAIN
  #--------------------------------------------------------------------#

  # copies data.table
  tmpdt <- data.table::copy(data)

  # Creates labels for significant and non-significant differential expression
  tmpdt[, (pvalue_col) := -log10(base::get(pvalue_col))]
  tmpdt[, "diffexpressed" := ifelse(
    base::get(logfold_col) > logfold.threshold &
    base::get(pvalue_col) > -log10(pvalue.threshold) &
    base::get(abundance_col) >= abundance.threshold, 
    "Upregulated",
    ifelse(
      base::get(logfold_col) < -logfold.threshold &
      base::get(pvalue_col) > -log10(pvalue.threshold) &
      base::get(abundance_col) >= abundance.threshold, 
      "Downregulated", 
      "non-significant"
      )
    )]
  tmpdt[, "diffexpressed_labels" := ifelse(base::get("diffexpressed") != "non-significant", base::get(feature_rank), "")]

  plt <- tmpdt %>%
      ggplot(mapping = aes(x = .data [[ logfold_col ]],
                           y = .data [[ pvalue_col ]],
                           label = .data[["diffexpressed_labels"]],
                           color = .data [[ logfold_col ]])) +
      theme_bw() +
      theme(axis.text.x = element_text(angle = 45, hjust = 1, size=12),
            axis.text.y = element_text(size=12),
            axis.text = element_text(size=12),
            text = element_text(size=12),
            legend.text = element_text(size=12),
            legend.title = element_text(size=14),
            strip.background = element_rect(fill = "#EEEEEE", color = "#FFFFFF")) +
      geom_vline(xintercept = c(-logfold.threshold, logfold.threshold),
                 col = "black", linetype = 'dashed') +
      geom_hline(yintercept = -log10(pvalue.threshold),
                 col = "black", linetype = 'dashed') +
      scale_color_gradient2(name = "foldchange",
                            low = "blue",
                            mid = "black",
                            high = "red",
                            na.value = "grey80") +
      ggrepel::geom_label_repel(show.legend = FALSE,
                                max.overlaps = getOption("ggrepel.max.overlaps", default = Inf),
                                color = "black")
    if (any(tmpdt$diffexpressed != "non-significant")) {
      plt <- plt +
        geom_point(
          aes(size = as.numeric(ifelse(.data[["diffexpressed"]] != "non-significant", .data[[ abundance_col ]]*100, 0))),
              shape = 16, alpha = 0.5
        )
    } else {
      plt <- plt + geom_point(shape = 16, alpha = 0.5)
    }
    plt <- plt +
      scale_size_continuous(name = "Mean Abundance (%)") +
      labs(x = paste0("Fold Change ( ", label_A," / ", label_B," )"),
           y = paste0("-log10( ", pvalue_col ," )"))

  return(plt)
}

test_that("Testing `volcano_plot`", {
# Create mock data frame
mock_volcano_data <- data.table(

  # Feature names (feature_rank)
  Feature = paste0("Gene", 1:20),

  # Log2 fold changes (X)        
  log2FC = c(1.2, -1.5, 0.3, -0.7, 2.3,
             -2.0, 0.1, 0.5, -1.0, 1.8,
             -0.4, 0.7, -1.4, 1.5, 0.9,
             -2.1, 0.2, 1.0, -0.3, -1.8),
  
  # P-values (Y)
  pvalue = c(0.001, 0.02, 0.3, 0.04, 0.0005,
             0.01, 0.7, 0.5, 0.02, 0.0008,
             0.15, 0.06, 0.01, 0.005, 0.3,
             0.02, 0.8, 0.04, 0.12, 0.03),
  
  # Mean (relative) abundance for point sizing
  rel_abun = runif(20, 0.01, 0.1)
)

  expect_error(volcano_plot(
    data = as.matrix(mock_volcano_data),
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "val2",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pval",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "abun",
    feature_rank = "Feature",
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feat",
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
    pvalue.threshold = "low"
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
    logfold.threshold = "low"
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
    abundance.threshold = "low"
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
    label_A = 1
  ))

  expect_error(volcano_plot(
    data = mock_volcano_data,
    logfold_col = "log2FC",
    pvalue_col = "pvalue",
    abundance_col = "rel_abun",
    feature_rank = "Feature",
    label_B = 1
  ))
})

#' Sparse implementation of Alpha Diversity Metrics
#'
#' @description Computes the alpha diversity based on Shannon index, simpson or invsimpson.
#' Code is adapted from \link[vegan]{diversity} and uses \link[Matrix]{sparseMatrix} in triplet format over the dense matrix.
#' The code is much faster and memory efficient, while still being mathematically correct.
#' This function is built into the class \link{omics} with method \code{alpha_diversity()} and inherited by other omics classes, such as;
#' \link{metagenomics} and \link{proteomics}.
#'
#' @param x A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.
#' @param metric A character variable for metric; shannon, simpson or invsimpson.
#' @param normalize A boolean variable for sample normalization by column sums.
#' @param base Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other (default: \code{exp(1)}).
#' @return A numeric vector with type double.
#' @seealso \link[vegan]{diversity}
#' @examples 
#' n_row <- 1000
#' n_col <- 100
#' density <- 0.2
#' num_entries <- n_row * n_col
#' num_nonzero <- round(num_entries * density)
#'
#' set.seed(123)
#' positions <- sample(num_entries, num_nonzero, replace=FALSE)
#' row_idx <- ((positions - 1) %% n_row) + 1
#' col_idx <- ((positions - 1) %/% n_row) + 1
#'
#' values <- runif(num_nonzero, min = 0, max = 1)
#' sparse_mat <- sparseMatrix(
#'   i = row_idx,
#'   j = col_idx,
#'   x = values,
#'   dims = c(n_row, n_col)
#' )
#'
#' # Alpha diversity is computed on column level
#' ## Transpose the sparseMatrix if required with t() from Matrix R package.
#' result <- OmicFlow::diversity(
#'   x = sparse_mat,
#'   metric = "shannon"
#' ) 
#' @export

diversity <- function(x,
                      metric = c("shannon", "simpson", "invsimpson"),
                      normalize = TRUE,
                      base = exp(1)) {

  ## Error handling
  #--------------------------------------------------------------------#

  if (inherits(x, "denseMatrix") || inherits(x, "matrix") || inherits(x, "sparseMatrix")) {
    x <- as(x, "CsparseMatrix")
  } else cli::cli_abort("Input isn't a {.cls matrix}, {.cls denseMatrix} or {.cls sparseMatrix}.")

  if (!is.numeric(x@x))
    cli::cli_abort("Input data must be {.cls numeric} type")

  if (any(x@x < 0, na.rm = TRUE))
    cli::cli_abort("Input data must be non-negative")
  
  if (!is.numeric(base))
    cli::cli_abort("{.val {base}} needs to be a {.cls numeric} type.")

  OPTIONS <- c("shannon", "simpson", "invsimpson")
  if (!is.character(metric) && length(metric) != 1) {
    cli::cli_abort("{.val {metric}} needs to contain characters with length of 1.")
  } else if (!metric %in% OPTIONS) {
    cli::cli_abort("{.val {metric}} is not a valid metric. Valid options: <{.val {OPTIONS}}>")
  }

  ## MAIN
  #--------------------------------------------------------------------#
  total <- rep(Matrix::colSums(x), base::diff(x@p))
  if (normalize) {
    x@x <- x@x / total
  }

  if (metric == "shannon") {
    x@x <- -x@x * log(x@x, base)
  } else {
    x@x <- x@x * x@x
  }
  if (length(dim(x)) > 1) {
    H <- Matrix::colSums(x, na.rm = TRUE)
  }
  if (metric == "simpson") {
    H <- 1 - H
  } else if (metric == "invsimpson") {
    H <- 1/H
  }
  ## check NA in data
  if (any(NAS <- is.na(total)))
    H[NAS] <- NA
  H
}

test_that("Testing Proteomics loading", {
  counts_with_rownames <- proteomics$new(
    metaData = "input/proteomics/metadata.csv",
    countData = "input/proteomics/counts.csv",
    treeData = "input/proteomics/tree.newick"
  )

  counts_without_rownames <- proteomics$new(
    metaData = "input/proteomics/metadata.csv",
    countData = "input/proteomics/counts_without_rownames.csv"
  )
  
  prot_data <- proteomics$new(
    countData = counts_with_rownames$countData,
    metaData = counts_with_rownames$metaData,
    featureData = counts_with_rownames$featureData,
    treeData = counts_with_rownames$treeData
  )
  expect_snapshot(counts_without_rownames)
  expect_snapshot(counts_with_rownames)
  expect_snapshot(prot_data)

  # Adding treeData after init
  prot_1 <- proteomics$new(
    countData = counts_with_rownames$countData,
    metaData = counts_with_rownames$metaData,
    featureData = counts_with_rownames$featureData,
  )

  expect_error(prot_1$treeData <- counts_with_rownames$countData)
  expect_no_error(prot_1$treeData <- counts_with_rownames$treeData)
})

test_that("Testing omics loading from existing data objects", {
  n_cols <- 5
  n_rows <- 100
  n_vals <- n_cols * n_rows
  
  metadata <- data.table::data.table("SAMPLE_ID" = paste0("Sample_", 1:n_cols))
  features <- data.table::data.table("FEATURE_ID" = paste0("protein_", 1:n_rows))
  counts <- Matrix::Matrix(
    1:n_vals, nrow = n_rows, ncol = n_cols, 
    dimnames = list(features$FEATURE_ID, metadata$SAMPLE_ID)
  )
  
  # building up omics with metaData -> countData -> featureData
  ends_with_featureData <- omics$new(metaData = metadata)
  ends_with_featureData$countData <- counts

  expect_snapshot(ends_with_featureData)

  # building up omics with metaData -> featureData -> countData
  ends_with_countData <- omics$new(metaData = metadata)
  ends_with_countData$featureData <- features
  ends_with_countData$countData <- counts

  expect_snapshot(ends_with_countData)
})

test_that("Testing omics loading from file", {
  metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
  counts_sparse_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
  counts_sparse_with_rownames_file <- system.file("extdata", "counts_with_rownames.tsv", package = "OmicFlow")
  counts_dense_file <- system.file("extdata", "counts_dense.tsv", package = "OmicFlow")
  features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

  # Loading data with features and with sparse or dense formats from file
  with_features_sparse <- omics$new(
    countData = counts_sparse_file,
    featureData = features_file,
    metaData = metadata_file
  )
  expect_snapshot(with_features_sparse)

  with_features_dense <- omics$new(
    countData = counts_dense_file,
    featureData = features_file,
    metaData = metadata_file
  )
  expect_snapshot(with_features_dense)

  # Loading data without features but with/without rownames from counts file
  without_features_with_rownames <- omics$new(
    countData = counts_sparse_with_rownames_file,
    metaData = metadata_file
  )
  expect_snapshot(without_features_with_rownames)
  
  without_features_without_rownames <- omics$new(
    countData = counts_sparse_file,
    metaData = metadata_file
  )
  expect_snapshot(without_features_without_rownames)
})

test_that("Testing Metagenomics reading and writing of BIOM files", {
  # Loading hdf5 format
  taxa_hdf5 <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_hdf5.biom",
    metaData = "input/metagenomics/metadata.tsv",
    treeData = "input/metagenomics/rooted_tree.newick"
  )
  
  taxa_ref <- metagenomics$new(
    countData = taxa_hdf5$countData,
    metaData = taxa_hdf5$metaData,
    treeData = taxa_hdf5$treeData,
    featureData = taxa_hdf5$featureData
  )
  
  expect_snapshot(taxa_hdf5)
  expect_snapshot(taxa_ref)

    # Adding treeData after init
  taxa_ref <- metagenomics$new(
    countData = taxa_hdf5$countData,
    metaData = taxa_hdf5$metaData,
    featureData = taxa_hdf5$featureData,
  )

  expect_error(taxa_ref$treeData <- taxa_hdf5$countData)
  expect_no_error(taxa_ref$treeData <- taxa_hdf5$treeData)

  # Loading JSON format
  taxa_json <- metagenomics$new(
    biomData = "input/metagenomics/biom_with_taxonomy_json.biom",
    metaData = data.table::data.table(SAMPLE_ID = c(
      "Sample1", "Sample2", "Sample3",
      "Sample4", "Sample5", "Sample6"))
  )
  expect_snapshot(taxa_json)
})

---
title: "Automated Flow (autoFlow)"
date: "`r Sys.Date()`"
author: "Gusinac A"

output:
  html_document:
    toc: true
    toc_float: true
    css: styles.css
---

```{r setup, include=FALSE} 
knitr::opts_chunk$set(warning = FALSE,
                      message = FALSE,
                      echo = FALSE,
                      results = "asis",
                      fig.width = 10,
                      fig.height = 5,
                      fig.alin = "left"
                      )
# You need this code to conduct the magic dependences attaching...(https://github.com/rstudio/DT/issues/67)
DT::datatable(matrix())
```

```{r}
create_plot_tabs <- function(plots, data = NULL, method, label_col, label_row, show_data = FALSE) {
  
  save_content <- function(data, name) {
    # Save each data frame as CSV with a name-based tempfile
    temp_file <- tempfile(pattern = name, fileext = ".csv")
    data.table::fwrite(data, temp_file)
    return(temp_file)
  }
  
  if (method == "matrix") {
    for (i in seq_len(nrow(plots))) {
      for (j in seq_len(ncol(plots))) {
        if (!is.null(plots[[i, j]])) {
          cat(paste0("\n\n### Plot: ", label_col[i], "-", label_row[j], "\n"))
          print(plots[[i,j]])
          cat("\n")
          
          if (show_data && !is.null(data)) {
            # Expect datas[[i,j]] to be a named list of data.frames
            data_list <- data[[i, j]]
            if (is.list(data_list) && length(data_list) > 0) {
              # table with HTML buttons
              overview <- data.frame(
                Dataset = character(),
                Download = character(),
                stringsAsFactors = FALSE
                )
              for (df_name in names(data_list)) {
                df <- data_list[[df_name]]
                file_path <- save_content(df, df_name)
                
                button_html <- as.character(
                  downloadthis::download_file(
                    path = file_path,
                    output_name = paste0(label_col[i], "_", label_row[j], "_", df_name, ".csv"),
                    button_label = "Download",
                    button_type = "primary",
                    has_icon = TRUE,
                    icon = "fa fa-save"
                  )
                )
                ## Add button to overview table
                overview <- rbind(
                  overview,
                  data.frame(
                    Dataset = df_name,
                    Download = button_html,
                    stringsAsFactors = FALSE
                  )
                )
              }
            }
          # Display as interactive table
          cat(knitr::knit_print(DT::datatable(
            overview,
            rownames = FALSE,
            escape = FALSE,
            height = "100%",
            width = "100%"
            )))
          cat("\n")
          }
        }
      }
    }
  }
  
  if (method == "vector") {
    for (i in seq_along(plots)) {
      if (!is.null(plots[[i]])) {
        cat(paste0("\n\n### Plot: ", label_col[i], "\n"))
        print(plots[[i]])
        cat("\n")
        
        if (show_data && !is.null(data)) {
          # Expect datas[[i]] to be a named list of data.frames
          data_list <- data[[i]]
          if (is.list(data_list) && length(data_list) > 0) {
            # table with HTML buttons
            overview <- data.frame(
              Dataset = character(),
              Download = character(),
              stringsAsFactors = FALSE
              )

            for (df_name in names(data_list)) {
              df <- data_list[[df_name]]
              file_path <- save_content(df, df_name)

              button_html <- as.character(
                downloadthis::download_file(
                  path = file_path,
                  output_name = paste0(label_col[i], "_", df_name, ".csv"),
                  button_label = "Download",
                  button_type = "primary",
                  has_icon = TRUE,
                  icon = "fa fa-save"
                )
              )
              ## Add button to overview table
              overview <- rbind(
                overview,
                data.frame(
                  Dataset = df_name,
                  Download = button_html,
                  stringsAsFactors = FALSE
                )
              )
            }
          }
        # Display as interactive table
        cat(knitr::knit_print(DT::datatable(
          overview,
          escape = FALSE,
          rownames = FALSE,
          height = "100%",
          width = "100%"
        )))
        cat("\n")
        }
      }
    }
  }
}

```

```{r}
if (!is.null(plots$rankstat_plot)) {
  cat("## Rank stats \n")
  plots$rankstat_plot
}
```

```{r, results='asis'}
if (!is.null(plots$alpha_div_plots)) {
  cat("## Alpha diversity {.tabset} \n")
  create_plot_tabs(
    plots = plots$alpha_div_plots, 
    data = data$alpha_div_data,
    method = "vector", 
    label_col = CONTRAST_names,
    show_data = TRUE
    )
}

```

```{r}
if (!is.null(plots$pcoa_plots)) {
  cat("## Beta Diversity: Principal Coordinate Analysis (PCoA) {.tabset} \n")
  create_plot_tabs(
    plots = plots$pcoa_plots,
    data = data$pcoa_data,
    method = "matrix", 
    label_col = CONTRAST_names,
    label_row = distance_metrics,
    show_data = TRUE
    )
}
```

```{r}
if (!is.null(plots$composition_plots)) {
  cat("## Composition {.tabset} \n")
  create_plot_tabs(
    plots = plots$composition_plots, 
    data = data$composition_data,
    method = "matrix", 
    label_col = CONTRAST_names, 
    label_row = feature_contrast,
    show_data = TRUE
    )
}

```

```{r}
if (!is.null(plots$Log2FC_plots)) {
  cat("## Log2 FoldChanges {.tabset} \n")
  create_plot_tabs(
    plots = plots$Log2FC_plots, 
    data = data$Log2FC_data,
    method = "matrix", 
    label_col = CONTRAST_names, 
    label_row = feature_contrast,
    show_data = TRUE
    )
}

```

```{r}
cat("## Thanks for using OmicFlow!\n")
print(citation("OmicFlow"), style = "text")
```

# Testing Proteomics loading

    Code
      counts_without_rownames
    Message
      
      -- <proteomics> object 
      metaData: 2 variables x 10 samples
      countData: 10 samples x 50 features
      featureData: 0 attributes x 50 features

---

    Code
      counts_with_rownames
    Message
      
      -- <proteomics> object 
      metaData: 2 variables x 10 samples
      countData: 10 samples x 50 features
      featureData: 0 attributes x 50 features
      treeData: 50 tips x 49 nodes

---

    Code
      prot_data
    Message
      
      -- <proteomics> object 
      metaData: 2 variables x 10 samples
      countData: 10 samples x 50 features
      featureData: 0 attributes x 50 features
      treeData: 50 tips x 49 nodes

# Testing Alpha diversity

    Code
      res_shannon$data
    Output
               V1 CONTRAST_sex
            <num>       <char>
      1: 3.403898         male
      2: 3.776849       female
      3: 3.682609       female
      4: 3.686005         male

---

    Code
      res_shannon$stats
    Output
        .y. group1 group2 n1 n2 statistic     p p.adj p.adj.signif y.position
      1  V1 female   male  2  2         3 0.667 0.667           ns    3.78792
              groups xmin xmax
      1 female, male    1    2

---

    Code
      res_grouped_shannon$data
    Output
               V1 CONTRAST_sex group_col
            <num>       <char>    <char>
      1: 3.403898         male     tumor
      2: 3.776849       female     tumor
      3: 3.682609       female   healthy
      4: 3.686005         male   healthy

---

    Code
      res_grouped_shannon$stats
    Output
        treatment .y. group1 group2 n1 n2 statistic p p.adj p.adj.signif y.position
      1     tumor  V1 female   male  1  1         1 1     1           ns    3.82176
      2   healthy  V1 female   male  1  1         0 1     1           ns    3.68636
              groups xmin xmax
      1 female, male   NA   NA
      2 female, male   NA   NA

---

    Code
      res_invsimpson$data
    Output
               V1 CONTRAST_sex
            <num>       <char>
      1: 22.91679         male
      2: 34.51324       female
      3: 27.09552       female
      4: 29.54116         male

---

    Code
      res_invsimpson$stats
    Output
        .y. group1 group2 n1 n2 statistic     p p.adj p.adj.signif y.position
      1  V1 female   male  2  2         3 0.667 0.667           ns   35.10964
              groups xmin xmax
      1 female, male    1    2

---

    Code
      res_simpson$data
    Output
                V1 CONTRAST_sex
             <num>       <char>
      1: 0.9563639         male
      2: 0.9710256       female
      3: 0.9630935       female
      4: 0.9661489         male

---

    Code
      res_simpson$stats
    Output
        .y. group1 group2 n1 n2 statistic     p p.adj p.adj.signif y.position
      1  V1 female   male  2  2         3 0.667 0.667           ns     0.9716
              groups xmin xmax
      1 female, male    1    2

# Testing Compositional data

    Code
      res$data
    Output
      Key: <SAMPLE_ID>
          SAMPLE_ID                   Genus      value CONTRAST_sex
             <char>                  <fctr>      <num>       <char>
       1:      S100         Woesearchaeales 0.02022059         male
       2:      S100            Sideroxydans 0.19209559         male
       3:      S100           Acinetobacter 0.16727941         male
       4:      S100  Candidatus_Omnitrophus 0.05147059         male
       5:      S100             Pseudomonas 0.09099265         male
       6:      S100          Nitrosarchaeum 0.06341912         male
       7:      S100 Candidatus_Peribacteria 0.00000000         male
       8:      S100              Nitrospira 0.03033088         male
       9:      S100           Sulfurifustis 0.00000000         male
      10:      S100           Aquabacterium 0.03400735         male
      11:      S100                   Other 0.09466912         male
      12:      S103         Woesearchaeales 0.28865979       female
      13:      S103            Sideroxydans 0.00000000       female
      14:      S103           Acinetobacter 0.02268041       female
      15:      S103  Candidatus_Omnitrophus 0.06391753       female
      16:      S103             Pseudomonas 0.00000000       female
      17:      S103          Nitrosarchaeum 0.00000000       female
      18:      S103 Candidatus_Peribacteria 0.04123711       female
      19:      S103              Nitrospira 0.00000000       female
      20:      S103           Sulfurifustis 0.03402062       female
      21:      S103           Aquabacterium 0.00000000       female
      22:      S103                   Other 0.20618557       female
      23:      S115         Woesearchaeales 0.03149002       female
      24:      S115            Sideroxydans 0.20890937       female
      25:      S115           Acinetobacter 0.09677419       female
      26:      S115  Candidatus_Omnitrophus 0.08986175       female
      27:      S115             Pseudomonas 0.04685100       female
      28:      S115          Nitrosarchaeum 0.09831029       female
      29:      S115 Candidatus_Peribacteria 0.00000000       female
      30:      S115              Nitrospira 0.03917051       female
      31:      S115           Sulfurifustis 0.01228879       female
      32:      S115           Aquabacterium 0.02534562       female
      33:      S115                   Other 0.14669739       female
      34:      S120         Woesearchaeales 0.12540894         male
      35:      S120            Sideroxydans 0.00000000         male
      36:      S120           Acinetobacter 0.00000000         male
      37:      S120  Candidatus_Omnitrophus 0.06434024         male
      38:      S120             Pseudomonas 0.05779716         male
      39:      S120          Nitrosarchaeum 0.00000000         male
      40:      S120 Candidatus_Peribacteria 0.05125409         male
      41:      S120              Nitrospira 0.01853871         male
      42:      S120           Sulfurifustis 0.03707743         male
      43:      S120           Aquabacterium 0.00000000         male
      44:      S120                   Other 0.20828790         male
          SAMPLE_ID                   Genus      value CONTRAST_sex
             <char>                  <fctr>      <num>       <char>

---

    Code
      res$palette
    Output
              Woesearchaeales            Sideroxydans           Acinetobacter 
                    "#A50026"               "#D73027"               "#F46D43" 
       Candidatus_Omnitrophus             Pseudomonas          Nitrosarchaeum 
                    "#FDAE61"               "#FEE090"               "#E0F3F8" 
      Candidatus_Peribacteria              Nitrospira           Sulfurifustis 
                    "#ABD9E9"               "#74ADD1"               "#4575B4" 
                Aquabacterium                   Other 
                    "#313695"             "lightgrey" 

# Testing omics loading from existing data objects

    Code
      ends_with_featureData
    Message
      
      -- <omics> object 
      metaData: 1 variables x 5 samples
      countData: 5 samples x 100 features
      featureData: 0 attributes x 100 features

---

    Code
      ends_with_countData
    Message
      
      -- <omics> object 
      metaData: 1 variables x 5 samples
      countData: 5 samples x 100 features
      featureData: 0 attributes x 100 features

# Testing omics loading from file

    Code
      with_features_sparse
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features

---

    Code
      with_features_dense
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features

---

    Code
      without_features_with_rownames
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 0 attributes x 242 features

---

    Code
      without_features_without_rownames
    Message
      
      -- <omics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 0 attributes x 242 features

# Testing subsetting of data

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 185 features
      featureData: 7 attributes x 185 features
      treeData: 185 tips x 184 nodes

---

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 2 samples
      countData: 2 samples x 115 features
      featureData: 7 attributes x 115 features
      treeData: 115 tips x 114 nodes

---

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 200 features
      featureData: 7 attributes x 200 features
      treeData: 200 tips x 199 nodes

---

    Code
      taxa
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 50 features
      featureData: 7 attributes x 50 features
      treeData: 50 tips x 49 nodes

# Testing Metagenomics reading and writing of BIOM files

    Code
      taxa_hdf5
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features
      treeData: 242 tips x 241 nodes

---

    Code
      taxa_ref
    Message
      
      -- <metagenomics> object 
      metaData: 9 variables x 4 samples
      countData: 4 samples x 242 features
      featureData: 7 attributes x 242 features
      treeData: 242 tips x 241 nodes

---

    Code
      taxa_json
    Message
      
      -- <metagenomics> object 
      metaData: 1 variables x 6 samples
      countData: 6 samples x 5 features
      featureData: 8 attributes x 5 features

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diversity_plot.R
\name{diversity_plot}
\alias{diversity_plot}
\title{Diversity plot}
\usage{
diversity_plot(
  data,
  values,
  col_name,
  group_by = NULL,
  palette,
  method,
  paired = FALSE,
  p.adjust.method = "fdr"
)
}
\arguments{
\item{data}{A \link[base]{data.frame} or \link[data.table]{data.table} computed from \link{diversity}.}

\item{values}{A column name of a continuous variable.}

\item{col_name}{A column name of a categorical variable.}

\item{group_by}{A column name to perform grouped statistical test (default: NULL).}

\item{palette}{An object with names and hexcode or color names, see \link{colormap}.}

\item{method}{A character variable indicating what method is used to compute the diversity.}

\item{paired}{A boolean value to perform paired analysis in \link[stats]{wilcox.test}.}

\item{p.adjust.method}{A character variable to specify the p.adjust.method to be used (Default: fdr).}
}
\value{
A \link[ggplot2]{ggplot2} object to be further modified
}
\description{
Creates an Alpha diversity plot. This function is built into the class \link{omics} with method \code{alpha_diversity()}.
It computes the pairwise wilcox test, paired or non-paired, given a data frame and adds useful labelling.
}
\examples{
library("ggplot2")
 
n_row <- 1000
n_col <- 100
density <- 0.2
num_entries <- n_row * n_col
num_nonzero <- round(num_entries * density)

set.seed(123)
positions <- sample(num_entries, num_nonzero, replace=FALSE)
row_idx <- ((positions - 1) \%\% n_row) + 1
col_idx <- ((positions - 1) \%/\% n_row) + 1

values <- runif(num_nonzero, min = 0, max = 1)
sparse_mat <- Matrix::sparseMatrix(
   i = row_idx,
   j = col_idx,
   x = values,
   dims = c(n_row, n_col)
 )

div <- OmicFlow::diversity(
  x = sparse_mat,
  metric = "shannon"
)

dt <- data.table::data.table(
  "shannon" = div,
  "treatment" = c(rep("healthy", n_col / 2), rep("tumor", n_col / 2)),
  "sex" = c(rep("male", n_col / 4), rep("female", n_col / 4))
)

colors <- OmicFlow::colormap(dt, "treatment")

# Comparing two groups
plt <- OmicFlow::diversity_plot(
 data = dt,
 values = "shannon",
 col_name = "treatment",
 palette = colors,
 method = "shannon",
 paired = FALSE,
 p.adjust.method = "fdr"
)

# Performing a test while stratifying the plot in two groups
plt <- OmicFlow::diversity_plot(
 data = dt,
 values = "shannon",
 col_name = "treatment",
 group_by = "sex",
 palette = colors,
 method = "shannon",
 paired = FALSE,
 p.adjust.method = "fdr"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ordination_plot.R
\name{ordination_plot}
\alias{ordination_plot}
\title{Ordination plot}
\usage{
ordination_plot(
  data,
  col_name,
  pair,
  dist_explained = NULL,
  dist_metric = NULL
)
}
\arguments{
\item{data}{A \link[base]{data.frame} or \link[data.table]{data.table} of Principal Components as columns and rows as loading scores.}

\item{col_name}{A categorical variable to color the contrasts (e.g. "groups").}

\item{pair}{A vector of character variables indicating what dimension names (e.g. PC1, NMDS2).}

\item{dist_explained}{A vector of numeric values of the percentage dissimilarity explained for the dimension pairs, default is NULL.}

\item{dist_metric}{A character variable indicating what metric is used (e.g. unifrac, bray-curtis), default is NULL.}
}
\value{
A \link[ggplot2]{ggplot2} object to be further modified
}
\description{
Creates an ordination plot pre-computed principal components from \link[vegan]{wcmdscale}.
This function is built into the class \link{omics} with method \code{ordination()} and inherited by other omics classes, such as;
\link{metagenomics} and \link{proteomics}.
}
\examples{
library(ggplot2)

# Mock principal component scores
set.seed(123)
mock_data <- data.frame(
  SampleID = paste0("Sample", 1:10),
  PC1 = rnorm(10, mean = 0, sd = 1),
  PC2 = rnorm(10, mean = 0, sd = 1),
  groups = rep(c("Group1", "Group2"), each = 5)
)

# Basic usage
ordination_plot(
  data = mock_data,
  col_name = "groups",
  pair = c("PC1", "PC2")
)

# Adding variance/dissimilarity explained.
ordination_plot(
  data = mock_data,
  col_name = "groups",
  pair = c("PC1", "PC2"),
  dist_explained = c(45, 22),
  dist_metric = "bray-curtis"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pairwise_stats.R
\name{plot_pairwise_stats}
\alias{plot_pairwise_stats}
\title{Create pairwise stats plot}
\usage{
plot_pairwise_stats(
  data,
  stats_col,
  group_col,
  label_col,
  y_axis_title = NULL,
  plot_title = NULL
)
}
\arguments{
\item{data}{A \link[base]{data.frame} or \link[data.table]{data.table}.}

\item{stats_col}{A column name of a continuous variable.}

\item{group_col}{A column name of a categorical variable.}

\item{label_col}{A column name of a categorical variable to label the bars.}

\item{y_axis_title}{A character variable to name the Y - axis title (default: NULL).}

\item{plot_title}{A character variable to name the plot title (default: NULL).}
}
\value{
A \link[ggplot2]{ggplot2} object to be further modified
}
\description{
Creates a pairwise stats plot from \link{pairwise_adonis} or \link{pairwise_anosim} results.
This function is built into the class \link{omics} with method \code{ordination()} and inherited by other omics classes, such as;
\link{metagenomics} and \link{proteomics}.
}
\examples{
library("ggplot2")

# Create random data
set.seed(42)
mock_data <- matrix(rnorm(15 * 10), nrow = 15, ncol = 10)

# Create euclidean dissimilarity matrix
mock_dist <- dist(mock_data, method = "euclidean")

# Define group labels, should be equal to number of columns and rows to dist
mock_groups <- rep(c("A", "B", "C"), each = 5)

# Compute pairwise adonis
adonis_res <- pairwise_adonis(x = mock_dist, 
                              groups = mock_groups, 
                              p.adjust.method = "bonferroni", 
                              perm = 99)
# Compute pairwise anosim
anosim_res <- pairwise_anosim(x = mock_dist, 
                              groups = mock_groups, 
                              p.adjust.method = "bonferroni", 
                              perm = 99)

# Visualize PERMANOVA pairwise stats
plot_pairwise_stats(data = adonis_res,
                    group_col = "pairs",
                    stats_col = "F.Model",
                    label_col = "p.adj",
                    y_axis_title = "Pseudo F test statistic",
                    plot_title = "PERMANOVA")

# Visualize ANOSIM pairwise stats
plot_pairwise_stats(data = anosim_res,
                    group_col = "pairs",
                    stats_col = "anosimR",
                    label_col = "p.adj",
                    y_axis_title = "ANOSIM R statistic",
                    plot_title = "ANOSIM")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/composition_plot.R
\name{composition_plot}
\alias{composition_plot}
\title{Compositional plot}
\usage{
composition_plot(
  data,
  palette,
  feature_rank,
  title_name = NULL,
  group_by = NULL
)
}
\arguments{
\item{data}{A \link[base]{data.frame} or \link[data.table]{data.table}.}

\item{palette}{An object with names and hexcode or color names, see \link{colormap}.}

\item{feature_rank}{A character variable of the feature column.}

\item{title_name}{A character to set the \code{ggtitle} of the \link[ggplot2]{ggplot}, (Default: NULL).}

\item{group_by}{A character variable to aggregate the stacked bars by group (Default: NULL).}
}
\value{
A \link[ggplot2]{ggplot2} object to be further modified
}
\description{
Creates a stacked barchart of features. It is possible to both show barcharts for each sample or group them by a categorical variable.
The function is compatible with the class \link{omics} method \code{composition()}.
}
\examples{
library("ggplot2")

# Create mock_data as data.frame (data.table is also supported)
mock_data <- data.frame(
  SAMPLE_ID = rep(paste0("Sample", 1:10), each = 5),
  Genus = rep(c("GenusA","GenusB","GenusC","GenusD","GenusE"), times = 10),
  value = c(
    0.1119, 0.1303, 0.0680, 0.5833, 0.1065,      # Sample1
    0.2080, 0.1179, 0.0211, 0.4578, 0.1951,      # Sample2
    0.4219, 0.1189, 0.2320, 0.1037, 0.1235,      # Sample3
    0.4026, 0.0898, 0.1703, 0.1063, 0.2309,      # Sample4
    0.1211, 0.0478, 0.5721, 0.1973, 0.0618,      # Sample5
    0.2355, 0.0293, 0.2304, 0.1520, 0.3528,      # Sample6
    0.2904, 0.0347, 0.3651, 0.0555, 0.2544,      # Sample7
    0.4138, 0.0299, 0.0223, 0.4996, 0.0345,      # Sample8
    0.4088, 0.0573, 0.0155, 0.2888, 0.2296,      # Sample9
    0.4941, 0.0722, 0.2331, 0.1023, 0.0983       # Sample10
  ),
  Group = rep(c("Group1","Group2","Group1",
                "Group1","Group2","Group2",
                 "Group1","Group1","Group1","Group2"),
               each = 5)
)

# Create a colormap
mock_palette <- c(
  GenusA = "#1f77b4",  # blue
  GenusB = "#ff7f0e",  # orange
  GenusC = "#2ca02c",  # green
  GenusD = "#d62728",  # red
  GenusE = "#9467bd"   # purple
)

# Optionally: Use OmicFlow::colormap()
mock_palette <- colormap(
  data = mock_data,
  col_name = "Genus",
  Brewer.palID = "RdYlBu"
)

composition_plot(
  data = mock_data,
  palette = mock_palette,
  feature_rank = "Genus",
  title_name = "Mock Genus Composition"
)

composition_plot(
  data = mock_data,
  palette = mock_palette,
  feature_rank = "Genus",
  title_name = "Mock Genus Composition by Group",
  group_by = "Group"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/volcano_plot.R
\name{volcano_plot}
\alias{volcano_plot}
\title{Volcano plot}
\usage{
volcano_plot(
  data,
  logfold_col,
  pvalue_col,
  feature_rank,
  abundance_col,
  pvalue.threshold = 0.05,
  logfold.threshold = 0.6,
  abundance.threshold = 0.01,
  label_A = "A",
  label_B = "B"
)
}
\arguments{
\item{data}{A \link[data.table]{data.table}.}

\item{logfold_col}{A column name of a continuous variable.}

\item{pvalue_col}{A column name of a continuous variable.}

\item{feature_rank}{A character variable of the feature column.}

\item{abundance_col}{A column name of a continuous variable.}

\item{pvalue.threshold}{A P-value threshold (default: 0.05).}

\item{logfold.threshold}{A Log2(A/B) Fold Change threshold (default: 0.6).}

\item{abundance.threshold}{An abundance threshold (default: 0.01).}

\item{label_A}{A character to describe condition A.}

\item{label_B}{A character to describe condition B.}
}
\value{
A \link[ggplot2]{ggplot2} object to be further modified.
}
\description{
Creates a Volcano plot from the output of \code{foldchange} method from class \link{omics}, it plots the foldchanges on the x-axis,
log10 trasnformed p-values on the y-axis and adjusts the scatter size based on the percentage abundance of the features.
This function is built into the class \link{omics} with method \code{DFE()} and inherited by other omics classes, such as;
\link{metagenomics} and \link{proteomics}.
}
\examples{
library(data.table)
library(ggplot2)

# Create mock data frame
mock_volcano_data <- data.table(

  # Feature names (feature_rank)
  Feature = paste0("Gene", 1:20),   

  # Log2 fold changes (X)        
  log2FC = c(1.2, -1.5, 0.3, -0.7, 2.3,    
             -2.0, 0.1, 0.5, -1.0, 1.8,
             -0.4, 0.7, -1.4, 1.5, 0.9,
             -2.1, 0.2, 1.0, -0.3, -1.8),
  
  # P-values (Y)
  pvalue = c(0.001, 0.02, 0.3, 0.04, 0.0005, 
             0.01, 0.7, 0.5, 0.02, 0.0008,
             0.15, 0.06, 0.01, 0.005, 0.3,
             0.02, 0.8, 0.04, 0.12, 0.03),
  
  # Mean (relative) abundance for point sizing
  rel_abun = runif(20, 0.01, 0.1)            
)

volcano_plot(
  data = mock_volcano_data,
  logfold_col = "log2FC",
  pvalue_col = "pvalue",
  abundance_col = "rel_abun",
  feature_rank = "Feature",
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{column_exists}
\alias{column_exists}
\title{Checks if column exists in table}
\usage{
column_exists(column, table)
}
\arguments{
\item{column}{A character of length 1.}

\item{table}{A \link[data.table]{data.table} or \link[base]{data.frame}.}
}
\value{
A boolean value.
}
\description{
Mainly used within \link{omics} and other functions to check if given column name does exist in the table and is not completely empty (containing NAs).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colormap.R
\name{colormap}
\alias{colormap}
\title{Color map of a variable}
\usage{
colormap(data, col_name, Brewer.palID = "Set2")
}
\arguments{
\item{data}{A \link[base]{data.frame} or \link[data.table]{data.table}.}

\item{col_name}{A column name of a categorical variable.}

\item{Brewer.palID}{A character name that exists in \link[RColorBrewer]{brewer.pal} (Default: \code{"Set2"}).}
}
\value{
A \link[stats]{setNames}.
}
\description{
Creates an object of hexcode colors with names given a vector of characters.
This function is built into the \code{ordination} method from the abstract class \link{omics} and inherited by other omics classes, such as;
\link{metagenomics} and \link{proteomics}.
}
\examples{
library("data.table")
dt <- data.table(
  "SAMPLE_ID" = c("sample_1", "sample_2", "sample_3"),
  "treatment" = c("healthy", "tumor", NA)
)

colors <- colormap(data = dt,
                   col_name = "treatment")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diversity.R
\name{diversity}
\alias{diversity}
\title{Sparse implementation of Alpha Diversity Metrics}
\usage{
diversity(
  x,
  metric = c("shannon", "simpson", "invsimpson"),
  normalize = TRUE,
  base = exp(1)
)
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.}

\item{metric}{A character variable for metric; shannon, simpson or invsimpson.}

\item{normalize}{A boolean variable for sample normalization by column sums.}

\item{base}{Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other (default: \code{exp(1)}).}
}
\value{
A numeric vector with type double.
}
\description{
Computes the alpha diversity based on Shannon index, simpson or invsimpson.
Code is adapted from \link[vegan]{diversity} and uses \link[Matrix]{sparseMatrix} in triplet format over the dense matrix.
The code is much faster and memory efficient, while still being mathematically correct.
This function is built into the class \link{omics} with method \code{alpha_diversity()} and inherited by other omics classes, such as;
\link{metagenomics} and \link{proteomics}.
}
\examples{
n_row <- 1000
n_col <- 100
density <- 0.2
num_entries <- n_row * n_col
num_nonzero <- round(num_entries * density)

set.seed(123)
positions <- sample(num_entries, num_nonzero, replace=FALSE)
row_idx <- ((positions - 1) \%\% n_row) + 1
col_idx <- ((positions - 1) \%/\% n_row) + 1

values <- runif(num_nonzero, min = 0, max = 1)
sparse_mat <- sparseMatrix(
  i = row_idx,
  j = col_idx,
  x = values,
  dims = c(n_row, n_col)
)

# Alpha diversity is computed on column level
## Transpose the sparseMatrix if required with t() from Matrix R package.
result <- OmicFlow::diversity(
  x = sparse_mat,
  metric = "shannon"
) 
}
\seealso{
\link[vegan]{diversity}
}

% Generated by roxygen2: do not edit by hand % Please edit documentation in R/proteomics-class.R \name{proteomics} \alias{proteomics} \title{Sub-class proteomics} \description{ This is a sub-class that is compatible to preprocessed data obtained from https://fragpipe.nesvilab.org/. It inherits all methods from the abstract class \link{omics} and only adapts the \code{initialize} function. It supports pre-existing data structures or paths to text files. When omics data is very large, data loading becomes very expensive. It is therefore recommended to use the \href{#method-reset}{\code{reset()}} method to reset your changes. Every omics class creates an internal memory efficient back-up of the data, the resetting of changes is an instant process. } \seealso{ \link{omics} } \section{Super class}{ \code{\link[OmicFlow:omics]{OmicFlow::omics}} -> \code{proteomics} } \section{Active bindings}{ \if{html}{\out{
}} \describe{ \item{\code{treeData}}{A "phylo" class, see \link[ape]{as.phylo}.} } \if{html}{\out{
}} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-proteomics-new}{\code{proteomics$new()}} \item \href{#method-proteomics-clone}{\code{proteomics$clone()}} } } \if{html}{\out{
Inherited methods

- `OmicFlow::omics$alpha_diversity()`
- `OmicFlow::omics$autoFlow()`
- `OmicFlow::omics$composition()`
- `OmicFlow::omics$copy()`
- `OmicFlow::omics$distance()`
- `OmicFlow::omics$feature_merge()`
- `OmicFlow::omics$feature_subset()`
- `OmicFlow::omics$foldchange()`
- `OmicFlow::omics$ordination()`
- `OmicFlow::omics$print()`
- `OmicFlow::omics$rankstat()`
- `OmicFlow::omics$removeNAs()`
- `OmicFlow::omics$reset()`
- `OmicFlow::omics$sample_subset()`
- `OmicFlow::omics$samplepair_subset()`
- `OmicFlow::omics$scale()`
- `OmicFlow::omics$validate()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-proteomics-new}{}}} \subsection{Method \code{new()}}{ Initializes the proteomics class object with \code{proteomics$new()} \subsection{Usage}{ \if{html}{\out{
}}\preformatted{proteomics$new( countData = NULL, metaData = NULL, featureData = NULL, treeData = NULL )}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{countData}}{A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.} \item{\code{metaData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.} \item{\code{featureData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.} \item{\code{treeData}}{A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.} } \if{html}{\out{
}} } \subsection{Returns}{ A new \code{proteomics} object. } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-proteomics-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{proteomics$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/omics-class.R
\name{omics}
\alias{omics}
\title{Abstract omics class}
\value{
A list of components:
\itemize{
\item \code{div} A \link[base]{data.frame} from \link{diversity}.
\item \code{stats} A pairwise statistics from \link[rstatix]{pairwise_wilcox_test}.
\item \code{plot} A \link[ggplot2]{ggplot} object.
}

A list of components:
\itemize{
\item \code{data} A \link[data.table]{data.table} of feature compositions.
\item \code{palette} A \link[stats]{setNames} palette from \link{colormap}.
}

A list of components:
\itemize{
\item \code{distmat} A distance dissimilarity in \link[base]{matrix} format.
\item \code{stats} A statistical test as a \link[base]{data.frame}.
\item \code{pcs} principal components as a \link[base]{data.frame}.
\item \code{scree_plot} A \link[ggplot2]{ggplot} object.
\item \code{anova_plot} A \link[ggplot2]{ggplot} object.
\item \code{scores_plot} A \link[ggplot2]{ggplot} object.
}

\itemize{
\item \code{data} A long \link[data.table]{data.table} table.
\item \code{volcano_plot} A \link[ggplot2]{ggplot} object.
\item \code{A} A \link[data.table]{data.table} table for (each) condition A
\item \code{B} A \link[data.table]{data.table} table for (each) condition B
}
}
\description{
This is the abstract class 'omics', contains a variety of methods that are inherited and applied in the omics classes:
\link{metagenomics} and \link{proteomics}.
}
\details{
Every class is created with the \link[R6]{R6Class} method. Methods are either public or private, and only the public components are inherited by other omic classes.
The omics class by default uses a \link[Matrix]{sparseMatrix} and \link[data.table]{data.table} data structures for quick and efficient data manipulation and returns the object by reference, same as the R6 class.
The method by reference is very efficient when dealing with big data.
}
\examples{

## ------------------------------------------------
## Method `omics$copy`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")

obj <- omics$new(
 metaData = metadata_file,
 countData = counts_file
)

# Perform a modification and copy
obj$scale()

cloned <- obj$copy(deep=TRUE)
cloned$scale(method = "clr")
cloned$reset() # resets to data after clone creation.


## ------------------------------------------------
## Method `omics$print`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")

obj <- omics$new(
 metaData = metadata_file,
 countData = counts_file
)

# method 1 to call print function
obj

# method 2 to call print function
obj$print()


## ------------------------------------------------
## Method `omics$reset`
## ------------------------------------------------

library(ggplot2)
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

taxa <- omics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file
)

# Performs modifications
taxa$scale(transform = log2)

# resets
taxa$reset()

# An inbuilt reset function prevents unwanted modification to the taxa object.
taxa$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))


## ------------------------------------------------
## Method `omics$removeNAs`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$removeNAs(column = "treatment")


## ------------------------------------------------
## Method `omics$feature_subset`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$feature_subset(Genus == "Pseudomonas")


## ------------------------------------------------
## Method `omics$sample_subset`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$sample_subset(treatment == "tumor")


## ------------------------------------------------
## Method `omics$feature_merge`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$feature_merge(feature_rank = c("Kingdom", "Phylum"))
obj$feature_merge(feature_rank = "Genus", feature_filter = c("uncultured", "metagenome"))


## ------------------------------------------------
## Method `omics$scale`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)
# standard relative abundance computation
obj$scale()

# CLR
obj$reset()
obj$scale(method = "clr")

# transform
obj$reset()
obj$scale(method = "none", transform = log2)


## ------------------------------------------------
## Method `omics$rankstat`
## ------------------------------------------------

library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

plt <- obj$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))
plt

## ------------------------------------------------
## Method `omics$alpha_diversity`
## ------------------------------------------------

library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

plt <- obj$alpha_diversity(col_name = "treatment",
                           metric = "shannon")


## ------------------------------------------------
## Method `omics$composition`
## ------------------------------------------------

library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

result <- obj$composition(feature_rank = "Genus",
                          feature_filter = c("uncultured"),
                          feature_top = 10)

plt <- composition_plot(data = result$data,
                        palette = result$palette,
                        feature_rank = "Genus")


## ------------------------------------------------
## Method `omics$distance`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
    metaData = metadata_file,
    countData = counts_file,
    featureData = features_file
)

obj$feature_subset(Kingdom == "Bacteria")
dist <- obj$distance(metric = "bray")

## ------------------------------------------------
## Method `omics$ordination`
## ------------------------------------------------

library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

pcoa_plots <- obj$ordination(metric = "bray",
                             method = "pcoa",
                             group_by = "treatment",
                             weighted = TRUE)
pcoa_plots


## ------------------------------------------------
## Method `omics$foldchange`
## ------------------------------------------------

library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- omics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file
)
obj$scale(method = "clr")

dfe <- obj$foldchange(feature_rank = "Genus",
                      paired = FALSE,
                      condition.group = "treatment",
                      condition_A = c("healthy"),
                      condition_B = c("tumor"))

}
\references{
Aitchison, J. (1986) The Statistical Analysis of Compositional Data. Chapman and Hall, London, 416 p.
}
\seealso{
\link{diversity_plot}

\link{composition_plot}

\link{bray}, \link{canberra}, \link{cosine}, \link{jaccard}, \link{jsd}, \link{manhattan}, \link{unifrac}

\link{ordination_plot}, \link{plot_pairwise_stats}, \link{pairwise_anosim}, \link{pairwise_adonis}

\link{volcano_plot}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{metaData}}{A \link[data.table]{data.table} with \code{SAMPLE_ID} column.}

\item{\code{featureData}}{A \link[data.table]{data.table} with \code{FEATURE_ID} column.}

\item{\code{countData}}{A dense or sparse \link[Matrix]{Matrix}.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-omics-new}{\code{omics$new()}}
\item \href{#method-omics-copy}{\code{omics$copy()}}
\item \href{#method-omics-validate}{\code{omics$validate()}}
\item \href{#method-omics-print}{\code{omics$print()}}
\item \href{#method-omics-reset}{\code{omics$reset()}}
\item \href{#method-omics-removeNAs}{\code{omics$removeNAs()}}
\item \href{#method-omics-feature_subset}{\code{omics$feature_subset()}}
\item \href{#method-omics-sample_subset}{\code{omics$sample_subset()}}
\item \href{#method-omics-samplepair_subset}{\code{omics$samplepair_subset()}}
\item \href{#method-omics-feature_merge}{\code{omics$feature_merge()}}
\item \href{#method-omics-scale}{\code{omics$scale()}}
\item \href{#method-omics-rankstat}{\code{omics$rankstat()}}
\item \href{#method-omics-alpha_diversity}{\code{omics$alpha_diversity()}}
\item \href{#method-omics-composition}{\code{omics$composition()}}
\item \href{#method-omics-distance}{\code{omics$distance()}}
\item \href{#method-omics-ordination}{\code{omics$ordination()}}
\item \href{#method-omics-foldchange}{\code{omics$foldchange()}}
\item \href{#method-omics-autoFlow}{\code{omics$autoFlow()}}
\item \href{#method-omics-clone}{\code{omics$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-new"></a>}}
\if{latex}{\out{\hypertarget{method-omics-new}{}}}
\subsection{Method \code{new()}}{
Wrapper function that is inherited and adapted for each omics class.
The omics classes requires a metadata samplesheet, that is validated by the metadata_schema.json.
It requires a column \code{SAMPLE_ID} and optionally a \code{SAMPLEPAIR_ID} can be supplied.
The \code{SAMPLE_ID} will be used to link the metaData to the countData, and will act as the key during subsetting of other columns.
To create a new object use \href{#method-new}{\code{new()}} method. Do notice that the abstract class only checks if the metadata is valid!
The \code{countData} and \code{featureData} will not be checked, these are handled by the sub-classes.
Using the omics class to load your data is not supported and still experimental.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$new(countData = NULL, featureData = NULL, metaData = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{countData}}{A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.}

\item{\code{featureData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.}

\item{\code{metaData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new \code{omics} object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-copy"></a>}}
\if{latex}{\out{\hypertarget{method-omics-copy}{}}}
\subsection{Method \code{copy()}}{
Create a copy of the object-class

This method is very similar to the existing \href{#method-clone}{\code{clone()}} function, except it also resets the back-up of the OmicFlow data types that is invoked with \href{#method-reset}{\code{reset()}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$copy(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{A boolean value to create a shallow or deep copy.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A copy of \code{omics} object
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")

obj <- omics$new(
 metaData = metadata_file,
 countData = counts_file
)

# Perform a modification and copy
obj$scale()

cloned <- obj$copy(deep=TRUE)
cloned$scale(method = "clr")
cloned$reset() # resets to data after clone creation.

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-validate"></a>}}
\if{latex}{\out{\hypertarget{method-omics-validate}{}}}
\subsection{Method \code{validate()}}{
Validates an input metadata against the JSON schema. The metadata should look as follows and should not contain any empty spaces.
For example; \code{'sample 1'} is not allowed, whereas \code{'sample1'} is allowed!

Acceptable column headers:
\itemize{
\item SAMPLE_ID (required)
\item SAMPLEPAIR_ID (optional)
\item CONTRAST_ (optional), used for \href{#method-autoFlow}{\code{autoFlow()}}.
\item VARIABLE_ (optional), not supported yet.
}

This function is used during the creation of a new object via \href{#method-new}{\code{new()}} to validate the supplied metadata
via a filepath or existing \link[data.table]{data.table} or \link[base]{data.frame}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$validate()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
None
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-print"></a>}}
\if{latex}{\out{\hypertarget{method-omics-print}{}}}
\subsection{Method \code{print()}}{
Displays parameters of the omics class via stdout.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$print()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")

obj <- omics$new(
 metaData = metadata_file,
 countData = counts_file
)

# method 1 to call print function
obj

# method 2 to call print function
obj$print()

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-reset"></a>}}
\if{latex}{\out{\hypertarget{method-omics-reset}{}}}
\subsection{Method \code{reset()}}{
Upon creation of a new \code{omics} object a small backup of the original data is created.
Since modification of the object is done by reference and duplicates are not made, it is possible to \code{reset} changes to the class.
The methods from the abstract class \link{omics} also contains a private method to prevent any changes to the original object when using methods such as \code{ordination} \code{alpha_diversity} or \code{foldchange}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$reset()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library(ggplot2)
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

taxa <- omics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file
)

# Performs modifications
taxa$scale(transform = log2)

# resets
taxa$reset()

# An inbuilt reset function prevents unwanted modification to the taxa object.
taxa$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-removeNAs"></a>}}
\if{latex}{\out{\hypertarget{method-omics-removeNAs}{}}}
\subsection{Method \code{removeNAs()}}{
Remove NAs from \code{metaData} and updates the \code{countData}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$removeNAs(column)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{column}}{The column from where NAs should be removed, this can be either a wholenumbers or characters. Vectors are also supported.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$removeNAs(column = "treatment")

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-feature_subset"></a>}}
\if{latex}{\out{\hypertarget{method-omics-feature_subset}{}}}
\subsection{Method \code{feature_subset()}}{
Feature subset (based on \code{featureData}), automatically applies data synchronization.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$feature_subset(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Expressions that return a logical value, and are defined in terms of the variables in \code{featureData}.
Only rows for which all conditions evaluate to TRUE are kept.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$feature_subset(Genus == "Pseudomonas")

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-sample_subset"></a>}}
\if{latex}{\out{\hypertarget{method-omics-sample_subset}{}}}
\subsection{Method \code{sample_subset()}}{
Sample subset (based on \code{metaData}), automatically applies synchronization.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$sample_subset(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Expressions that return a logical value, and are defined in terms of the variables in \code{metaData}.
Only rows for which all conditions evaluate to TRUE are kept.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$sample_subset(treatment == "tumor")

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-samplepair_subset"></a>}}
\if{latex}{\out{\hypertarget{method-omics-samplepair_subset}{}}}
\subsection{Method \code{samplepair_subset()}}{
Samplepair subset (based on \code{metaData}), automatically applies synchronization.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$samplepair_subset(num_unique_pairs = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{num_unique_pairs}}{An integer value to define the number of pairs to subset. The default is NULL,
meaning the maximum number of unique pairs will be used to subset the data.
Let's say you have three samples for each pair, then the \code{num_unique_pairs} will be set to 3.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object in place
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-feature_merge"></a>}}
\if{latex}{\out{\hypertarget{method-omics-feature_merge}{}}}
\subsection{Method \code{feature_merge()}}{
Agglomerates features by column, automatically applies synchronization.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$feature_merge(feature_rank, feature_filter = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{feature_rank}}{A character value or vector of columns to aggregate from the \code{featureData}.}

\item{\code{feature_filter}}{A character value or vector of characters to remove features via regex pattern.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

obj$feature_merge(feature_rank = c("Kingdom", "Phylum"))
obj$feature_merge(feature_rank = "Genus", feature_filter = c("uncultured", "metagenome"))

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-scale"></a>}}
\if{latex}{\out{\hypertarget{method-omics-scale}{}}}
\subsection{Method \code{scale()}}{
Feature scaling on the \code{countData}. The \code{scale} function is able to apply transformations element-wise on the positive values, (optional: add pseudocounts) and perform normalisation or standardisation methods.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$scale(
  method = "tss",
  transform = NULL,
  base = exp(1),
  pseudocount = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{method}}{A character to choose a standardisation/normalisation method, options: \code{tss}, \code{clr}, \code{binary}, \code{hellinger}, \code{none} (default: \code{"tss"}).}

\item{\code{transform}}{A function to apply on the positive values of \code{countData}, skip standardisation/normalisation with \code{method = "none"} (default: \code{NULL}).}

\item{\code{base}}{Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other (default: \code{exp(1)}) in CLR.}

\item{\code{pseudocount}}{A numeric value to replace zero's (default: \code{NULL}).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object in place
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)
# standard relative abundance computation
obj$scale()

# CLR
obj$reset()
obj$scale(method = "clr")

# transform
obj$reset()
obj$scale(method = "none", transform = log2)

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-rankstat"></a>}}
\if{latex}{\out{\hypertarget{method-omics-rankstat}{}}}
\subsection{Method \code{rankstat()}}{
Rank statistics based on \code{featureData}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$rankstat(feature_ranks, unique = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{feature_ranks}}{A vector of characters or integers that match the \code{featureData}.}

\item{\code{unique}}{A boolean value to display only unique entries in \code{feature_ranks}.}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
Counts the number of features identified for each column, for example in case of 16S metagenomics it would be the number of OTUs or ASVs on different taxonomy levels.
}

\subsection{Returns}{
A \link[ggplot2]{ggplot} object.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

plt <- obj$rankstat(feature_ranks = c("Kingdom", "Phylum", "Family", "Genus", "Species"))
plt
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-alpha_diversity"></a>}}
\if{latex}{\out{\hypertarget{method-omics-alpha_diversity}{}}}
\subsection{Method \code{alpha_diversity()}}{
Alpha diversity based on \link{diversity}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$alpha_diversity(
  col_name,
  metric = c("shannon", "invsimpson", "simpson"),
  Brewer.palID = "Set2",
  group_by = NULL,
  evenness = FALSE,
  paired = FALSE,
  p.adjust.method = "fdr"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{col_name}}{A character variable from the \code{metaData}.}

\item{\code{metric}}{An alpha diversity metric as input to \link{diversity}.}

\item{\code{Brewer.palID}}{A character name for the palette set to be applied, see \link[RColorBrewer]{brewer.pal} or \link{colormap}.}

\item{\code{group_by}}{A column name to perform grouped statistical test in \link{diversity_plot} (default: NULL).}

\item{\code{evenness}}{A boolean wether to divide diversity by number of species, see \link[vegan]{specnumber}.}

\item{\code{paired}}{A boolean value to perform paired analysis in \link[stats]{wilcox.test} and samplepair subsetting via \href{#method-samplepair_subset}{\code{samplepair_subset()}}}

\item{\code{p.adjust.method}}{A character variable to specify the p.adjust.method to be used, default is 'fdr'.}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

plt <- obj$alpha_diversity(col_name = "treatment",
                           metric = "shannon")

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-composition"></a>}}
\if{latex}{\out{\hypertarget{method-omics-composition}{}}}
\subsection{Method \code{composition()}}{
Creates a table most abundant compositional features. Also assigns a color blind friendly palette for visualizations.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$composition(
  feature_rank,
  feature_filter = NULL,
  col_name = NULL,
  feature_top = c(10, 15),
  Brewer.palID = "RdYlBu"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{feature_rank}}{A character variable in \code{featureData} to aggregate via \href{#method-feature_merge}{\code{feature_merge()}}.}

\item{\code{feature_filter}}{A character or vector of characters to removes features by regex pattern.}

\item{\code{col_name}}{Optional, a character or vector of characters to add to the final compositional data output.}

\item{\code{feature_top}}{A wholenumber of the top features to visualize, the max is 15, due to a limit of palettes.}

\item{\code{Brewer.palID}}{A character name for the palette set to be applied, see \link[RColorBrewer]{brewer.pal} or \link{colormap}.}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

result <- obj$composition(feature_rank = "Genus",
                          feature_filter = c("uncultured"),
                          feature_top = 10)

plt <- composition_plot(data = result$data,
                        palette = result$palette,
                        feature_rank = "Genus")

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-distance"></a>}}
\if{latex}{\out{\hypertarget{method-omics-distance}{}}}
\subsection{Method \code{distance()}}{
Compute a distance metric from \code{countData}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$distance(
  metric,
  weighted = TRUE,
  threads = 1,
  normalized = TRUE,
  base = exp(1)
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metric}}{A dissimilarity metric to be applied on the \code{countData},
thus far supports 'bray', 'jaccard', 'cosine', 'manhattan', 'aitchison', 'euclidean', 'jsd' (jensen-shannon divergence), 'canberra' and 'unifrac' when a tree is provided via \code{treeData}, see \href{#method-distance}{\code{distance()}}.}

\item{\code{weighted}}{A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).}

\item{\code{threads}}{A wholenumber, indicating the number of threads to use (Default: 1).}

\item{\code{normalized}}{A boolean value, whether to normalize weighted UniFrac distances to be between 0 and 1. Unweighted UniFrac is always normalized (default: TRUE).}

\item{\code{base}}{Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other (default: \code{exp(1)}).}

\item{\code{pseudocount}}{A numeric value to replace zero's, used in \href{#method-scale}{\code{scale()}} (default: \code{1e-15}).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A column x column \link[stats]{dist} object.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
    metaData = metadata_file,
    countData = counts_file,
    featureData = features_file
)

obj$feature_subset(Kingdom == "Bacteria")
dist <- obj$distance(metric = "bray")
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-ordination"></a>}}
\if{latex}{\out{\hypertarget{method-omics-ordination}{}}}
\subsection{Method \code{ordination()}}{
Ordination of \code{countData} with statistical testing.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$ordination(
  metric = "bray",
  method = c("pcoa", "nmds"),
  group_by,
  distmat = NULL,
  weighted = TRUE,
  threads = 1,
  perm_design = NULL,
  perm = 999
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metric}}{A dissimilarity or similarity metric to be applied on the \code{countData},
thus far supports 'bray', 'jaccard', 'cosine', 'manhattan', 'jsd' (jensen-shannon divergence), 'canberra' and 'unifrac' when a tree is provided via \code{treeData}, see \href{#method-distance}{\code{distance()}}.}

\item{\code{method}}{Ordination method, supports "pcoa" and "nmds", see \link[vegan]{wcmdscale}.}

\item{\code{group_by}}{A character variable in \code{metaData} to be used for the \link{pairwise_adonis} or \link{pairwise_anosim} statistical test.}

\item{\code{distmat}}{A custom distance matrix in either \link[stats]{dist} or \link[Matrix]{Matrix} format.}

\item{\code{weighted}}{A boolean value, whether to compute weighted or unweighted dissimilarities (default: \code{TRUE}).}

\item{\code{threads}}{A wholenumber, indicating the number of threads to use (Default: 1).}

\item{\code{perm_design}}{A function that takes \code{metaData} and constructs a permutation design with \link[permute]{how} (default: \code{NULL}).}

\item{\code{perm}}{A wholenumber, number of permutations to compare against the null hypothesis of \link[vegan]{adonis2} and \link[vegan]{anosim} (default: \code{perm=999}).}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
)

pcoa_plots <- obj$ordination(metric = "bray",
                             method = "pcoa",
                             group_by = "treatment",
                             weighted = TRUE)
pcoa_plots

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-foldchange"></a>}}
\if{latex}{\out{\hypertarget{method-omics-foldchange}{}}}
\subsection{Method \code{foldchange()}}{
Differential feature expression (DFE) on log-transformed values for both paired and non-paired test.

The function performs feature agglomeration, subsetting to remove NAs in \code{condition.group} and finding samplepairs. It expects that the data is already log-transformed, this can be accomplished via \href{#method-scale}{\code{scale()}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$foldchange(
  condition.group,
  condition_A,
  condition_B,
  group_by = NULL,
  feature_rank = "FEATURE_ID",
  feature_filter = NULL,
  paired = FALSE,
  pvalue.threshold = 0.05,
  logfold.threshold = 0.06,
  abundance.threshold = 0
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{condition.group}}{A character variable of an existing column name in \code{metaData}, wherein the conditions A and B are located.}

\item{\code{condition_A}}{A character value or vector of characters.}

\item{\code{condition_B}}{A character value or vector of characters.}

\item{\code{group_by}}{A character variable of an existing column in \code{metaData} to split the table in chunks prior to fold-change computation (default: \code{NULL}). When disabled then column names will end with \verb{_in_all}.}

\item{\code{feature_rank}}{A character or vector of characters in the \code{featureData} to aggregate via \href{#method-feature_merge}{\code{feature_merge()}} (default: \code{"FEATURE_ID"}).}

\item{\code{feature_filter}}{A character or vector of characters to remove features via regex pattern (default: \code{NULL}).}

\item{\code{paired}}{A boolean value, the paired is only applicable when a \code{SAMPLEPAIR_ID} column exists within the \code{metaData}. See \link[stats]{wilcox.test} and \href{#method-samplepair_subset}{\code{samplepair_subset()}}.}

\item{\code{pvalue.threshold}}{A numeric value used as a p-value threshold to label and color significant features (default: 0.05).}

\item{\code{logfold.threshold}}{A numeric value used as a fold-change threshold to label and color significantly expressed features (default: 0.06).}

\item{\code{abundance.threshold}}{A numeric value used as an abundance threshold to size the scatter dots based on their mean abundance (default: 0.01).}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- omics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file
)
obj$scale(method = "clr")

dfe <- obj$foldchange(feature_rank = "Genus",
                      paired = FALSE,
                      condition.group = "treatment",
                      condition_A = c("healthy"),
                      condition_B = c("tumor"))

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-autoFlow"></a>}}
\if{latex}{\out{\hypertarget{method-omics-autoFlow}{}}}
\subsection{Method \code{autoFlow()}}{
Automated Omics Analysis based on the \code{metaData}, see \href{#method-validate}{\code{validate()}}.
For now only works with headers that start with prefix \code{CONTRAST_}. If the data is from the class \code{omics} or \code{proteomics} than FDR adjusted p-values are computed for the volcano plots. Log-transformed values will lead to the skipping of \href{#method-composition}{\code{composition()}} and \href{#method-alpha_diversity}{\code{alpha_diversity()}} methods.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$autoFlow(
  feature_contrast = "FEATURE_ID",
  feature_filter = NULL,
  feature_ranks = NULL,
  distance_metrics = c("bray"),
  distmat = NULL,
  weighted = TRUE,
  pvalue.threshold = 0.05,
  logfold.threshold = 1,
  abundance.threshold = 0.01,
  perm = 999,
  threads = 1,
  report = TRUE,
  filename = paste0(getwd(), "/report.html")
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{feature_contrast}}{A character vector of feature columns in the \code{featureData} to aggregate via \href{#method-feature_merge}{\code{feature_merge()}} (default: \code{"FEATURE_ID"}).}

\item{\code{feature_filter}}{A character vector to filter unwanted features, (default: \code{NULL}).}

\item{\code{feature_ranks}}{A character vector as input to \href{#method-rankstat}{\code{rankstat()}} (default: \code{NULL}).}

\item{\code{distance_metrics}}{A character vector specifying what (dis)similarity metrics to use (default: \code{c("bray")}) When you are working with log-transformed data it is advised to use the \code{euclidean}.}

\item{\code{distmat}}{A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format (default: \code{NULL}).}

\item{\code{weighted}}{A boolean value, whether to compute weighted or unweighted dissimilarities (default: \code{TRUE}).}

\item{\code{pvalue.threshold}}{A numeric value, the p-value is used to include/exclude composition and foldchanges plots coming from alpha- and beta diversity analysis (default: 0.05).}

\item{\code{logfold.threshold}}{A numeric value used as a fold-change threshold to label and color significantly expressed features, see \href{#method-foldchange}{\code{foldchange()}} (Default: 1).}

\item{\code{abundance.threshold}}{A numeric value used as an abundance threshold to size the scatter dots based on their mean abundance, see \href{#method-foldchange}{\code{foldchange()}} (default: 0.01).}

\item{\code{perm}}{A wholenumber, number of permutations to compare against the null hypothesis of \link[vegan]{adonis2} or \link[vegan]{anosim} (default: 999).}

\item{\code{threads}}{Number of threads to use, only used in \href{#method-distance}{\code{distance()}} when distmat is not supplied (default: 1).}

\item{\code{report}}{A boolean value to create a HTML markdown report (default: \code{FALSE}). If \code{FALSE} a nested list of the plots and data is returned.}

\item{\code{filename}}{A character to name the HTML report to be saved in the current working directory (default: \code{paste0(getwd(), "/report.html")}). The \code{getwd()} is required for rmarkdown to save it in the right path.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
List of plots/data or rendered HTML report
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-omics-clone"></a>}}
\if{latex}{\out{\hypertarget{method-omics-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{omics$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hill_taxa.R
\name{hill_taxa}
\alias{hill_taxa}
\title{Sparse implementation of Hill numbers}
\usage{
hill_taxa(x, q = 0, normalize = TRUE, base = exp(1))
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.}

\item{q}{A wholenumber for 0, 1 or 2, default is 0.}

\item{normalize}{A boolean variable for sample normalization by column sums.}

\item{base}{Input for \link[base]{log} to use natural logarithmic scale, log2, log10 or other.}
}
\value{
A numeric vector with type double.
}
\description{
Computes the hill numbers for q is 0, 1 or 2.
Code is adapted from \link[hillR]{hill_taxa} and uses \link[Matrix]{sparseMatrix} in triplet format over the dense matrix.
The code is much faster and memory efficient, while still being mathematical correct.
}
\examples{
library("Matrix")

n_row <- 1000
n_col <- 100
density <- 0.2
num_entries <- n_row * n_col
num_nonzero <- round(num_entries * density)

set.seed(123)
positions <- sample(num_entries, num_nonzero, replace=FALSE)
row_idx <- ((positions - 1) \%\% n_row) + 1
col_idx <- ((positions - 1) \%/\% n_row) + 1

values <- runif(num_nonzero, min = 0, max = 1)
sparse_mat <- sparseMatrix(
  i = row_idx,
  j = col_idx,
  x = values,
  dims = c(n_row, n_col)
)

result <- OmicFlow::hill_taxa(
 x = sparse_mat,
 q = 2
)
}
\seealso{
\link[hillR]{hill_taxa}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jsd.R
\name{jsd}
\alias{jsd}
\title{Compute Jensen-Shannon Divergence from a Dense or Sparse Matrix.}
\usage{
jsd(x, weighted = TRUE, threads = 1)
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.}

\item{weighted}{A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).}

\item{threads}{A wholenumber, the number of threads to use in \link[RcppParallel]{setThreadOptions} (default: 1).}
}
\value{
A column x column \link[stats]{dist} object.
}
\description{
Calculates the Jensen-Shannon divergence of a Matrix pairwise for each column.
}
\details{
The Jensen-Shannon divergence between two probability distributions \eqn{A} and \eqn{B}, each of length \eqn{n}, is defined as:

\eqn{ d(A, B) = \frac{1}{2} D_{KL}(A \parallel M) + \frac{1}{2} D_{KL}(B \parallel M) }

where \eqn{M = \frac{1}{2} (A + B)} is the mixture distribution,
and \eqn{D_{KL}} is the Kullback-Leibler divergence.
When weighted is set to FALSE, counts are replaced by presence/absence data.
}
\examples{
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")

taxa <- metagenomics$new(
    metaData = metadata_file,
    countData = counts_file,
    featureData = features_file,
    treeData = tree_file
)

taxa$feature_subset(Kingdom == "Bacteria")
taxa$scale(method = "tss")

jsd(taxa$countData)
}
\references{
Lin, J. (1991). Divergence measures based on the Shannon entropy. IEEE Transactions on Information Theory, 37(1), 145-151.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{matrix_to_dtable}
\alias{matrix_to_dtable}
\title{Converting a Matrix to data.table}
\usage{
matrix_to_dtable(x)
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.}
}
\value{
A \link[data.table]{data.table} class.
}
\description{
Wrapper function that converts a sparseMatrix to data.table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OmicFlow-package.R
\docType{package}
\name{OmicFlow-package}
\alias{OmicFlow}
\alias{OmicFlow-package}
\title{OmicFlow: Fast and Efficient (Automated) Analysis of Sparse Omics Data}
\description{
A generalised data structure for fast and efficient loading and data munching of sparse omics data. The 'OmicFlow' requires an up-front validated metadata template from the user, which serves as a guide to connect all the pieces together by aligning them into a single object that is defined as an 'omics' class. Once this unified structure is established, users can perform manual subsetting, visualisation, and statistical analysis, or leverage the automated 'autoFlow' method to generate a comprehensive report.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/agusinac/OmicFlow}
  \item Report bugs at \url{https://github.com/agusinac/OmicFlow/issues}
}

}
\author{
\strong{Maintainer}: Alem Gusinac \email{alem.gusinac@gmail.com} (\href{https://orcid.org/0009-0006-1896-4176}{ORCID})

Authors:
\itemize{
  \item Thomas Ederveen (\href{https://orcid.org/0000-0003-0068-1275}{ORCID})
  \item Annemarie Boleij (\href{https://orcid.org/0000-0003-4495-5880}{ORCID}) [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairwise_adonis.R
\name{pairwise_adonis}
\alias{pairwise_adonis}
\title{Pairwise adonis2 (PERMANOVA) computation}
\usage{
pairwise_adonis(
  x,
  groups,
  metadata = NULL,
  perm_design = NULL,
  p.adjust.method = "bonferroni",
  perm = 999
)
}
\arguments{
\item{x}{A distance matrix in the form of \link[stats]{dist}.
Obtained from a dissimilarity metric, in the case of similarity metric please use \code{1-dist}}

\item{groups}{A character vector (column from a table) of labels.}

\item{metadata}{A data.table or data.frame of extra metadata for \code{perm_design} (default: NULL).}

\item{perm_design}{A function that takes a data.frame and constructs a permutation design with \link[permute]{how} (default: NULL).}

\item{p.adjust.method}{P adjust method see \link[stats]{p.adjust}.}

\item{perm}{Number of permutations to compare against the null hypothesis of adonis2 (default: \code{perm=999}).}
}
\value{
A \link[base]{data.frame} of
\itemize{
\item pairs that are used
\item Degrees of freedom (Df)
\item Sums of Squares of H_0
\item F.Model of H_0
\item R2 of H_0
\item p value of F^p > F
\item p adjusted
}
}
\description{
Computes pairwise \link[vegan]{adonis2}, given a distance matrix and a vector of labels.
This function is built into the class \link{omics} with method \code{ordination()} and inherited by other omics classes, such as;
\link{metagenomics} and \link{proteomics}.
}
\examples{
# Create random data
set.seed(42)
mock_data <- matrix(rnorm(15 * 10), nrow = 15, ncol = 10)

# Create euclidean dissimilarity matrix
mock_dist <- dist(mock_data, method = "euclidean")

# Define group labels, should be equal to number of columns and rows to dist
mock_groups <- rep(c("A", "B", "C"), each = 5)

# Compute pairwise adonis (PERMANOVA)
result <- pairwise_adonis(x = mock_dist, 
                          groups = mock_groups, 
                          p.adjust.method = "bonferroni", 
                          perm = 99)
}
\seealso{
\link[vegan]{adonis2}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manhattan.R
\name{manhattan}
\alias{manhattan}
\title{Compute Manhattan Dissimilarity from a Dense or Sparse Matrix.}
\usage{
manhattan(x, weighted = TRUE, threads = 1)
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix}.}

\item{weighted}{A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).}

\item{threads}{A wholenumber, the number of threads to use in \link[RcppParallel]{setThreadOptions} (default: 1).}
}
\value{
A column x column \link[stats]{dist} object.
}
\description{
Calculates the Manhattan dissimilarity of a Matrix pairwise for each column.
}
\details{
The Manhattan dissimilarity between two samples \eqn{A} and \eqn{B}, each of length \eqn{n}, is defined as:

\eqn{d(A, B) = \sum_{i}^n |A_i - B_i|}

where \eqn{A_i} and \eqn{B_i} are the abundances of the \eqn{i}-th feature in sample \eqn{A} and \eqn{B}, respectively.
When weighted is set to FALSE, counts are replaced by presence/absence data.
}
\examples{
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")

taxa <- metagenomics$new(
    metaData = metadata_file,
    countData = counts_file,
    featureData = features_file,
    treeData = tree_file
)

taxa$feature_subset(Kingdom == "Bacteria")
taxa$scale(method = "tss")

manhattan(taxa$countData)
}
\references{
Deza, M. M., & Deza, E. (2009). Encyclopedia of Distances. Springer Science & Business Media., 313.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metagenomics-class.R
\name{metagenomics}
\alias{metagenomics}
\title{Sub-class metagenomics}
\value{
\itemize{
\item \code{data} A long \link[data.table]{data.table} table.
\item \code{volcano_plot} A \link[ggplot2]{ggplot} object.
\item \code{A} A \link[data.table]{data.table} table for (each) condition A.
\item \code{B} A \link[data.table]{data.table} table for (each) condition B.
}
}
\description{
This is a sub-class that is compatible to data obtained from either 16S rRNA marker-gene sequencing or shot-gun metagenomics sequencing.
It inherits all methods from the abstract class \link{omics} and only adapts the \code{initialize} function.
It supports BIOM format data (v2.1.0 from \url{http://biom-format.org/}) in both HDF5 and JSON format, also pre-existing data structures can be used or text files.
When omics data is very large, data loading becomes very expensive. It is therefore recommended to use the \href{#method-reset}{\code{reset()}} method to reset your changes.
Every omics class creates an internal memory efficient back-up of the data, the resetting of changes is an instant process.
}
\examples{

## ------------------------------------------------
## Method `metagenomics$write_biom`
## ------------------------------------------------

library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")

taxa <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
 treeData = tree_file
)

taxa$write_biom(filename = "output.biom")
file.remove("output.biom")


## ------------------------------------------------
## Method `metagenomics$foldchange`
## ------------------------------------------------

library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file
)

dfe <- obj$foldchange(feature_rank = "Genus",
                      paired = FALSE,
                      condition.group = "treatment",
                      condition_A = c("healthy"),
                      condition_B = c("tumor"))

}
\seealso{
\link{omics}

\link{volcano_plot}
}
\section{Super class}{
\code{\link[OmicFlow:omics]{OmicFlow::omics}} -> \code{metagenomics}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{treeData}}{A "phylo" class, see \link[ape]{as.phylo}.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-metagenomics-new}{\code{metagenomics$new()}}
\item \href{#method-metagenomics-write_biom}{\code{metagenomics$write_biom()}}
\item \href{#method-metagenomics-foldchange}{\code{metagenomics$foldchange()}}
\item \href{#method-metagenomics-clone}{\code{metagenomics$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="alpha_diversity"><a href='../../OmicFlow/html/omics.html#method-omics-alpha_diversity'><code>OmicFlow::omics$alpha_diversity()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="autoFlow"><a href='../../OmicFlow/html/omics.html#method-omics-autoFlow'><code>OmicFlow::omics$autoFlow()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="composition"><a href='../../OmicFlow/html/omics.html#method-omics-composition'><code>OmicFlow::omics$composition()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="copy"><a href='../../OmicFlow/html/omics.html#method-omics-copy'><code>OmicFlow::omics$copy()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="distance"><a href='../../OmicFlow/html/omics.html#method-omics-distance'><code>OmicFlow::omics$distance()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="feature_merge"><a href='../../OmicFlow/html/omics.html#method-omics-feature_merge'><code>OmicFlow::omics$feature_merge()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="feature_subset"><a href='../../OmicFlow/html/omics.html#method-omics-feature_subset'><code>OmicFlow::omics$feature_subset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="ordination"><a href='../../OmicFlow/html/omics.html#method-omics-ordination'><code>OmicFlow::omics$ordination()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="print"><a href='../../OmicFlow/html/omics.html#method-omics-print'><code>OmicFlow::omics$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="rankstat"><a href='../../OmicFlow/html/omics.html#method-omics-rankstat'><code>OmicFlow::omics$rankstat()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="removeNAs"><a href='../../OmicFlow/html/omics.html#method-omics-removeNAs'><code>OmicFlow::omics$removeNAs()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="reset"><a href='../../OmicFlow/html/omics.html#method-omics-reset'><code>OmicFlow::omics$reset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="sample_subset"><a href='../../OmicFlow/html/omics.html#method-omics-sample_subset'><code>OmicFlow::omics$sample_subset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="samplepair_subset"><a href='../../OmicFlow/html/omics.html#method-omics-samplepair_subset'><code>OmicFlow::omics$samplepair_subset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="scale"><a href='../../OmicFlow/html/omics.html#method-omics-scale'><code>OmicFlow::omics$scale()</code></a></span></li>
<li><span class="pkg-link" data-pkg="OmicFlow" data-topic="omics" data-id="validate"><a href='../../OmicFlow/html/omics.html#method-omics-validate'><code>OmicFlow::omics$validate()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-metagenomics-new"></a>}}
\if{latex}{\out{\hypertarget{method-metagenomics-new}{}}}
\subsection{Method \code{new()}}{
Initializes the metagenomics class object with \code{metagenomics$new()}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{metagenomics$new(
  countData = NULL,
  metaData = NULL,
  featureData = NULL,
  treeData = NULL,
  biomData = NULL,
  feature_names = c("Kingdom", "Phylum", "Class", "Order", "Family", "Genus", "Species")
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{countData}}{A path to an existing file or a dense/sparse \link[Matrix]{Matrix} format.}

\item{\code{metaData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.}

\item{\code{featureData}}{A path to an existing file, \link[data.table]{data.table} or data.frame.}

\item{\code{treeData}}{A path to an existing newick file or class "phylo", see \link[ape]{read.tree}.}

\item{\code{biomData}}{A path to an existing biom file, version 2.1.0 (http://biom-format.org/), see \link[rhdf5]{h5read}.}

\item{\code{feature_names}}{A character vector to name the feature names that fit the supplied \code{featureData}.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new \code{metagenomics} object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-metagenomics-write_biom"></a>}}
\if{latex}{\out{\hypertarget{method-metagenomics-write_biom}{}}}
\subsection{Method \code{write_biom()}}{
Creates a BIOM file in HDF5 format of the loaded items via \href{#method-new}{'new()'}, which is compatible to the python biom-format version 2.1, see http://biom-format.org.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{metagenomics$write_biom(filename)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{filename}}{A character variable of either the full path of filename of the biom file (e.g. \code{output.biom})}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")

taxa <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file,
 treeData = tree_file
)

taxa$write_biom(filename = "output.biom")
file.remove("output.biom")

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-metagenomics-foldchange"></a>}}
\if{latex}{\out{\hypertarget{method-metagenomics-foldchange}{}}}
\subsection{Method \code{foldchange()}}{
Differential feature expression (DFE) Total Sample Sum (TSS) transformed values for both paired and non-paired test.

The function performs feature agglomeration, subsetting to remove NAs in \code{condition.group}, finding samplepairs and finally feature scaling prior to fold-change computation.
Based on the \code{transform} method, fold-changes will be computed either via subtraction or division.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{metagenomics$foldchange(
  condition.group,
  condition_A,
  condition_B,
  group_by = NULL,
  feature_rank = "FEATURE_ID",
  feature_filter = NULL,
  paired = FALSE,
  normalize = FALSE,
  pvalue.threshold = 0.05,
  logfold.threshold = 0.06,
  abundance.threshold = 0
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{condition.group}}{A character variable of an existing column name in \code{metaData}, wherein the conditions A and B are located.}

\item{\code{condition_A}}{A character value or vector of characters.}

\item{\code{condition_B}}{A character value or vector of characters.}

\item{\code{group_by}}{A character variable of an existing column in \code{metaData} to split the table in chunks prior to fold-change computation (default: NULL).}

\item{\code{feature_rank}}{A character or vector of characters in the \code{featureData} to aggregate via \href{#method-feature_merge}{\code{feature_merge()}} (default: \code{"FEATURE_ID"}).}

\item{\code{feature_filter}}{A character or vector of characters to remove features via regex pattern (default: \code{NULL}).}

\item{\code{paired}}{A boolean value, the paired is only applicable when a \code{SAMPLEPAIR_ID} column exists within the \code{metaData}. See \link[stats]{wilcox.test} and \href{#method-samplepair_subset}{\code{samplepair_subset()}}.}

\item{\code{normalize}}{A boolean value wether to normalize via Total Sample Sums (TSS) or not (default: \code{FALSE}).}

\item{\code{pvalue.threshold}}{A numeric value used as a p-value threshold to label and color significant features (default: 0.05).}

\item{\code{logfold.threshold}}{A numeric value used as a fold-change threshold to label and color significantly expressed features (default: 0.06).}

\item{\code{abundance.threshold}}{A numeric value used as an abundance threshold to size the scatter dots based on their mean abundance (default: 0.01).}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{library("ggplot2")
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")

obj <- metagenomics$new(
 metaData = metadata_file,
 countData = counts_file,
 featureData = features_file
)

dfe <- obj$foldchange(feature_rank = "Genus",
                      paired = FALSE,
                      condition.group = "treatment",
                      condition_A = c("healthy"),
                      condition_B = c("tumor"))

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-metagenomics-clone"></a>}}
\if{latex}{\out{\hypertarget{method-metagenomics-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{metagenomics$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unifrac.R
\name{unifrac}
\alias{unifrac}
\title{Compute UniFrac Dissimilarity from a Dense or Sparse Matrix.}
\usage{
unifrac(x, tree, weighted = TRUE, normalized = TRUE, threads = 1)
}
\arguments{
\item{x}{A \link[base]{matrix}, \link[Matrix]{sparseMatrix} or \link[Matrix]{Matrix} of strictly positive counts or presence/absence data.}

\item{tree}{A \code{phylo} class tree.}

\item{weighted}{A boolean value, to use abundances (\code{weighted = TRUE}) or absence/presence (\code{weighted=FALSE}) (default: TRUE).}

\item{normalized}{A boolean value, whether to normalize weighted UniFrac distances to be between 0 and 1 (default: TRUE). Unweighted UniFrac is always normalized.}

\item{threads}{A wholenumber, the number of threads to use in \link[RcppParallel]{setThreadOptions} (default: 1).}
}
\value{
A column x column \link[stats]{dist} object.
}
\description{
Calculates the UniFrac dissimilarity between samples based on phylogenetic branch lengths and abundance or presence/absence data.
}
\details{
The UniFrac distance between two samples \eqn{A} and \eqn{B}, with phylogenetic tree edges \eqn{i = 1 \ldots n} of lengths \eqn{L_i}, is computed differently depending on the \code{weighted} and \code{normalized} flags.
When \code{weighted = FALSE}, input counts are first converted to presence/absence data.
\describe{
\item{Weighted UniFrac (\code{normalized = FALSE} and \code{weighted = TRUE}):}{
\eqn{d(A,B) = \frac{\sum_{i}^n L_i |A_i - B_i|}{\sum_{i}^n L_i (A_i + B_i)}}
}
\item{Normalized Weighted UniFrac (\code{normalized = TRUE} and \code{weighted = TRUE}):}{
\eqn{d(A,B) = \sum_{i}^n L_i |A_i - B_i|}
}
\item{Unweighted UniFrac (\code{weighted = FALSE}, unweighted is always normalized):}{
\eqn{d(A,B) = \frac{\sum_{i}^n L_i |A_i - B_i|}{\sum_{i}^n L_i \max(A_i, B_i)}}
}
}
}
\examples{
library("OmicFlow")

metadata_file <- system.file("extdata", "metadata.tsv", package = "OmicFlow")
counts_file <- system.file("extdata", "counts.tsv", package = "OmicFlow")
features_file <- system.file("extdata", "features.tsv", package = "OmicFlow")
tree_file <- system.file("extdata", "tree.newick", package = "OmicFlow")

taxa <- metagenomics$new(
    metaData = metadata_file,
    countData = counts_file,
    featureData = features_file,
    treeData = tree_file
)

taxa$feature_subset(Kingdom == "Bacteria")
taxa$scale(method = "tss")

# Weighted UniFrac
unifrac(x = taxa$countData, tree = taxa$treeData, weighted=TRUE, normalized=FALSE)

# Weighted Normalized UniFrac
unifrac(x = taxa$countData, tree = taxa$treeData, weighted=TRUE, normalized=TRUE)

# Unweighted UniFrac
unifrac(x = taxa$countData, tree = taxa$treeData, weighted=FALSE)
}
\references{
Lozupone, C., & Knight, R. (2005). UniFrac: a new phylogenetic method for comparing microbial communities. Applied and Environmental Microbiology, 71(12), 8228–8235.
}
